c $Header: /usr/people/flittner/radtran/flatpsv/src/brdfsea.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
      subroutine brdfsea(tht,phi,tht0,phi0,refr,refi,V,PHEEV,p,
     & domega)
c
c calc the reflection matrix for the sea surface using distribution
c of frenel reflectors
c
c The coordinate system used is the same as that in the main rt code.
c The x axis points in the 0 deg azimuth direction or the direction that
c the rays of light from the sun are headed.  The z axis points in
c toward the center of the earth.  The y axis is defined by the cross-
c product of z and x (right handed coordinate system).  Therefore, the
c positive phi direction is measured in the clockwise direction when
c looking down from above.
c
c phi  =  output azimuth relative to solar plane (radians)
c phi0 =   input azimuth relative to solar plane (radians)
c
c last modified: April 16, 1997...def
c purpose: use domega to get the flux correct for the out direction
c          Also, don't calc. matrix if probv is lt 1e-20.
c
c last modified: June 20, 1997...def
c purpose: Change the definitions of the angles passed to SANGAA.
c
      implicit none
      include 'consts.cmn'
c
c passed
      real*8 tht,phi,tht0,phi0,refr,refi,V,PHEEV,p(4,4),domega
c local
      integer i,j
      complex*16 r1,r2
      real*8 amu0,amu,factor
      real*8 CALF0,C,S,C0,S0,XMNAA0,CPNMP,SPNMP
      real*8 PROBV,SIGMU2,SIGMC2
      logical debug
c
      debug=.FALSE.
c
c compute the angles for this problem so we can rotate the scattering
c matrix from the scattering plane to the vertical plane
c
      amu0=cos(tht0*dr)
      amu=abs(cos(tht*dr))
      CALL SANGAA(amu0,phi0,amu,phi,
     *            CALF0,C,S,C0,S0,XMNAA0,CPNMP,SPNMP)
c
c calculate the probability of a facet such that the incident direction
c is (tht0,phi0) and the reflected direction is (tht,phi).
c
      CALL SPROB (XMNAA0,CPNMP,SPNMP,V,PHEEV,phi,
     *            PROBV,SIGMU2,SIGMC2)
      if(probv.gt.1.0e-20)then
c
c compute frenel reflection matrix elements, cosine of incidence angle=calf0 
c
        call frenel(refr,refi,CALF0,r1,r2)
c 
c compute the reflection matrix transformed to the vertical plane
c first rotate angle -psip in clockwise direction and then (pi-psi) in CW
c
        call scat2vert(r1,r2,C,S,C0,S0,p)
c
c multiply by factor to represent the sea reflectance
c
        factor=PROBV*0.25d0/(amu*XMNAA0**4)/domega
        do i=1,4
           do j=1,4
              p(j,i)=factor*p(j,i)
           enddo
        enddo
        if(debug)then
          write(6,*)'brdfsea. tht tht0 phi phi0',tht,tht0,phi,phi0
          write(6,*)'brdfsea. factor',factor,probv
          write(6,10)((p(j,i),i=1,4),j=1,4)
10      format(4(1P4E12.3,/))
        endif
      else
        do i=1,4
           do j=1,4
              p(j,i)=0.0d0
           enddo
        enddo
      endif
      return
      end
c
c
      subroutine scat2vert(r1,r2,c,s,cs,ss,p)
c
c compute the scattering matrix transformed from the scattering plane to
c the vertical plane given the scattering functions for perpendicular
c and parallel polarizations, the cosine and sine of the rotation
c of the incident ray from vertical plane to scattering plane (angle psip),
c and the cosine and sine of the rotation of the scattered ray from the
c scattering plane to the vertical plane (angle psi).
c
c inputs
c
c r1     =  reflection coefficient for perpendicular polarization
c r2     =  reflection coefficient for parallel polarization
c c      =  cosine(psi)
c s      =  sine(psi)
c cs     =  cosine(psip)
c ss     =  sine(psip)
c 
c output
c p(4,4) =  scattering matrix with respect to the vertical plane
c
      implicit none
c input
      complex*16 r1,r2
      real*8 c,s,cs,ss
c output
      real*8 p(4,4)
c
c local
      integer n,i,j
      real*8 r(4,4),d(4,4),ctmp,stmp
c
      n=4
c
c make the scattering matrix from the 2 complex scattering functions,
c assuming that the off-diagional terms of the electric field scattering
c matrix are zero.  (See Van d'Hulst, "Light Scattering by Small
c Particles", page 45.)
c
      p(1,1)=dble(r2*conjg(r2))
      p(2,2)=dble(r1*conjg(r1))
      p(3,3)=0.5d0*dble(r1*conjg(r2)+r2*conjg(r1))
      p(4,4)=0.5d0*dble(r1*conjg(r2)+r2*conjg(r1))
      p(3,4)=-.5d0*dimag(r1*conjg(r2)-r2*conjg(r1))
      p(4,3)=-p(3,4)
      write(6,*)'scat2vert. ',(p(1,1)+p(2,2))*0.5
c
c all other elements are zero
c
      p(1,2)=0.0d0
      p(1,3)=0.0d0
      p(1,4)=0.0d0
      p(2,1)=0.0d0
      p(2,3)=0.0d0
      p(2,4)=0.0d0
      p(3,1)=0.0d0
      p(3,2)=0.0d0
      p(4,1)=0.0d0
      p(4,2)=0.0d0
c
c make the first rotation matrix for angle pi-psi in the CW direction.
c However, we have cosine(2psi)=c and sine(2psi)=s.
c Remember, cosine(pi-psi)=-cosine(psi) and sine(pi-psi)=sine(psi),
c and cosine(2*(pi-psi))=cos(2*psi) & sine(2*(pi-psi))=-sine(2*psi).
c So, change the sign of sine(2*psi).
c
      ctmp=c
      stmp=-s
      call mkrot(ctmp,stmp,r)
c
c multiply first rotation times the scattering matrix
c
      call matmul(r,n,n,p,n,d)
c
c the second rotation matrix for angle -psip in the CW direction
c However, we have cosine(2psip)=cs and sine(2psip)=ss.
c Remember, cosine(-2psip)=cosine(2psip), sine(-2psip)=-sine(2psip).
c So have adjusted stmp with minus sign.
c
      ctmp=cs
      stmp=-ss
      call mkrot(ctmp,stmp,r)
c
c multiply second rotation times the scattering matrix
c
      call matmul(d,n,n,r,n,p)
      return
      end
c***********************************************************************
c
      subroutine matmul(a,m,n,b,l,c)
c
c matrix multiple a(m by n) times b(n by l) and put result in c
c
      implicit none
      integer m,n,l
      real*8 a(m,n),b(n,l),c(m,l)
      integer i,j,k
      real*8 sum
c
      do i=1,m
         do j=1,l
            sum=0.0d0
            do k=1,n
               sum=sum+a(i,k)*b(k,j)
            enddo
            c(i,j)=sum
         enddo
      enddo
      return
      end
c
c
      subroutine mkrot(c,s,r)
c
c make the rotation matrix for clockwise rotation of psi, given the
c values of cosine(2psi)=c and sine(2psi)=s.
c
      implicit none
      real*8 c,s,r(4,4)
c
      r(1,1)=0.5d0*(1.0d0+c)		!cos(psi)**2
      r(1,2)=0.5d0*(1.0d0-c)		!sin(psi)**2
      r(1,3)=0.5d0*s			!0.5*sin(2psi)
      r(1,4)=0.0d0
      r(2,1)=0.5d0*(1.0d0-c)		!sin(psi)**2
      r(2,2)=0.5d0*(1.0d0+c)		!cos(psi)**2
      r(2,3)=-.5d0*s			!-0.5*sin(2psi)
      r(2,4)=0.0d0
      r(3,1)=-s				!-sin(2psi)
      r(3,2)=s				!sin(2psi)
      r(3,3)=c				!cos(2psi)
      r(3,4)=0.0d0
      r(4,1)=0.0d0
      r(4,2)=0.0d0
      r(4,3)=0.0d0
      r(4,4)=1.0d0
      return
      end
      subroutine frenel(refr,refi,costh,r1,r2)
c
c calc the frenel reflection (perpendicular and parallel to normal),
c r1 and r2,
c given the complex index of refraction of the medium and the cosine of the
c incidence angle.  It is assumed that the light is incident upon the
c surface from air.
c
c inputs
c
c costh  =  cosine(incidence angle)
c refr   =  real index of refraction
c refi   =  imaginary index of refraction
c
c outputs
c r1     =  reflection coefficient for perpendicular polarization
c r2     =  reflection coefficient for parallel polarization
c
      implicit none
      include 'consts.cmn'
c passed
      complex*16 r1,r2
      real*8 costh,refr,refi
c local
      complex*16 a,m
      real*8 costhsq
c
      m=cmplx(refr,refi)
      costhsq=costh*costh
      a=sqrt(m*m-1.0d0+costhsq)
      r1=(costh-a)/(costh+a)
      r2=(m*m*costh-a)/(m*m*costh+a)
      return
      end
C     SPROB          	                                                !   0001
      SUBROUTINE SPROB(XMNAA,CPNMP,SPNMP,V,PHEEV,PHEE,                  !   0003
     *                 PROBV,SIGMU2,SIGMC2)                             !   0004
C                                                                       !   0005
C     Input variables:                                                  !   0010
C                                                                       !   0011
C     XMNAA                                                             !   0012
C     CPNMP                                                             !   0013
C     SPNMP                                                             !   0014
C     V      - wind speed                                               !   0015
C     PHEEV                                                             !   0016
C     PHEE                                                              !   0017
C                                                                       !   0018
C     Output variables                                                  !   0019
C     PROBV                                                             !   0020
C        SIGMU2 - along wind direction
C        SIGMC2 - cross-wind direction                                  !   0029
C                                                                       !   0030
c last modified: April, 16, 1997...def
c purpose: Check that FPROB is non-negative.  If not, then
c          Set the PROBV to zero.
c
      implicit none
      real*8 XMNAA,CPNMP,SPNMP,V,PHEEV,PHEE,PROBV,SIGMU2,SIGMC2
C LOCAL
      real*8 C21,C03,C40,C22,C04,TTN,PMPV,AC,AU,FPROB,ARG,SPHEE
      real*8 SPHEEV,STETAN
c 
      SIGMU2 = 0.000 + 0.00316*V                                        !   0031
      SIGMC2 = 0.003 + 0.00192*V                                        !   0032
      C21    = 0.01  - 0.00860*V                                        !   0033
      C03    = 0.04  - 0.03300*V                                        !   0034
      C40    = 0.40                                                     !   0035
      C22    = 0.12                                                     !   0036
      C04    = 0.23                                                     !   0037
C                                                                       !   0040
      TTN    = SQRT(1.0-XMNAA**2)/XMNAA                                 !   0041
      PMPV   = PHEE - PHEEV                                             !   0042
      AC     = TTN*(SPNMP*COS(PMPV) + CPNMP*SIN(PMPV))/SQRT(SIGMC2)     !   0043
      AU     =+TTN*(CPNMP*COS(PMPV) - SPNMP*SIN(PMPV))/SQRT(SIGMU2)     !   0044
C                                                                       !   0045
C     3.  FPROB 			                                !   0046
C                                                                       !   0047
      FPROB = 1.0 - 0.5*C21*(AC**2-1.0)*AU - 0.167*C03*(AU**3-3.0*AU)   !   0048
     *      + 0.0417*C40*(AC**4-6.0*AC**2+3.0) + 0.25*C22*(AC**2-1.0)   !   0049
     *      *(AU**2 - 1.0) + 0.0417*C04*(AU**4-6.0*AU**2+3.0)           !   0050
C                                                                       !   0051
c check to see that fprob is greater than zero				!def
      if(FPROB.gt.0.0d0)then						!def
      PROBV=0.0                                                         !   0054
      ARG   = 0.5*(AC**2+AU**2)                                         !   0055
      IF(ARG.LE.30) PROBV=0.159155/SQRT(SIGMC2*SIGMU2)*EXP(-ARG)*FPROB  !   0056
      else								!def
        FPROB=0.0							!def
        PROBV=0.0							!def
      endif								!def
C                                                                       !   0057
      SPHEE = PHEE /3.1415926*180                                       !   0058
      SPHEEV= PHEEV/3.1415926*180                                       !   0059
      STETAN= ACOS(XMNAA)/3.1415926*180                                 !   0060
C     PRINT 1, V,SPHEEV,STETAN,SPHEE,CPNMP,SPNMP,FPROB,PROBV            !   0061
  1   FORMAT(1X,'V=',F4.1,1X,'FV=',E9.2,1X,'TETAN=',E10.3,1X,           !   0062
     * 'FI=',F6.1,1X,'CPNMP=',E10.3,1X,'SPNMP=',E10.3,1X,'FPROB=',      !   0063
     * E9.2,1X,'PROBV=',E10.3)                                          !   0064
      RETURN                                                            !   0071
      END                                                               !   0072
      real*8 FUNCTION COSKR(X)                                          !   0001
      implicit none
      real*8 x
      COSKR = COS(X)                                                    !   0002
      IF(COSKR.GE.  0.999900)  COSKR=  1.0d0                            !   0003
      IF(COSKR.LE.(-0.999900)) COSKR= -1.0d0                            !   0004
      RETURN                                                            !   0006
      END                                                               !   0007
      real*8 FUNCTION SINKR(X)                                          !   0001
      implicit none
      real*8 x
      SINKR = SIN(X)                                                    !   0003
      IF(ABS(SINKR).LE.0.00010) SINKR=0.0d0                             !   0004
      RETURN                                                            !   0005
      END                                                               !   0006
C     SANGAA    	                                                !   0001
      SUBROUTINE SANGAA(XMYU,PHEE,XMYUS,PHEES,                          !   0011
     *                  CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP)               !   0012
C                                                                       !   0013
c     purpose: 
c     given the direction of the incident beam (XMYU,PHEE) and the
c     direction of the reflected beam (XMYUS,PHEES)
c     figures out the angles of the "mirror" wave facet and
c     calculates all the angles required for complete description of
c     the Stokes parameters reflection from this
c     (generally non-horizontali)  wave facet (rotational matrices )
c
c     special case:  nadir incience and/or zenith reflectance
c
C     input variables:                                                  !   0014
C                                                                       !   0015
C     XMYUS - cosine of the vertical angle of the incident beam         !   0016
C     PHEES - azimuth of the incident beam                              !   0018
C     XMYU  - cosine of the vertical angle of the reflected beam        !   0019
C     PHEE  - azimuth of the reflectance angle                          !   0021
C                                                                       !   0022
C     output variables:  (see chart 2 of the documentation)             !   0023
C                                                                       !   0024
C     CALF  -   COS(ALPHA) - cosine of the local reflectance angle      !   0025
C     C     -   COS(2*PSI)                                              !   0026
C     S     -   SIN(2*PSI)                                              !   0027
C     CS    -   COS(2*PSI')                                             !   0028
C     SS    -   SIN(2*PSI')                                             !   0029
C     XMNAA -   COS(TETAN)                                              !   0030
C     CPNMP -   COS(PHEEN-PHEE)                                         !   0031
C     SPNMP -   SIN(PHEEN-PHEE)                                         !   0032
C                                                                       !   0033
C     PRINT 1, XMYU,PHEE,XMYUS,PHEES                                    !   0034
C    *     ,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP                            !   0035
C
C last modified: June 20, 1997...def
C purpose: Correct the definition of XMYU,phee,xmyus,phees (switch them).
C                                                                       !   0036
      implicit none
c passed
      real*8 XMYU,PHEE,XMYUS,PHEES,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP
c local
      real*8 C2ALF,S2ALF,SALF,CPSIS,SPSIS,CPSI,SPSI,XSNAA
      real*8 SINKR,COSKR
      external SINKR,COSKR
c
   1  FORMAT(2X,'XMYU=',E15.8,2X,'PHEE=',E15.8,2X,                      !   0037
     *          'XMYUS=',E15.8,2X,'PHEES=',E15.8,2X/                    !   0038
     *       2X,'CALF=',E15.8,2X,'C=',E15.8,2X,'S=',E15.8,2X,           !   0039
     *          'CS=',E15.8,2X,'SS=',E15.8,2X/                          !   0040
     *       2X,'XMNAA=',E15.8,2X,'CPNMP=',E15.8,2X,'SPNMP=',E15.8/)    !   0041
C                                                                       !   0042
   2  FORMAT(2X,'C2ALF=',E15.8,2X,'S2ALF=',E15.8,2X,                    !   0043
     *          'CALF=',E15.8,2X,'SALF=',E15.8//)                       !   0044
   7  FORMAT(10X,'...') 		                                !   0045
C                                                                       !   0046
C====================================================================== !   0049
C                                                                       !   0052
      C2ALF = XMYU*XMYUS - SQRT(1.-XMYU**2)*SQRT(1.-XMYUS**2)*          !   0053
     *                     COSKR(PHEES-PHEE)                            !   0054
C====================================================================== !   0055
C                                                                       !   0056
C                                                                       !   0057
      IF(C2ALF.GE.0.999990) GO TO 11                                    !   0058
C                                                                       !   0059
  6   FORMAT(10X,' A=/=B')       		                        !   0060
C     PRINT 6                                                           !   0061
      S2ALF = SQRT(1. - C2ALF**2)                                       !   0062
      CALF  = SQRT( 0.5*(1.0+C2ALF))                                    !   0063
      SALF  = SQRT( 0.5*(1.0-C2ALF))                                    !   0064
C                                                                       !   0065
      CPSIS  = ( XMYU*SQRT(1.-XMYUS**2) + XMYUS*SQRT(1.-XMYU**2)*       !   0066
     *           COSKR(PHEE - PHEES) )/S2ALF                            !   0067
      SPSIS  = SQRT(1.-XMYU**2)*SINKR(PHEES-PHEE)/S2ALF                 !   0068
      CS     = 2.*CPSIS**2 - 1.0                                        !   0069
      SS     = 2.*CPSIS*SPSIS                                           !   0070
C                                                                       !   0071
      CPSI   = ( XMYUS*SQRT(1.-XMYU**2) + XMYU*SQRT(1.-XMYUS**2)*       !   0072
     *           COSKR(PHEE - PHEES) )/S2ALF                            !   0073
      SPSI   = SQRT(1.-XMYUS**2)*SINKR(PHEE-PHEES)/S2ALF                !   0074
      C      = 2.*CPSI**2 - 1.0                                         !   0075
      S      = 2.*CPSI*SPSI                                             !   0076
C                                                                       !   0077
      XMNAA  = XMYU*CALF + SQRT(1.-XMYU**2)*SALF*CPSI                   !   0078
C                                                                       !   0079
      IF(XMNAA.GE.0.99999) GO TO 13                                     !   0080
C                                                                       !   0081
      XSNAA  = SQRT(1.- XMNAA**2)                                       !   0082
      CPNMP  = ( CALF*SQRT(1.-XMYU**2) - SALF*XMYU*CPSI )/XSNAA         !   0083
      SPNMP  = SALF*SPSI/XSNAA                                          !   0084
C     PRINT 1, XMYU,PHEE,XMYUS,PHEES,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP   !   0085
      GO TO 14                                                          !   0086
C                                                                       !   0087
  13  CONTINUE                                                          !   0088
      XMNAA = 1.0                                                       !   0089
      CPNMP = 1.0                                                       !   0090
      SPNMP = 0.0                                                       !   0091
C     PRINT 1, XMYU,PHEE,XMYUS,PHEES,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP   !   0092
      GO TO 14                                                          !   0093
  11  CONTINUE                                                          !   0094
      C2ALF=1.0                                                         !   0095
      S2ALF=0.0                                                         !   0096
      CALF =1.0                                                         !   0097
      SALF =0.0                                                         !   0098
C                                                                       !   0099
      IF(XMYU.GE.0.99999) GO TO 12                                      !   0100
C                                                                       !   0101
  5   FORMAT(10X,': A=B=/=C')       	                                !   0102
C     PRINT 5                                                           !   0103
C     PRINT 2, C2ALF,S2ALF,CALF,SALF                                    !   0104
C                                                                       !   0105
      C     = 1.0                                                       !   0106
      S     = 0.0                                                       !   0107
      CS    = 1.0                                                       !   0108
      SS    = 0.0                                                       !   0109
      XMNAA = XMYU                                                      !   0110
      CPNMP = 1.0                                                       !   0111
      SPNMP = 0.0                                                       !   0112
C                                                                       !   0113
C     PRINT 1, XMYU,PHEE,XMYUS,PHEES,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP   !   0114
C                                                                       !   0115
      GO TO 14                                                          !   0116
C                                                                       !   0117
  12  CONTINUE                                                          !   0118
  4   FORMAT(10X,': A=B=C')       	                                !   0119
C     PRINT 4                                                           !   0120
C                                                                       !   0121
      C     = COSKR(2.*PHEE )                                           !   0122
      S     = SINKR(2.*PHEE )                                           !   0123
      CS    = COSKR(2.*PHEES)                                           !   0124
      SS    = SINKR(2.*PHEES)                                           !   0125
      XMNAA = 1.0                                                       !   0126
      CPNMP = 1.0                                                       !   0127
      SPNMP = 0.0                                                       !   0128
C     PRINT 1, XMYU,PHEE,XMYUS,PHEES,CALF,C,S,CS,SS,XMNAA,CPNMP,SPNMP   !   0130
      GO TO 14                                                          !   0132
  14  CONTINUE                                                          !   0134
      RETURN                                                            !   0135
      END                                                               !   0136
