c $Header: /usr/people/flittner/radtran/flatpsv/src/cmppa.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
************************************************************************
      subroutine cmppa(nz,ipseudo,cossun,dt,z,pa)
************************************************************************
C This subroutine calculates the solar attenuation path length to 
C a point on the zenith in either a flat atmosphere or a spherical one.
C
      implicit none
      include 'param.inc'
      include 'consts.cmn'
      include 'cstdatm.cmn'
c varibles that are passed as inputs:
      integer nz,ipseudo
      real*8 dt(maxnz-1),cossun,z(maxnz)
c varibles that are passed as outputs: 
      real*8 pa(maxnz)
c varibles that are local
      integer nlev,i,ians,idx,idxzc,idxznlev
      real*8 oum,sumtau,d1,d2,pc,rpz,sinsun,d2b,dist1,dist2
      real*8 zc,pcpc
      logical debug,addtop,prtf
      integer fistd
      real*8 topchap
      external fistd,topchap
c
      debug=.FALSE.
      addtop=.TRUE.
      prtf=.FALSE.
c
      if(ipseudo.eq.0)then
c compute using flat atmo
        if(cossun.le.0.0d0)then
          write(6,*)'Cannot have a solar zenith angle ge 90 ',
     &'in flat atmo'
          stop
        endif
        oum=1.0d0/(cossun)
        pa(1)=0.0d0
        sumtau=0.0d0
        do nlev=2,nz
           sumtau=sumtau+dt(nlev-1)
           pa(nlev)=sumtau*oum
        enddo
      else
c compute using pseudo spherical approx
c make the dttotaldz array
        d1=tauray(nstd)+tauo3(nstd)+tauf(nstd)+tauh2o(nstd)+tauc(nstd)
        do i=nstd-1,1,-1
           d2=tauray(i)+tauo3(i)+tauf(i)+tauh2o(i)+tauc(i)
           dttotaldz(i)=(d2-d1)/dzstd(i)
           d1=d2
        enddo
c check for simple case of sza less than or equal to 90
        if(cossun.ge.0.0d0)then
          sinsun=max(0.0d0,(1.0d0-cossun**2))
          sinsun=dsqrt(sinsun)
          pa(1)=0.0d0
c now compute pa for each level from level 2 to nz
          do nlev=2,nz
             d1=cossun*(z(nlev)+re)
             pc=sinsun*(z(nlev)+re)
             pcpc=pc*pc
c find index of zstd that is just less than z(nlev)
             idx=fistd(zstd,nstd,z(nlev))
             if(debug)
     &write(6,*)'cmppa. znlev',nlev,z(nlev),zstd(idx),idx
	     sumtau=0.0d0
             if(idx.lt.nstd)then
               d2=((zstd(idx+1)+re)**2-pcpc)
               if(d2.le.0.0d0)then
                 d2=0.0d0
               else
                 d2=sqrt(d2)
               endif
               sumtau=(d2-d1)*dttotaldz(idx)
               if(debug)
     &write(6,*)'cmppa. 1',zstd(idx+1),sumtau,d2-d1
c now work our way back to the layer with top altitude zstd(nstd)
               d1=d2
               do i=idx+1,nstd-1,1
                  d2=((zstd(i+1)+re)**2-pcpc)
                  if(d2.le.0.0d0)then
                    d2=0.0d0
                  else
                    d2=sqrt(d2)
                  endif
                  sumtau=sumtau+(d2-d1)*dttotaldz(i)
                  d1=d2
               enddo
             endif
             if(addtop)then
c use chapman function for the top layer
               rpz=re+zstd(nstd)
c rayleigh
               if(Hp.gt.0.0d0)then
                  sumtau=sumtau+topchap(pc,rpz,Hp)/Hp*tauray(nstd)
               endif
c ozone
               if((tauo3(nstd).gt.0.0d0).and.(Ho3.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Ho3)/Ho3*tauo3(nstd)
               endif
c aerosol
               if((tauf(nstd).gt.0.0d0).and.(Ho3.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Hf)/Hf*tauf(nstd)
               endif
c water vapor
               if((tauh2o(nstd).gt.0.0d0).and.(Hh2o.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Hh2o)/Hh2o*tauh2o(nstd)
               endif
c assume that no clouds above zstd(nstd)
             endif
             pa(nlev)=sumtau
          enddo          
        else
c solar beam is tangent in atmosphere
          pa(1)=0.0d0
          do nlev=2,nz
c find the solar tangent height
             pc=sinsun*(z(nlev)+re)
             pcpc=pc*pc
             zc=pc-re
c is the point z(nlev) in the shadow of th earth?
             if(zc.lt.z(nz))then
               pa(nlev)=300.0d0
             else
c find index of zstd that is just less than zc
               idxzc=fistd(zstd,nstd,zc)
c find index of zstd that is just less than z(nlev)
               idxznlev=fistd(zstd,nstd,z(nlev))
c is the solar tangent layer, idxzc, equal to this layer, idxznlev?
               if(idxzc.eq.idxznlev)then
                 d1=0.0d0
               else
                 d1=((zstd(idxznlev)+re)**2-pcpc)
                 if(d1.le.0.0d0)then
                   d1=0.0d0
                 else
                   d1=sqrt(d1)
                 endif
               endif
c do portion in layer from z(nlev) to z(idxnlev)
               d2=((z(nlev)+re)**2-pcpc)
               if(d2.le.0.0d0)then
                 d2=0.0d0
               else
                 d2=sqrt(d2)
               endif
               dist1=d2-d1
c portion in layer idxznlev, but on other side of tangent point
               d2b=((zstd(idxznlev+1)+re)**2-pcpc)
               if(d2b.le.0.0d0)then
                 d2b=0.0d0
               else
                 d2b=sqrt(d2b)
               endif
               dist2=d2b-d1
               sumtau=(dist1+dist2)*dttotaldz(idxznlev)
c double path layers
               d2=d1
               do i=idxznlev+1,idxzc-1
                  d1=((zstd(i)+re)**2-pcpc)
                  if(d1.le.0.0d0)then
                    d1=0.0d0
                  else
                    d1=sqrt(d1)
                  endif
                  sumtau=sumtau+2.0d0*(d2-d1)*dttotaldz(i)
                  d2=d1
               enddo
c double path for the tangent layer
               if(idxzc.ne.idxznlev)then
                 sumtau=sumtau+2.0d0*d2*dttotaldz(idxzc)
               endif
c layers above layer containing z(nlev)
               d1=d2b
               do i=idxznlev+1,nstd-1,1
                  d2=((zstd(i)+re)**2-pcpc)
                  if(d2.le.0.0d0)then
                    d2=0.0d0
                  else
                    d2=sqrt(d2)
                  endif
                  sumtau=sumtau+(d2-d1)*dttotaldz(i)
                  d1=d2
               enddo
c use chapman function for the top layer
               rpz=re+zstd(nstd)
c rayleigh
               if(Hp.gt.0.0d0)
     &sumtau=sumtau+topchap(pc,rpz,Hp)/Hp*tauray(nstd)
c ozone
               if((tauo3(nstd).gt.0.0d0).and.(Ho3.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Ho3)/Ho3*tauo3(nstd)
               endif
c aerosol
               if((tauf(nstd).gt.0.0d0).and.(Ho3.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Hf)/Hf*tauf(nstd)
               endif
c water vapor
               if((tauh2o(nstd).gt.0.0d0).and.(Hh2o.gt.0.0d0))then
                 sumtau=sumtau+topchap(pc,rpz,Hh2o)/Hh2o*tauh2o(nstd)
               endif
c assume that no clouds above zstd(nstd)
               pa(nlev)=sumtau
             endif
          enddo
c now for the top of the atmosphere. Check to see if the point just below
c the top of the atmo (z(2)) is in the shadow.  If so, then will set pa(1)
c equal to pa(2).  Otherwise, pa(1)=0.0d0
          if(pa(2).eq.300.0d0)pa(1)=pa(2)
        endif
      endif
      if(debug)then
        write(6,*)'cmppa. nlev=',nlev,' pa=',pa(nlev)
        write(6,*)'continue? (0=yes;1=no)'
        read(5,*)ians
        if(ians.eq.1)stop
      endif
      if(prtf)then
	write(6,*)'cmppa. will open pa.out '
        open(14,file='pa.out',status='unknown')
        do i=1,nz
           write(14,*)z(i),pa(i)
        enddo
        close(14)
      endif
      return
      end
c
c***********************************************************************
c
      integer function fistd(z,mttf,zp) 
c
c find the index of the array z that is closest to zp, but less than zp
c
      integer mttf
      real*8 z(mttf),zp
      integer n
c
      if(zp.le.z(1))then
        n=1
      else
        n=mttf
        do while((zp.lt.z(n)).and.(n.gt.1))
           n=n-1
        enddo
      endif
      fistd=n
      return
      end
c***********************************************************************
