c $Header: /usr/people/flittner/radtran/flatpsv/src/downit.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
c ---------------------------------------------------------------------

      subroutine downit(itest,ndownlos,esi0,esing0,pal,dmu,wa,wr,
     &fmatrx,rmatrx,trans,nz,nlos,np,convrg,wc,cmatrx,itlmax)

C Performs the downward pass of the Gauss-Seidel iteration scheme
C itest - Used to check for convergence, if intensity change of more 
C         than convrg occurs between the current and last iteration
C         another iteration is performed.
C nlos2  - The index of the line of sight angle closest to but less than
C         90 degrees.
C The level is selected by the do 50 loop:
C For level=2, layer=1 and this is the first layer. The incident intensities
C          are zero and the total scatter is simply that which occurs
C          in the first layer.
C For all other layers there will be a boundary term and a term for 
C          scattering within the layer. In a flat atmosphere model
C          this boundary intensity is simply the intensity from the
C          level directly above in the same direction.
C
C Set the l.o.s such that theta=0,90(ilos=1,nlos2) for downward intensities
C This theta is the output direction for the phase function calculations 
C in the integration of the source function.
C
C jlos is used for the output phi direction
C
C Subroutine source performs the source term integration.

      implicit none
      include 'consts.cmn'
      include 'param.inc'
c variables that are passed
      integer itest,ndownlos,nz,nlos,np,itlmax
      real*8 esi0(4,nfi,mlos,maxnz),esing0(4,nfi,mlos,maxnz),
     &pal(mlos,maxnz), dmu(mlos), trans(mlos,maxnz),wr(maxnlayr),
     &wa(maxnlayr),wc(maxnlayr),
     &convrg,fmatrx(4,4,nfi,mlos,mlos),rmatrx(4,4,nfi,mlos,mlos),
     &cmatrx(4,4,nfi,mlos,mlos)
c variables that are local
      integer level,layer,labove,ilos,jlos,m,itm,it
      real*8 bndry(4),enew(4),prcntdif,delt,c1,cf1,cf2,T,wal,wrl,wcl
	logical debug
c
	debug=.false.
      do 50 level=2,nz
         layer=level-1
         labove=level-1
         wal=wa(layer)
         wrl=wr(layer)
         wcl=wc(layer)
         if(wcl.gt.0.5)then
           itm=itlmax
         else
           itm=1
         endif
         do it=1,itm
            do 40 ilos=1,ndownlos 
               delt=pal(ilos,level)
               c1=1.0d0-trans(ilos,level)
               cf2=(1.0d0-c1/delt)
               cf1=(c1-cf2)
               T=trans(ilos,level)
               do 30 jlos=1,nfi
                  call source(level,layer,labove,ilos,jlos,esi0,
     &dmu,wal,wrl,wcl,cf1,cf2,fmatrx,rmatrx,cmatrx,nlos,np,enew,
     &debug)
c load the boundary
                  if(layer.gt.1)then
                    do m=1,np
                       bndry(m)=esi0(m,jlos,ilos,layer)-
     &esing0(m,jlos,ilos,layer)
                    enddo
                  else
                    do m=1,np
                       bndry(m)=0.0d0
                    enddo
                  endif
c m=1
                  enew(1)=enew(1)+bndry(1)*T+esing0(1,jlos,ilos,level)
                  if(itest.ne.100.and.enew(1).ne.0.0) then
                    prcntdif=abs(1.0d0-esi0(1,jlos,ilos,level)
     &/enew(1))*100.0d0
                    if(prcntdif.gt.convrg) then
                      if(itest.eq.0)then
                        write(6,'(a29,1pe7.1,a34,i4)')
     &'downit. PERCENT DIFFERENCE >',convrg,
     &'% FOR DOWNWARD RADIATION AT LEVEL ',level
                       write(6,'(8x,a22,3(i5))')'ILOS, JLOS, and M ARE',
     &ilos,jlos,1
                        itest=100
                      endif
                    endif
                  endif
                  esi0(1,jlos,ilos,level)=enew(1)
c m=2
                  if(np.gt.1)then
                    enew(2)=enew(2)+bndry(2)*T+esing0(2,jlos,ilos,level)
                    if(itest.ne.100.and.enew(2).ne.0.0) then
                      prcntdif=abs(1.0d0-esi0(2,jlos,ilos,level)
     &/enew(2))*100.0d0
                      if(prcntdif.gt.convrg) then
                        if(itest.eq.0)then
                          itest=100
                          write(6,'(a29,1pe7.1,a34,i4)')
     &'downit. PERCENT DIFFERENCE >',convrg,
     &'% FOR DOWNWARD RADIATION AT LEVEL ',level
                          write(6,'(a22,3(i5))')'ILOS, JLOS, and M ARE',
     &ilos,jlos,2
                        end if
                      end if
                    endif
                    esi0(2,jlos,ilos,level)=enew(2)
c m greater than 2
                    if(np.gt.2)then
                      esi0(3,jlos,ilos,level)=enew(3)+bndry(3)*T+
     &esing0(3,jlos,ilos,level)
                      if(np.gt.3)esi0(4,jlos,ilos,level)=enew(4)+
     &bndry(4)*T+esing0(4,jlos,ilos,level)
                    endif
                  endif
30             continue
40          continue
c            it=it+1
         enddo
50    continue 
      return
      end
c ---------------------------------------------------------------------
