c $Header: /usr/people/flittner/radtran/flatpsv/src/paero.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
      subroutine paero(Tfine,p,ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,
     &CFISQ,SIFI,SFISQ,AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,
     &AMU,ANU,fsbs)
c
c compute the aerosol phase matrix using the equations developed by Z.
c Sekra and in Dr. Herman's dissertation.
c for reference use Herman et al., 1975, JAS, 32, pp. 1430-1445.
c NOTE that P13 of the above paper is missing a mu. See below in the code.
c
c Here the angles are defined as:  The +z axis is from the bottom of the
c atmosphere to the top.  Theta is the polar angle of the
c radiation propagation vector measured from the +z axis. Phi is the 
c azimuth angle measured clockwise from the +x axis when looking in the
c +z direction.  In summary, this is a standard right handed coordinate
c system with the +z axis pointing from the bottom of the atmosphere to
c the top.
c
c last modified: Aug. 9, 1997...def
c purpose: Change the def. of the azimuth angle see comments in Rayphase.
c
c last modified: Dec. 30, 1997...def
c purpose: Increase second dimension of Tfine from 4 to 7 so that can
c          use phase functions created by tmatrix program and liou.
c
c
c Tfine  = Combinations of T1 & T2 functions found in Herman's dissertation.
c          or Herman et al., 1971, JAS, 28, pp. 419-428.
c          Note that eq. (2b) of this paper is missing a minus sign.
c
c         To be used in phase matrix for scattering angles 0 to 180.
c         Tfine( ,1)=T1T1*
c         Tfine( ,2)=T2T2*
c         Tfine( ,3)=Real(T1T2*)
c         Tfine( ,4)=-Im(T1T2*)
c COOSN = cosine(phi_in - phi_out)
c SIIN  = sine(phi_in - phi_out)
c AMUP  = cosine(theta_in)
c ANUP  = sine(theta_in)
c AMUPS=AMUP*AMUP
c AMU   = cosine(theta_out)
c ANU   = sine(theta_out)
c AMUSQ=AMU*AMU
c AMUMU=AMU*AMUP
c COPSI=ANU*ANUP+AMU*AMUP*COOSN
c SISQ=COPSI*COPSI
c CFISQ=COOSN*COOSN
c SIFI=COPSI*COOSN
c SFISQ=SIIN*SIIN
c cos_captheta=AMUMU+ANUP*ANU*COOSN
c fsbs
c
c output
c
c p(4,4)  - aerosol scattering matrix
c
c
c get the index in the Tfine array for the values that straddle the 
c scattering angle.
c
      implicit none
      include 'param.inc'
c passed
      real*8 Tfine(nan,7),p(4,4)
      real*8 ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,CFISQ,SIFI,SFISQ
      real*8 AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,AMU,ANU
      logical fsbs
c local
      integer MT,MTP1,i,j
      real*8 T1,T2,T3,T4
      real*8 YA,onemya
c
      call Tfineindx(cos_captheta,captheta,YA,onemya,MT,MTP1)
c
c compute the T values or R values as written in Herman's paper
c
      T1=YA*Tfine(MT,1)+onemya*Tfine(MT+1,1)
      T2=YA*Tfine(MT,2)+onemya*Tfine(MT+1,2)
      T3=YA*Tfine(MT,3)+onemya*Tfine(MT+1,3)
      T4=YA*Tfine(MT,4)+onemya*Tfine(MT+1,4)
***********************************************************************
      p(1,1)=(CFISQ*T1+SISQ*T2+2.0D0*SIFI*T3)
      p(1,2)=(SFISQ*(AMUPS*T1+AMUSQ*T2+2.0d0*AMUMU*T3))
      p(2,1)=(SFISQ*(AMUPS*T2+AMUSQ*T1+2.0D0*AMUMU*T3))
      p(2,2)=(SISQ*T1+CFISQ*T2+2.*SIFI*T3)
      p(3,3)=((SIFI-AMUMU*SFISQ)*(T1+T2)+
     &(CFISQ+SISQ-SFISQ*(AMUSQ+AMUPS))*T3)
      p(3,4)=((CFISQ-SISQ-SFISQ*(AMUSQ-AMUPS))*T4)
      p(4,3)=((SISQ-CFISQ-SFISQ*(AMUSQ-AMUPS))*T4)
      p(4,4)=((SIFI+AMUMU*SFISQ)*(T1+T2)+
     &(CFISQ+SISQ+SFISQ*(AMUSQ+AMUPS))*T3)
      if(fsbs)then
        p(1,3)=0.0d0
        p(1,4)=0.0d0
        p(2,3)=0.0d0
        p(2,4)=0.0d0
        p(3,1)=0.0d0
        p(3,2)=0.0d0
        p(4,1)=0.0d0
        p(4,2)=0.0d0
      else
        p(1,3)=(-SIIN*(AMUP*COOSN*T1+AMU*COPSI*T2+
     &(AMUP*COPSI+AMU*COOSN)*T3))
        p(1,4)=(-SIIN*(AMU*COOSN-AMUP*COPSI)*T4)
        p(2,3)=(SIIN*(AMUP*COOSN*T2+AMU*COPSI*T1+
     &(AMUP*COPSI+AMU*COOSN)*T3))
        p(2,4)=-p(1,4)
        p(3,1)=(2.0d0*SIIN*(AMU*COOSN*T1+AMUP*COPSI*T2+
     &(AMUP*COOSN+AMU*COPSI)*T3))
        p(3,2)=(-2.0d0*SIIN*(AMU*COOSN*T2+AMUP*COPSI*T1+
     &(AMUP*COOSN+AMU*COPSI)*T3))
        p(4,1)=(2.0D0*SIIN*(AMU*COPSI-AMUP*COOSN)*T4)
        p(4,2)=-p(4,1)
      endif
c***********************************************************************
c for testing
c
c      do i=1,4
c         do j=1,4
c            p(j,i)=0.0
c         enddo
c      enddo
c      do i=1,2
c         do j=1,2
c            p(j,i)=0.5*(YA*Tfine(MT,5)+onemya*Tfine(MT+1,5))
c         enddo
c      enddo
      return
      end
