c $Header: /usr/people/flittner/radtran/flatpsv/src/chap.f,v 1.16 2000/07/29 00:15:41 flittner Exp flittner $
      function chap(r0,H,mu)
c     ******************************************************************
c
c        this subroutine calculates chapman function as approximated by
c        john a. fitzmaurice, appl. opt. 3,640(1964).
c
c        method:
c                chap(x,mu)= sqrt(x*pi/2) exp(x*(mu**2)/2)
c
c                where-  x=layer height from the center of earth
c                          (in units of scale height of attenuating
c                                          species)
c                        x=r0/H
c
c     ******************************************************************
      implicit none
      include 'consts.cmn'
c passed
      real*8 r0,H,mu
c local
      real*8 a(5),dp,dy,dx,dt,dsum,dsq,domerf,x
      real*8 c2,c3,dsqatc1,yp,si,y,chap,c1
      real*8 f2
      external f2
      integer i
      data dp/0.3275911d+00/
      data a    /
     1         1.061405429d+00,        -1.453152027d+00,
     2         1.421413741d+00,        -0.284496736d+00,
     3         0.254829592d+00     /
c
      x=r0/H
      y=mu
      dy = mu
      dx = mu*sqrt(x*0.5)
      dsq = sqrt(x*0.5*pi)
      dt = 1.0d+00 / (1.0d+00 + dp * dx)
      dsum = dt * a(1)
      if(y.ge.0.2)then
c
c*****for all but large solar zenith angles use the series expansion
c*****       of the chapman function
c    execpt if the value of x is less than 50
        if(x.lt.50.)then
          domerf=f2(r0,H,mu)
          chap = domerf
        else
          dt = 0.5d+00 / (dx**2)
          domerf=(1.0d+00-dt*(1.d+00-3.d+00*dt*(1.d+00-5.d+00*dt)))/dy
          chap = domerf
        endif
      else
        if(y.ge.0.0)then
          dsum = dt * a(1)
          do 10 i=2,5
   10        dsum = dt * (a(i) + dsum)
          domerf = dsum * dsq
          chap = domerf
        else
c ray goes tanget at altitude below observer
          c1=x*sqrt(1.-y*y)
          si=acos(mu)
          dsqatc1 = sqrt(c1*0.5*pi)
c for 90 angle dt=1.0
          dsum = a(1)
          do 20 i=2,5
   20        dsum = (a(i) + dsum)
          domerf = dsum * dsqatc1
          c2=exp(c1*(1.-sin(si)))
          yp=-y
          dx = yp*sqrt(x*0.5)
          dt = 1.0/ (1.0 + dp * dx)
          dy = yp
          if(yp.ge.0.2)then
            dt = 0.5 / (dx*dx)
            c3=(1.0-dt*(1.-3.*dt*(1.-5.*dt)))/dy
          else
            dsum = dt * a(1)
            do 30 i=2,5
   30          dsum = dt * (a(i) + dsum)
            c3= dsum * dsq
          endif
          domerf =2.0d0*c2*domerf -c3
        endif
      endif
      chap=domerf
      return
      end
c
      function f2(r0,H,mu)
      implicit none
      include 'consts.cmn'
c passed
      real*8 r0,H,mu,f2
c local
      real*8 x,nu,w,tmp
      real*4 erf,rsqrtw
      external erf
c
      x=r0/H
      nu=1.-mu*mu
      nu=min(nu,1.)
      nu=max(nu,-1.)
      nu=sqrt(nu)
      w=x*(1.-nu)
      rsqrtw=real(sqrt(w))
      if(w.lt.9.5)then
       tmp=-x*mu+sqrt(1.+2.0*x-w)*(sqrt(w)+sqrt(pi)/2.*
     &(1.-dble(erf(rsqrtw)))*exp(w))
      else
       tmp=-x*mu+sqrt(1.+2.*x-w)*(sqrt(w)+0.5/sqrt(w)*
     &(1.-1./2./w+3./4./w/w-15./8./w/w/w))
      endif
      f2=tmp
      return
      end
      FUNCTION erf(x)
      REAL erf,x
CU    USES gammp
      REAL gammp
      if(x.lt.0.)then
        erf=-gammp(.5,x**2)
      else
        erf=gammp(.5,x**2)
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $2s%{-5[K.
      FUNCTION gammp(a,x)
      REAL a,gammp,x
CU    USES gcf,gser
      REAL gammcf,gamser,gln
      if(x.lt.0..or.a.le.0.)pause 'bad arguments in gammp'
      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $2s%{-5[K.
      SUBROUTINE gser(gamser,a,x,gln)
      INTEGER ITMAX
      REAL a,gamser,gln,x,EPS
      PARAMETER (ITMAX=100,EPS=3.e-7)
CU    USES gammln
      INTEGER n
      REAL ap,del,sum,gammln
      gln=gammln(a)
      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in gser'
        gamser=0.
        return
      endif
      ap=a
      sum=1./a
      del=sum
      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gser'
1     gamser=sum*exp(-x+a*log(x)-gln)
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $2s%{-5[K.
      SUBROUTINE gcf(gammcf,a,x,gln)
      INTEGER ITMAX
      REAL a,gammcf,gln,x,EPS,FPMIN
      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)
CU    USES gammln
      INTEGER i
      REAL an,b,c,d,del,h,gammln
      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d
      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b
        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c
        if(abs(c).lt.FPMIN)c=FPMIN
        d=1./d
        del=d*c
        h=h*del
        if(abs(del-1.).lt.EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gcf'
1     gammcf=exp(-x+a*log(x)-gln)*h
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $2s%{-5[K.
      FUNCTION gammln(xx)
      REAL gammln,xx
      INTEGER j
      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)
      SAVE cof,stp
      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      gammln=tmp+log(stp*ser/x)
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $2s%{-5[K.
