c $Header: /usr/people/flittner/radtran/flatpsv/src/compc.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
c***********************************************************************

      subroutine compc(dmu,fmatrx,caer,nlos,ii,np,ifwd,
     &useold)

C COMPC CALCULATES THE INTEGRAL OF THE PHASE FUNCTION OVER ALL SOLID ANGLES
C FOR INPUT ANGLE THETA0 THE ANGLE OF THE INCIDENT LIGHT.
C THE OUTPUT ANGLES ARE ARRAY THETA(1,2,..,NANG) WHICH ARE THE SCATTERING ANGLES
C TO BE USED IN THE RADIATIVE TRANSFER CALCULATIONS.
C SINCE THESE SCATTERING DIRECTIONS ARE TO BE USED AS INCIDENT ANGLES FOR THE
C NEXT LAYER, THE INPUT ANGLE THETA0 WILL BE ONE OF THE THETA ARRAY VALUES.
C FOR EACH THETA0 THE SCATTERING ANGLE (I.E BETTWEEN THETA0 AND THE THETA ARRAY)
C WILL BE DIFFERENT, THEREFORE THE RESULT OF THE INTEGRATION OF THE PHASE 
C FUNCTION WILL BE A FUNCTION OF THETA0.
C FOR CONSERVATION OF ENERGY THE INTEGRAL OF THE PHASE FUNCTION OVER ALL 
C SCATTERING ANGLES (ALL SOLID ANGLES) SHOULD BE ONE FOR ALL INCIDENT ANGLES
C USED IN THE RADIATIVE PROGRAM.
C THE COMPUTED CONSTANT C WILL BE USED TO NORMALIZED THE PHASE FUNCTION WHEN
C THE INCIDENT ANGLE IS THETA0, AND THE SCATTERED LIGHT IS CALCULATED FOR
C ANGLES THETA.
C THE SINGLE SCATTERING ALBEDO OMEGA0 IS USED (IN THE RADIATIVE TRANSFER 
C PROGRAM) TO MULTIPLY THE NORMALIZED (TO ONE) PHASE FUNCTION (C*PF(ANG)) 
C I.E (OMEGA0*(CPF(I)*FMATRX(k,I,j)) IS A PHSE FUNCTION NORMALIZED TO OMEGA0
C FOR INCIDENT RAY AT ANGLE THETA0(I) AND SCATTERING DIRECTION THETA
C
C PF(I)= PHASE FUNCTION FOR SCATTERING ANGLE (I-1) DEGREES. (INPUT)
C THETA0= INCIDENT DIRECTION IN DEGREES. (INPUT)
C THETA(I)= SCATTERED DIRECTION IN DEGREES. (INPUT)
C C= THE NORMALIZATION CONSTANT FOR THE PHASE FUNCTION WHEN THE INCIDENT 
C DIRECTION IS THETA0
C IDENTIFY INDEX FOR THE LARGEST THETA SMALLER THAN 90
C DELTA PHI=DPHI
C DELTA THETA=TH2-TH1
C THESE DELTA'S ARE USED IN THE RADIATIVE TRANSFER CALCULATION.
C
C last mod. July 27, 1998...def
C purpose: change useold from logical to integer so can switch between
C different normalization schemes:
C useold=0 : use the herman & browning method of applying all the error to the
C            forward direction
C useold=1 : apply the same normalization factor to all directions
C useold=2 : weight the factor by the product of the phase function and the
C            solid angle.
c
c last mod. Oct. 22, 1998...def
c purpose: move write statements for useold=2 inside if for useold=2.
c
c---------------------------------------------------------------------

      implicit none
      include 'consts.cmn'
      include 'param.inc'
c variables that are passed
      integer nlos,ii,np,ifwd
      real*8 dmu(mlos),fmatrx(4,4,nfi,mlos,mlos),caer
      integer useold
c variables that are local
      integer j,k
      real*8  dphi,sumtht,sumphi,phase,sumfwd,sumwat,sumwatphi,chksum
      logical debug
c
      debug=.FALSE.
      dphi=180.0d0/dble(nfi-1)

C INTEGRATE OVER PHI (NOTE: THE INTEGRATION SHOULD BE EXACTLY AS IN THE
C RADIATIVE TRANSFER CALCULATION.
C SET FIRST INCIDENT ANGLE THETA0(I)
C SET SCATTERING ANGLE THETA(J) AND INTEGRATE OVER PHI
C FOR DPHI=30 THERE ARE 12 PHI'S (0,30,60,90,120,...330) THUS THE INDEX 
C    FOR PHI GOES FROM 1 TO 12
C NOTE K=1 IS PHI=0 K=7 IS PHI=180 FOR BOTH THERE IS NO SYMMETRY.
C SYMMETRY FOR THE INDEX K FMATRX(14-k,j,I)=FOR K.GT.7
      
      sumtht=0.00d0
      sumfwd=0.0d0
      sumwat=0.0d0
c loop over output theta angle, j
      do 30 j=1,nlos
         k=1
c if only using one polarization parameter, which is actually unpolarized
c case, then matrix should be actually a scalar.
         if(np.eq.1)then
            phase=fmatrx(1,1,k,ii,j)
         else
            phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
            phase=phase*0.5d0
         endif
         sumphi=phase*df*dmu(j)
         sumwatphi=phase*df*phase*df*dmu(j)
         if((useold.eq.0).and.(ifwd.eq.j))then
           sumfwd=sumphi*dmu(ifwd)
           if(debug)then
             write(6,*)'compc. fraction of solid angle ',
     &(dmu(ifwd)*df)/4.0d0/pi
           endif
           sumphi=0.0d0
         endif
         do 20 k=2,(nfi-1)
            if(np.eq.1)then
               phase=2.0d0*fmatrx(1,1,k,ii,j)
            else
               phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
            endif
            sumphi=sumphi+phase*df*dmu(j)
            sumwatphi=sumwatphi+phase*df*dmu(j)*phase/2.*df*dmu(j)
20       continue
         k=nfi
         if(np.eq.1)then
            phase=fmatrx(1,1,k,ii,j)
         else
            phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
            phase=phase*0.5d0
         endif
         sumphi=sumphi+phase*df*dmu(j)
         sumwatphi=sumwatphi+phase*df*dmu(j)*phase*df*dmu(j)

C SUMPHI IS THE RESULT OF THE INTEGRATION OVER PHI
C INTEGRATE OVER THETA
         sumtht=sumtht+sumphi
         sumwat=sumwat+sumwatphi
30    continue
      if(useold.eq.0)then
        caer=(1.0d0-sumtht)/sumfwd
      else if(useold.eq.1)then
        caer=1.0d0/sumtht
      else if(useold.eq.2)then
        caer=sumtht/sumwat
      endif
      if(debug)then
        write(6,*)'compc. sumfwd sumtht sumwat',sumfwd,sumtht,sumwat
        write(6,*)'compc. ',sumfwd/(sumtht+sumfwd),(sumtht+sumfwd),
     &1./(sumtht+sumfwd)
        write(6,*)'compc. ',1./sumtht,sumtht/sumwat,caer
      endif
      if(useold.eq.2)then
c
c check the useold=2 case
c
c
c output so can check
c
        open(8,file='compc.out',status='unknown')
        chksum=0.0d0
        do j=1,nlos
           k=1
c if only using one polarization parameter, which is actually unpolarized
c case, then matrix should be actually a scalar.
           if(np.eq.1)then
              phase=fmatrx(1,1,k,ii,j)
           else
              phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
              phase=phase*0.5d0
           endif
           write(8,*)phase,dmu(j),df
           sumphi=phase*caer*phase*df*dmu(j)
           do k=2,(nfi-1)
              if(np.eq.1)then
                 phase=2.0d0*fmatrx(1,1,k,ii,j)
              else
                 phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
              endif
              write(8,*)phase,dmu(j),df
              sumphi=sumphi+phase*caer*phase/2.*df*dmu(j)
           enddo
           k=nfi
           if(np.eq.1)then
              phase=fmatrx(1,1,k,ii,j)
           else
              phase=fmatrx(1,1,k,ii,j)+fmatrx(1,2,k,ii,j)+
     &fmatrx(2,1,k,ii,j)+fmatrx(2,2,k,ii,j)
              phase=phase*0.5d0
           endif
           write(8,*)phase,dmu(j),df
           sumphi=sumphi+phase*caer*phase*df*dmu(j)
           chksum=chksum+sumphi*dmu(j)*df
        enddo
        write(6,*)'compc. chksum= ',chksum
        close(8)
        call exit
      endif
      return
      end
c***********************************************************************
