c $Header: /usr/people/flittner/radtran/flatpsv/src/phasf.f,v 1.14 1999/08/16 18:57:13 flittner Exp flittner $
c***********************************************************************

      subroutine phasf(f,r,n1,wavel,omega0,n,refre,refim,refmed,cext,
     &nmode,g)

C this subroutine calculates the phase matrix for scattering angles 
C ranging from 0 to 180 for every 0.1 degree.
C Using an algorithm from bohren and huffman 1983 (appendix a in 
C absorption and scattering of light by small particles) 
C p(theta) is the phase function of the normalized aerosol
C      calculate p(theta) for each radius rad
C      size distribution f(r)(normalized to 1 when f(r) is integrated)
C      p(angle) is normalized to 1. (integration over all solid angles)   
C ang(i) is the scattering angle for p(i)           
C refre and refim are the real and imaginary refractive index of a spherical
C      particle.
C refmed is the refractive index of the mediuM
C refrel is the relative refractive index
C wavel is the wavelength (in microns) in free space
C r is particle radius (in microns)
C f(r) is the number of particles (#/cm^3) per radius interval [#/cm^3/um]
C n1=number of radii bins.
C omega0 is the single scattering albedo for the distribution f
C nang is the number of angles between 0 and 90 degrees
C cext and cscat are total extinction and scattering x-sections of the
C      distribution F
c subroutine bhmie calculates scattering matrix elements at nan angles
C including 0,90, and 180 degrees.
C
C---OUTPUT---
C FILE 81 - MIECALC.DAT SAVES THE FOLLOWING PARAMETERS:
c       ang and T1 & T2 functions from Herman's diss.
C      omega0 - single scatter albedo
C      cscat - normalization constant for scatter
C      cext - normalization constant for extinction
C
C modified Aug 9, 1996...def
C purpose: Change output format of TT to hexidecimal to increase
C          precision.
C
C last modified 12-19-96...def
C purpose: Calc and pass the scattering asymmetry parameter (g)
C
C last modified 01-02-98...def
C purpose: Correct error incompuation of TT(4). This should be 
C          TT(4)=Im[T2*conj(T1)]=-Im[T1*conj(T2)]
C
C-----------------------------------------------------------------------

      implicit none
      include 'consts.cmn'
      include 'param.inc'
c passed in
      integer n1,n,nmode
      real*8 f(n+1,3),r(n+1),wavel
      real*8 refre(3),refim(3),refmed
c passed out
      real*8 omega0,cext,g
c local
c
c T1 and T2 are Herman's T1 and T2 functions found in his disseration.
c HT1 & HT2 are these functions for the forward and backward directions.
      complex*16 refrel,s1(nan),s2(nan),T1,T2,czero,HT1(2),HT2(2)
      integer j,i,imode
      real*8 ak,aksqr,angle,rad,area,x,a1,a2,ai,s11,xsect,qext
      real*8 qscat,dang,pf(nan),ang(nan),costht(nan),TT(4,nan)
      real*8 qback,cscat,const,sum,rqsenis(nan),sumf,tmp1,c1
      logical lrayleighphase
      real*8 cosa1,cosa2,pf1,pf2,sum2,mom2,mom2qe,ratio
c***********************************************************************
      czero=cmplx(0.0,0.0)
      lrayleighphase=.FALSE.
      write(31,*)' input data in phasf'
      write(31,*)' wavel= ',wavel,' refre= ',refre
      write(31,*)' refim= ',refim,' refmed= ',refmed
      write(31,*)' n1=',n1,'nmode',nmode

      dang=(pi/2.0d0)/float(nang-1)
      cext=0.0d0
        cscat=0.0d0
        sumf=0.0d0
        ak=(2.0d0*pi*refmed/wavel)
      aksqr=ak*ak

      open(81,file='miecalc.dat',status='unknown')
      write(81,'(3x,a3,7x,a1,7x,a4,5x,a3,2x,a14,3x,a4,5x,a2)')'rad',
     &      	'x','qext','DNc','qext*area*f(i)','cext','s1'

      ai=0.0d0
      do i=1,nan
         ang(i)=dang*ai/dr
           pf(i)=0.0d0
         ai=ai+1.0d0
      enddo

      do i=1,nan
         TT(1,i)=0.0d0
         TT(2,i)=0.0d0
         TT(3,i)=0.0d0
         TT(4,i)=0.0d0
      enddo
      ai=1
      do i=2,nan-1
         angle=dang*ai
         costht(i)=cos(angle)
         rqsenis(i)=1.0d0/(1.0d0-costht(i)*costht(i))
         ai=ai+1.0d0
      end do
      costht(1)=1.0d0
      costht(nan)=-1.0d0
      if(lrayleighphase)then
        c1=sqrt(aksqr*1.0d08*3.0d0/pi/8.0d0)
        do i=1,nan
           s2(i)=cmplx(costht(i),0.0d0)*c1
           s1(i)=cmplx(1.0d0,0.0d0)*c1
        enddo
        HT1(1)=0.0d0
        HT1(2)=0.0d0
        HT2(1)=1.0d0*c1
        HT2(2)=1.0d0*c1
      endif
      do imode=1,nmode
         refrel=cmplx(refre(imode),refim(imode))/refmed
         do i=1,n1+1
            rad=r(i)
            if(i.eq.1)then
              tmp1=0.5d0*f(1,imode)*(r(2)-r(1))
            else if(i.eq.n1+1)then
              tmp1=0.5d0*f(n1+1,imode)*(r(n1+1)-r(n1))
            else
              tmp1=f(i,imode)*(r(i+1)-r(i-1))*0.5d0
            endif
            if(tmp1.gt.0.0d0)then
c rad is in microns and will perform cross-section in terms of 
c square centimeters.  That is why there is a 1.0d-08 factor in the area.
              x=ak*rad
              if(lrayleighphase)then
                qscat=1.0d0
                qext=1.0d0
              else
                call bhmie(x,refrel,nang,s1,s2,nan,HT1,HT2,qext,qscat,
     &qback)
              endif
              area=pi*(rad**2)*1.0d-08
              write(81,1010)rad,x,qext,f(i,imode),qext*area*tmp1,cext,
     &      		dble(s1(2)),dimag(s1(2))
              cscat=cscat+qscat*area*tmp1
              xsect=qext*area
              cext=cext+xsect*tmp1
              sumf=sumf+tmp1
           if(i.eq.1)then
            mom2=tmp1*r(i)**2.0d0
            mom2qe=qext*tmp1*r(i)**2.0d0
           else
            mom2=mom2+tmp1*r(i)**2.0d0
            mom2qe=mom2qe+qext*tmp1*r(i)**2.0d0
           endif

c s1 is the scattering coefficient in the perpendicular direction and
c s2 is the scattering coefficient in the parallel direction.
c comparison with findt output, s1=T(3)-iT(4) and s2=T(1)-iT(2).  The
c minus sign is due to the fact that Bohren and Huffman define the index of
c refraction as RFR + i RFI and findt uses RFR - i RFI.
c In Herman's diss. he has defined T1=(Si-xSj)/(1-x*x) and T2=(Sj-xSi)/(1-x*x).
c Findt uses Si=T(1)+iT(2) and Sj=T(3)+iT(4).  Thus Si=s2* and Sj=s1*.
c This means that T1=conjg((s2-xs1)/(1-x*x)) and T2=conjg((s1-xs2)/(1-x*x)).
c      		write(81,*)HT1(1),HT2(1)
c will add to s1sum and s2sum
              T1=conjg(HT1(1))
              T2=conjg(HT2(1))
              TT(1,1)=TT(1,1)+T1*conjg(T1)*tmp1
              TT(2,1)=TT(2,1)+T2*conjg(T2)*tmp1
              TT(3,1)=TT(3,1)+dble(T1*conjg(T2))*tmp1
              TT(4,1)=TT(4,1)-dimag(T1*conjg(T2))*tmp1
              do j=2,nan-1
                 T1=(s2(j)-costht(j)*s1(j))*rqsenis(j)
                 T2=(s1(j)-costht(j)*s2(j))*rqsenis(j)
                 TT(1,j)=TT(1,j)+T1*conjg(T1)*tmp1
                 TT(2,j)=TT(2,j)+T2*conjg(T2)*tmp1
                 TT(3,j)=TT(3,j)+dble(T1*conjg(T2))*tmp1
                 TT(4,j)=TT(4,j)-dimag(T1*conjg(T2))*tmp1
              end do
              T1=conjg(HT1(2))
              T2=conjg(HT2(2))
              TT(1,nan)=TT(1,nan)+T1*conjg(T1)*tmp1
              TT(2,nan)=TT(2,nan)+T2*conjg(T2)*tmp1
              TT(3,nan)=TT(3,nan)+dble(T1*conjg(T2))*tmp1
              TT(4,nan)=TT(4,nan)-dimag(T1*conjg(T2))*tmp1
              do j=1,nan
                 a1=.5*abs(s2(j))*abs(s2(j))
                 a2=.5*abs(s1(j))*abs(s1(j))
                 s11=a1+a2
                 pf(j)=pf(j)+s11*tmp1
              enddo
            endif
         enddo
      enddo
      if(lrayleighphase)then
        do j=1,nan
           TT(1,j)=TT(1,j)*cscat
           TT(2,j)=TT(2,j)*cscat
           TT(3,j)=TT(3,j)*cscat
           TT(4,j)=TT(4,j)*cscat
           pf(j)=pf(j)*cscat
        end do
      endif
C WHEN THERE IS NO ABSORBTION CSCAT SHOULD BE EQUAL TO CEXT
C BUT CAN BE NUMERICALY OMEGA0=1.+1.E-10. IN THESE CASES OMEGA0 IS SET 
C TO 1.0
      write(6,*)'phasf. mom2',mom2
      write(6,*)'phasf. mom2qe',mom2qe
      write(6,*)'phasf. mom2/mom2qe',mom2/mom2qe
      omega0=cscat/cext
      omega0=min(omega0,1.0d0)
      write(31,2000)cext,cscat
c normalize to unity
      const=1.0d0/(aksqr*cscat*1.0d+08)
      g=0.0d0
      do j=1,nan
         pf(j)=pf(j)*const
      enddo
c also print every 1.0 deg
      open(12,file='phasf.out',status='unknown')
      do j=1,nan,nan/180
         write(12,'(f8.2,1PE13.4)')ang(j),pf(j)
      enddo
      close(12)
      sum=0.0d0
      g=0.0d0
      cosa1=cos(ang(1)*dr)
      pf1=(TT(1,1)+TT(2,1))*(1.0d0+cosa1*cosa1)+TT(3,1)*4.0d0*cosa1
c      write(6,*)'phasf. pf pf2',ang(1),(pf(1)/pf1/const*2.0-1.)*100.
      sum2=0.0
      do j=1,nan-1
         cosa2=cos(ang(j+1)*dr)
         sum=sum+0.5d0*(pf(j)+pf(j+1))*
     &(cosa1-cosa2)
         g=g+0.5d0*(pf(j)*cos(ang(j)*dr)+pf(j+1)*cos(ang(j+1)*dr))*
     &(cosa1-cosa2)
         pf2=(TT(1,j+1)+TT(2,j+1))*(1.0d0+cosa2*cosa2)+
     &TT(3,j+1)*4.0d0*cosa2
         ratio=pf(j+1)/pf2/const*2.0
c         write(6,*)'phasf. pf pf2',ang(j+1),(ratio-1.0)*100
         sum2=sum2+0.5d0*(pf1+pf2)*(cosa1-cosa2)
         cosa1=cosa2
         pf1=pf2
      enddo
      g=g/sum
      write(31,2010)omega0,g
      sum=sum*2.0d0*pi
      sum2=sum2*const*pi
c      write(6,*)'phasf. the sum for integrating Pf is ',sum
      write(6,*)'phasf. the asymmetry parameter for Pf is ',g
      write(6,*)'phasf. the sum for integrating PhaseFunction is ',sum2
      sum=0.0d0
c
      write(18,2000)cext,cscat
      write(18,2010)omega0,g
      write(18,2020)
      write(18,*)cext,cscat,nang
      write(18,*)g
      do j=1,nan
c         write(18,'(1x,f5.1,4(1x,z16))')ang(j),(TT(i,j),i=1,4)
c include the phase function for testing
         write(18,'(1x,f5.1,5(1x,z16))')ang(j),(TT(i,j),i=1,4),
     &pf(j)/const
      end do
c
200   return
1010  format(f7.3,f9.3,2x,1Pe9.3,1P4e10.3,1Pe11.3)
2000  format('EXTINCTION COEFFICIENT = ',1PE12.4,' (cm*cm)',/,
     &'SCATTERING COEFFICIENT = ',1PE12.4,' (cm*cm)')
2010  format('SINGLE SCATTERING ALBEDO = ',f6.4,
     &'; ASYMMETRY PARAMETER = ',f6.4)
2020  format(72('*'))
      end
c***********************************************************************
