      program  FRONT_AERONET           
C***********************************************************************
C	AUTHOR: Nickolay Krotkov and Alexander P. Vasilkov
C
C	FUNCTION:
c     To fit imaginary refr index 
C     using AERONET PSD, real refr index and MFRSR diffuse and total irradiance  measurenets
C	
C       WARNING
C       1) Only one wavelength version (lambda(1) and icount=1 ONLY !!! )                      
C       re-calculated at each wavelength !
C       2) only bi-modal log-normal PSD
C       3) Refractive index is the same for both modes (one component model)
C        4) surface pressure should be 1 atm=1013.25 in both flat.in and pressure.prf
C           (otherwise tau_aer is changed)! Pressure in flat.in can not be larger than
C          surface pressure in pressure.prf file.
C
C	HISTORY: 
c     Modified May 8     , 2007 to re-process all inversions at 440nm with NO2 corrections                                 
c     Modified July 15   , 2005 to re-process all inversions with NO2 corrections                                 
c     Modified January 9, 2005 to include ozone absorption and replace O2-O2 by NO2 in  water vapor absorption
c     Modified January 5, 2005 get rid of coe.dat file ( read lambda and x-sections from MFRSR input file)
c     Modified January 4, 2005 reduce AOT according to tau_NO2 at 368nm                          
c     Modified January 4, 2005 change ozone (NO2) profile to 1st Umkehr layer only               
c     Modified January 4, 2005 change total ozone to total NO2 (still ozone profile)              
c     Modified January 4, 2005 to add NO2 correction (using Brewer direct sun NO2 measurements)
c     Modified March 19, 2004 to add interpolated AERONET AOT to the output
c     Modified March 18, 2004 to compensate for ARIZONA code diffuse flux overestimation by 0.3%
c                             which is equivalent to enhancing measured diffuse by the same amount *1.003
c     Modified March 16, 2004 to fix bug with new format input file reading                      
c     Modified March 15, 2004 to re-calculate total after diffuse correction            
c     Modified March 14, 2004 to change diffuse due to additional diffuse cosine correction!
c     Modified March 11, 2004 to use CT fit: t_fit=.true.                          
c     Modified March 10, 2004 to  use dt fit tolerance=0.0001   and ipseudo=1                            
c     Modified March 10, 2004 to use diffuse/total fraction fit: dt_fit=.true.                          
c     Modified March 2, 2004 to  use absolute DD fit tolerance=  0.001                                  
c     Modified March 2, 2004 to read new input data format (AERONET+20mfrsr)                             
c     Modified Sep  28, 2003 to double   AERONET  k                                                
c     Modified Sep  26, 2003 to use only AERONET data  and no fitting                              
c     Modified July 30, 2003 to use MFRSR AOT correction at 368nm only !!!           
c     Modified July 30, 2003 to fix output format for RVC>10micron                    
c     Modified July 29, 2003 to use AERONET interpolated AOT at 368nm for SSA at 368nm
c     Modified July 18, 2003 to read new format AERONET/MFRSR data                         
c     Modified July 10, 2003 restrict fitting to SZA<70deg                                 
c     Modified July 8 , 2003 to use absolute DD fit tolerance=  0.01                       
c     Modified July 6 , 2003 to constrain number of k iterations: max_k_iter              
c     Modified July 6 , 2003 to process missing MFRSR AOT data for zero direct irradiance)
c     Developed June - July, 2003
C     template program: front_end_TOMRAD_GS_V6_440nm.f Developed by AV in June, 2000-June 2003
C***********************************************************************
      implicit none

      integer :: imfrsr_ldir
      integer aeronet_dump_length
      parameter (aeronet_dump_length=125)
      real aeronet_dump(aeronet_dump_length)	! AERONET web data only: 
						! it's also affects get_aer_mf_v2.f and get_aeronet subs !!!
      real c0b,c1b,c2b,rhob	! Ozone, Rayleigh and depolarization values are defined here and passed to gs_fit to get rid of coe.dat gas2.coe files
            
      logical no2_corr
      parameter (no2_corr=.true.)	! no2 correction
      real xno2,tau_no2

      logical real_refr_const	! to use constant real refractive index
      real real_refr_value
      parameter (real_refr_const=.false.,real_refr_value=1.4)	! set constant value for real refractive index
      logical dd_fit		! whether to use diffuse/direct fit of k
      logical dt_fit		! whether to use diffuse/total  fit of k
      logical  t_fit		! whether to use total          fit of k
      logical  d_fit		! whether to use diffuse        fit of k
      logical log_fit		! whether to use logarithms of dd and  k
      logical no_radiance_output
c---  choose fitting method:
      parameter (dd_fit=.false.,dt_fit=.true.,t_fit=.false.)
      parameter (d_fit=.false.,log_fit=.false.)
      parameter (no_radiance_output=.true.) 

      logical    cimelaot,mtau_corr,mdif_corr
      parameter (mtau_corr=.false.,mdif_corr=.false.)	! correct MFRSR AOT and/or measured diffuse DIF ( for sensitivity study)
      real mean_lnk,mdif_corr_cos
      parameter (mean_lnk=+0.08,mdif_corr_cos=1.003)	! dtau=myu*<lnk>, increase measured diffuse irradiance to account for exces sky radiance blocking by shadowband
      parameter (cimelaot=.false.)	! Not use CIMEL AOT in DD fit
c     parameter (cimelaot=.true.)	! use CIMEL AOT in DD fit
      real alpha

      real gas2maxh,gas2width,gas2tot   ! gas2tot is obtained from the input UV-MFRSR-CIMEL merged file
      parameter (gas2maxh=0.0)		! Gas2 (NO2) constant mixing profile in the boundary layer
      parameter (gas2width=800.0)	! top of the boundary layer [mb], constant mixing profile

      logical mie_only
      parameter (mie_only=.false.) 
      logical gsfc_psd_clim,gsfc_refr_clim,ref_440

      parameter (gsfc_psd_clim=.false.,gsfc_refr_clim=.false.)
c---  if gsfc_refr_clim=.true. then ref_440 does not matter
      parameter (ref_440      =.true. )  ! Use constant value of ref_440 ?
                                         ! use linear exrapolation (440,670)

      integer iskip,max_k_iter,n_iter(3)
      parameter (iskip=0,max_k_iter=6)	! skip 0 raws in addition to 5 header raws
c     parameter (iskip=0,max_k_iter=0)	! No mfrsr fitting                        
c     parameter (iskip=16880,max_k_iter=6)	! skip 0 raws in addition to 5 header raws
      real fit_tolerance
      parameter (fit_tolerance=0.0001)  ! absolute DD fit tolerance
      real err_dd(3),dd_new(3)  

      integer i,j,k,nstr,maxnum,iaertype,nlayer
      integer iaermod,isurfmodel,icount,iozpr,nozpr
      integer jj,jheader,nvza,nphi
c                vert   azimuth 
      parameter (nvza=6,nphi=7)	!  max dimensions of vza and phi arrays
      parameter (icount=1)	!  just 368nm (see coe.dat and gas2.coe files)

      real sza,szab(10),vza(nvza),phi(nphi),vzatom,phitom,vzar,phir
      real lambda(5000),wave
      real aermaxh,aerwidthh,lat
      real wavea,lamalb(12),surfalb(12)
      real*8 surfpres,ozone(11),ozoneb(11),temp(11)
      real refrr1(12),refri1(12),refrr2(12),refri2(12),lam(12)
      real sigma1,rmean1,Nc1,sigma2,rmean2,Nc2
      real tau440,totoz,taumd,taur,tauc,pi,dt      
      real tau_real(1000),om_real(1000)
      real n_real(1000),k_real(1000)
      real ratio(5000,nvza,nphi),taum(12),omega(12)
      real rad_rayl(5000,nvza,nphi)
      real e_dir_ray(50),e_dir_ratio(50),e_dif_ray(50),e_dif_ratio(50)
      real e_dir(50),e_dif(50),dd_calc(50)

c---    aeronet AOT measurements
      integer iaeronet                          
      real aeronet_aot368,aeronet_aot332,aeronet_aot325               
      real aot340m,aot380m,aot440m,aot500m,aot670m,aot870m,aot1020m
c---    aeronet retrievals       
      real oleg_aot340,oleg_aot380,oleg_aot440,oleg_aot670,oleg_aot870
      real oleg_aot1020
      real oleg_w440,oleg_w670,oleg_w870,oleg_w1020    
      real oleg_dayofyear
      integer oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss
c---   aeronet size distribution parameters appoximated as bi-modal log-normal distribution
      real lnsf,rvf,vf,lnsc,rvc,vc  ! Volumne size distribution parameters (AERONET)
      real      rnf,nf,     rnc,nc  ! Number  size distribution parameters (ARIZONA)
      real  rr440,rr670,rr870,rr1020 ! Oleg's refractive index
      real  ri440,ri670,ri870,ri1020 ! Oleg's refractive index

c---   MFRSR measurements
      integer imfrsr,n_meas		! number of mfrsr measurements for a given AERONET inversion
      real mfrsr_dump(29)	! mfrsr_dump with added NO2 information
      real mfrsr_aot(3),mfrsr_lam(3), 
     *mfrsr_tot(3),mfrsr_dif(3),mfrsr_dirn,mfrsr_v0raw
      real mfrsr_dd(3),mfrsr_dt(3)
      real meas_par,mfrsr_dir_hor
c123456789012345678901234567890123456789012345678901234567890123456789012
      data mfrsr_lam/0.368,0.332,0.325/
      real mfrsr_tr,mfrsr_sza,mfrsr_ozone,md12_pressure
      real mfrsr_no2_atmcm,mfrsr_errno2DU,mfrsr_dtno2,
     *     mfrsr_toz,mfrsr_ldir
      real mfrsr_hour,mfrsr_min,mfrsr_sec      

      character*256 aeronethead
      character*580 header
      character*90 fin,fout   
      character*90 str(24),buf(8)
      character*15 fdata
      character*25 buff,faerosol
      character*37 buftom
      character*4 ozprof
      character*7 aers
      character*1 surfs,ozlat
      character*14 environm
      
      parameter (nstr=18,nozpr=26)	! number of dump rows in arizona output files

      data vza/0.0,15.0,30.0,45.0,60.0,75.0/
      data phi/0.0,30.0,60.0,90.0,120.0,150.0,180.0/
      data szab/0.0,30.0,45.0,60.0,70.0,77.0,81.0,84.0,86.0,88.0/
c--   V7 325 ozone & Temp profile
c     data ozoneb/16.0,14.0,26.0,45.0,74.7,66.9,41.7,24.5,11.1,3.7,1.4/
c     data temp/273.0,239.0,219.1,216.6,217.0,220.8,227.6,239.4,
c    &253.6,263.9,262.6/
c--   V7 275 ozone & Temp profile
      data ozoneb/16.0,12.0,15.0,29.0,58.0,63.7,40.6,24.5,11.1,3.7,1.4/
c     data ozoneb/265.0,1.,1.,1., 1.,1.,1., 1.,1.,1.,1. /
      data temp/273.0,239.0,217.1,212.2,214.9,220.4,227.6,239.4,
     &253.6,263.9,262.6/


      jheader=1

c     -- open output file
 	open(28,file='./convergence.dt368',status='unknown')
      open (25,file='DATA/output.dt440.no2_20060724_v1_test3',           
c     open (25,file='DATA/output.dt440.no2_20060724_v1_test2',           
c     open (25,file='DATA/output.dt440.no2_20060724',           
     *status='unknown')

c23456789012345678901234567890123456789012345678901234567890123456789012
      header='oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss,oleg_
     *dayofyear,aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,aot340m
     *,oleg_aot440,oleg_aot670,oleg_aot870,oleg_aot1020,oleg_w440,oleg_w
     *670,oleg_w870,oleg_w1020,rr440,rr670,rr870
     *,rr1020,ri440,ri670,ri870,ri1020,vf,vc, rvf,rvc, lnsf,lnsc,
     *int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec),mfrsr_sza,mfrsr_ozon
     *e,surfpres,lam,n_real,k_real,tau_real,om_real,rad_rayl(i,1,1),
     *ratio(i,1,1),e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio,
     *calc_par,meas_par,mfrsr_errno2DU,mfrsr_dtno2'

        write(25,*) header
      
C----------set up orbit and environment models

c---   Surface albedo model
      isurfmodel=6		! albedo=0.0543 at 440nm from AERONET albedo on 07/24/2006
      call surfmodel(isurfmodel,lamalb,surfalb)
	write(*,*) 'lambda,  spec albedo'
	write(*,'(2f8.3)') (lamalb(i),surfalb(i),i=1,12)


c---	AERONET aerosol model
c     open(10,file='inp.dat',status='old')
c     read(10,*) iaermod,tau440,aermaxh
c     read(10,*) iozpr
c     close(10)

c	iaermod=2
        iaertype=4   ! only bi-modal log-normal PSD accepted with AERONET data


      open(17,file='./clear440_20060724_v1_test3', 		!  test data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
c     open(17,file='./clear440_20060724_v1_test2', 		!  test data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
c     open(17,file='./clear440_20060724_v1',       		!  test data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
c     open(17,file='./clear440_20060724test_v1',   		!  test data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
     &     form='formatted',status='old',err=900)
      write(6,*)              'Reading AERONET-MFRSR retrievals'  ! reading header
      do iaeronet=1,iskip+5
       read(17,"(1a256)",ERR=901) aeronethead
      enddo
       write(6,*) aeronethead


  100   continue        !   MAIN LOOP THOUGH ALL AERONET-MFRSRS DATA
        write(*,*)
        write(*,*)
        write(*,*)
	write(6,*) 'start reading new AERONET retrieval'

        read(17,*,err=901,end=210) aeronet_dump
        write(6,*) aeronet_dump

	oleg_dd=aeronet_dump(1)
	oleg_month=aeronet_dump(2)
	oleg_yyyy=aeronet_dump(3)
	oleg_hh=aeronet_dump(4)
	oleg_mm=aeronet_dump(5)
	oleg_ss=aeronet_dump(6)
        oleg_dayofyear=aeronet_dump(7)
        aot1020m  =aeronet_dump(8)
        aot870m  =aeronet_dump(9)
        aot670m  =aeronet_dump(10)
        aot500m  =aeronet_dump(11)
        aot440m  =aeronet_dump(12)
        aot380m  =aeronet_dump(13)
        aot340m  =aeronet_dump(14)
c       oleg_aot340   =aeronet_dump(15)	! dummy parameters
c       oleg_aot380   =aeronet_dump(16)	! dummy parameters

        oleg_aot440   =aeronet_dump(19)
        oleg_aot670   =aeronet_dump(20)
        oleg_aot870   =aeronet_dump(21)
        oleg_aot1020   =aeronet_dump(22)

        oleg_w440     =aeronet_dump(32)
        oleg_w670     =aeronet_dump(33)
        oleg_w870     =aeronet_dump(34)
        oleg_w1020     =aeronet_dump(35)

        rr440		=aeronet_dump(57)
        rr670		=aeronet_dump(58)
        rr870		=aeronet_dump(59)
        rr1020		=aeronet_dump(60)

        ri440		=aeronet_dump(61)
        ri670		=aeronet_dump(62)
        ri870		=aeronet_dump(63)
        ri1020		=aeronet_dump(64)

        vf    		=aeronet_dump(103)
        vc    		=aeronet_dump(107)
        rvf    		=aeronet_dump(105)
        rvc    		=aeronet_dump(109)
        lnsf    		=aeronet_dump(106)
        lnsc    		=aeronet_dump(110)

      write(6,*) oleg_dayofyear

      call get_aer_mf_v2(real_refr_const,real_refr_value,
     *gsfc_psd_clim,gsfc_refr_clim,
     *ref_440,aeronet_dump,aeronet_dump_length,
     *lam,refrr1,refri1,refrr2,refri2,
     *sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

c	!!! make sure aeronet_dump length is the same as in main program!
c     call get_aeronet(gsfc_psd_clim,gsfc_refr_clim,
c    *ref_440,aeronet_dump,  ! calc parameters for Arizona code
c    *lam,refrr1,refri1,refrr2,refri2,
c    *sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

c     iaertype=4
c     if(iaermod.eq.17) iaertype=2
c     call aermodel(iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
c    &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

	 write(*,*) 'Nc1 Nc2',Nc1,Nc2
	 write(*,*) 'sigma1,rmean1,sigma2,rmean2',sigma1,rmean1,sigma2,rmean2
	 write(*,*) 'lambda,  real,   imaginary index'
	 write(*,'(5f10.4)') (lam(i),refrr1(i),refri1(i),refrr2(i),
     &refri2(i),i=1,12)

c---	mfrsr measuremets
	read(17,*) n_meas		! get number of MFRSR measurements for a given almucantar retrieval
        write(6,*) n_meas

      do imfrsr=1,n_meas		! main cycle of MFRSR retrievals

 	read(17,*,err=902) mfrsr_dump
      write(6,*) mfrsr_dump

c		Aerosol optical Thickness

c     mfrsr_aot(3) 	=0.353            	! 325nm: Oct 1 2002, 14:36
c     mfrsr_aot(2) 	=0.340            	! 332nm: Oct 1 2002, 14:36
      mfrsr_aot(1) 	=mfrsr_dump(21)		! 368nm

      if(cimelaot) then	!	Using interpolated CIMEL AOT instead of MFRSR AOT
        write(6,*) 'Using CIMEL AOT !!!'
        write(28,*) 'Using CIMEL AOT !!!'
      mfrsr_aot(1) =mfrsr_dump(18)              ! interpolated to 368nm AERONET AOT
      mfrsr_aot(2) =aot340m*1.02410**alpha      ! 332nm      
      mfrsr_aot(3) =aot340m*1.04620**alpha      ! 325nm      
      endif

	if (mfrsr_aot(1) .lt. 0.0) then
	write(6,*) 'front_mfrsr: NEGATIVE MFRSR AOT'
        write(6,*) 'Skip to the next measurement:'
        goto 101
	endif


      mfrsr_tot(1)	=mfrsr_dump(8)		! USDA Total V at 368nm: !! make sure to change if either mfrsr_dif or mfrsr_dirn is changed!
      mfrsr_dif(1)	=mfrsr_dump(9)		! Diffuse V at 368nm
      if(mdif_corr) then			! Using corrected MFRSR diffuse 
      mfrsr_dif(1)=mdif_corr_cos*mfrsr_dump(9)		! increase diffuse Voltage           
      endif
c     mfrsr_dif(3)	=0.165811         	! 325nm: Oct 1 2002, 14:36 NOT USED !
c     mfrsr_dif(2)	=0.211896         	! 332nm: Oct 1 2002, 14:36 NOT USED !

      mfrsr_dirn 	=mfrsr_dump(10)		! Direc_normal V at 368nm

      mfrsr_hour  	=mfrsr_dump(5)    	! hour of the  3 min averaging interval
      mfrsr_min   	=mfrsr_dump(6)    	! minute of 3 min averaging interval
      mfrsr_sec   	=mfrsr_dump(7)    	! sec of 3 min averaging interval
      mfrsr_sza   	=mfrsr_dump(13)   	! SZA at the end of 3 min averaging interval, [deg]
      pi=4.0*atan(1.0)
      dt=pi/180.0
      mfrsr_tot(1)	=mfrsr_dirn*cos(dt*mfrsr_sza)+mfrsr_dif(1)	! re-calculate total V at 368nm after mfrsr_dif correction !!!
c     mfrsr_tot(3)	=0.222590         	! 325nm: Oct 1 2002, 14:36
c     mfrsr_tot(2)	=0.293774              	! 332nm: Oct 1 2002, 14:36


c				Effective monochromatic wavelength and spectral constants for the channel (former coe.dat and gas2.coe)
      mfrsr_ldir	=mfrsr_dump(25)		!   mfrsr_ldir is radiatively effective wavelength (direct irradiance)
      lambda(1) =  10.*mfrsr_ldir		!  dynamic mfrsr_ldir from mfrsr_dump(25)

      SELECT CASE (INT(mfrsr_ldir))
	CASE (438:442)
      		xno2=1.3380e+01				! NO2 x-section at 440nm [atm_cm-1] rectangular slit FWHM=10nm, T=294K
      		rhob= 2.9100e-02				! Rayleigh depolarization factor at 440nm 
	CASE (364:370)
      		xno2=1.4765e+01				! NO2 x-section at 368nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0100e-02				! Rayleigh depolarization factor at 368nm 
	CASE (330:334) 
      		xno2=8.8113e+00				! NO2 x-section at 332nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0600e-02				! Rayleigh depolarization factor at 332nm
	CASE (323:327)
      		xno2=7.5192e+00				! NO2 x-section at 325nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0867e-02				! Rayleigh depolarization factor at 325nm
	CASE (315:319)
      		xno2=6.2748e+00				! NO2 x-section at 317nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1321e-02				! Rayleigh depolarization factor at 317nm
	CASE (309:313)
      		xno2=5.2258e+00				! NO2 x-section at 311nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1628e-02				! Rayleigh depolarization factor at 311nm
	CASE (303:307)
      		xno2=4.2322e+00				! NO2 x-section at 305nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1700e-02				! Rayleigh depolarization factor at 305nm
	CASE (298:302)
      		xno2=3.4081e+00				! NO2 x-section at 300nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1726e-02				! Rayleigh depolarization factor at 300nm 
 	CASE DEFAULT                         
 	write(*,*) 'mfrsr_ldir is outside UV-MFRSR spectral channels: ',
     * mfrsr_ldir
      	stop
      END SELECT

c					O3 optical thickness and effective dynamic x-section
c	define effective ozone x-sec (c0b) that will produce exactly the same tau_ozone (mfrsr_toz) subtraction used in calibration: mfrsr_dump(23)  
c        c0b,c1b=0,c2b=0,rhob and pass them to gs_fit routine
      mfrsr_ozone 	=mfrsr_dump(16)   	! total ozone , Brewer [DU] ( fill in with EP/TOMS data): !! no fill in values allowed !!
      mfrsr_toz		=mfrsr_dump(23)		!   mfrsr_toz is ozone tau                                
      c0b=1000.*mfrsr_toz/mfrsr_ozone		!  effective dynamic ozone x-sections [atm-cm-1]
      c1b=0.0000  				! neglect Temperature dependence of ozone absorption
      c2b=0.000 				! neglect Temperature dependence of ozone absorption

c					NO2 optical thickness
      mfrsr_no2_atmcm 	=0.001*mfrsr_dump(27)	!  NO2 total amount [atm*cm]
      mfrsr_errno2DU 	=mfrsr_dump(28)	!  NO2 expanded uncertainty [DU] total amount [atm*cm]
      mfrsr_dtno2	=mfrsr_dump(29)	!  minimal time difference between NO2 and MFRSR measurements [min]
	if ( (mfrsr_no2_atmcm .lt. 0.0 ) .or. 
     *       (mfrsr_errno2DU  .gt. 0.5)  .or. 
     *       (mfrsr_dtno2 .gt. 30.)) 		then
	write(6,*) 'no2_corr: no NO2 measurements',mfrsr_no2_atmcm,
     *              mfrsr_errno2DU,mfrsr_dtno2
        write(6,*) 'Skip to the next measurement:'
        goto 101
	endif
      tau_no2= mfrsr_no2_atmcm*xno2		!  NO2 effective optical thickness (rectangular, FWHM=2nm, T=294K )
      write(6,*) 'NO2 [DU], x_no2',1000.0*mfrsr_no2_atmcm, xno2

      if (no2_corr) then 			!   reduce AOT by tau_NO2           
      mfrsr_aot(1) 	=mfrsr_dump(21)- tau_no2              ! reduce AOT by tau_NO2         
c	mfrsr_no2_atmcm and xno2 are passed to gs_fit to add as gas2 absorber

	if (mfrsr_aot(1) .lt. 0.0) then
	write(6,*) 'no2_corr: tau_no2 > mfrsr_aot',mfrsr_aot(1),tau_no2
        write(6,*) 'Skip to the next measurement:'
        goto 101
	endif


	endif		! end NO2 correction

      md12_pressure	=mfrsr_dump(15)     	!  pressure measurement at GSFC roof bld 33 [mbars]  NOT USED:  keeping Psurf=1013.25mb as input to Arizina code
      aeronet_aot368	=mfrsr_dump(18)     	!  interpolated AERONET AOT368                               
      mfrsr_tr		=mfrsr_dump(22)		!   mfrsr_tr is scaled for the actual surface pressure and l_dir, keeping Psurf=1013.25mb as input to Arizina code

        if (mfrsr_sza   .gt. 78.0) then
        write(6,*) mfrsr_hour,mfrsr_min,mfrsr_sec,mfrsr_sza
        write(6,*) 'front_mfrsr: mfrsr_sza>78 deg'
        write(6,*) 'Skip to the next MFRSR measurement:'
        goto 113		! skip to the next MFRSR record
        endif		! end check SZA>78

c---	V0 correction: V0_corr=V0*k at 368nm => dtau=myu*ln(k), ln(k) from CIMEL comparisons

      if(mtau_corr) then	!	Using corrected MFRSR AOT
        write(6,*) 'Using corrected mfrsr 368 AOT: dtau=myu*ln(k), 
     * ln(k)=',mean_lnk
        write(28,*) 'Using corrected mfrsr 368 AOT: dtau=myu*ln(k), 
     * ln(k)=',mean_lnk
      mfrsr_aot(1) =mfrsr_aot(1)     +mean_lnk*cos(pi*mfrsr_sza/180.) 
      endif


c---   Calculation of Diffuse/Direct ratio
      do i=1,1
        mfrsr_dir_hor=mfrsr_tot(i) - mfrsr_dif(i)
	if (abs(mfrsr_dirn   ) .lt. 0.0001) then
        mfrsr_dd(i)= - 9999.999
        mfrsr_dt(i)= - 9999.999
        mfrsr_tot(i) = - 9999.999
        mfrsr_dif(i) = - 9999.999

        else
	mfrsr_dd(i) = mfrsr_dif(i)/mfrsr_dirn    ! normalize to dir_normal
	mfrsr_dt(i) = mfrsr_dif(i)/mfrsr_tot(i)  ! normalize to total_hor 
        endif
      enddo

c--- Calculate MFRSR total and diffuse transmittances
      mfrsr_v0raw	=mfrsr_dump(11)			! V0_raw at 368nm
      mfrsr_tot(1)	=mfrsr_dump(8)/mfrsr_v0raw		! Total transmittance
      mfrsr_dif(1)	=mfrsr_dump(9)/mfrsr_v0raw		! Diffuse  transmittance
      mfrsr_dirn        =mfrsr_dump(10)/mfrsr_v0raw	! direct normal transmittance

	if(mie_only) go to 102		! no irradiance calculation

c---	vertical structure model
      nlayer=100
      aerwidthh=1.5
      aermaxh=0.0

c!!!  Keep surface pressure to 1 atm, until Arizona code will be modified not to scale tau_aer
c!!!  Scal tau Rayleigh = mfrsr_tr                                                             
      surfpres=1013.25


c---    ozone vert profile model
	iozpr=0		!iozpr=0; if iozpr=1 then enhanced trop. ozone
      totoz= 0.001*mfrsr_ozone    ! Arizona input total ozone [atmcm] or NO2 (substitution)

      do i=1,11
      ozone(i)=ozoneb(i) 	! ozone profile in DU/Umkehr
      enddo

      if(iozpr.eq.1) then
c     totoz=totoz+0.5*ozone(1)/1000.0
      ozone(1)=1.5*ozone(1)
      endif
      write(ozprof,'(i3,''M'')') int(totoz*1000.0)
	write(*,*) 'ozone profile  ',ozprof
c      call ozonemodel(ozprof,ozone,temp)
	write(*,*) 'umkehr layer,   ozone,   temperature'
	write(*,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11)

      
      
C----------call a subrotine interpolating the TOMRAD ozone and temperature
C----------profiles

      call profiles(1,surfpres,ozone,temp)

C----------call a subroutine adding an aerosol profile and gas2 profile to be used for
C----------Gauss-Seidel computations

c     gas2tot=0.001		! gas2  column amount [atm-cm] : gas2tot=0.001 for 1DU of SO2
      gas2tot=mfrsr_no2_atmcm	! NO2  column amount [atm-cm] 

      call aerprof(aermaxh,aerwidthh,gas2maxh,gas2width,gas2tot,nlayer)


c---	Cloud parameters
      tauc=0.0		!  NO CLOUDS

      
C-----------end the orbit and environmental models

 102  continue		! skip here if Mie only calculations

c     do jj=1,10		!  loop through SZA
      do jj=1,1 
c     sza=szab(jj)
      sza=mfrsr_sza              ! MFRSR SZA 

      tau440=aeronet_dump(12)      !  AERONET measurement at 441nm
      write(*,*) 'tau440=',tau440


      call gs_fit(xno2,c0b,c1b,c2b,rhob,
     *max_k_iter,dd_fit,dt_fit,t_fit,d_fit,
     *fit_tolerance,log_fit,mie_only,
     *totoz,gas2tot,surfpres,mfrsr_tr,   
     &mfrsr_lam,mfrsr_dd,mfrsr_dt,mfrsr_tot,mfrsr_dif,mfrsr_aot,
     &tau440,tauc,sza,vza,phi,nvza,nphi,
     &iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
     &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,isurfmodel,lamalb,surfalb,
     &icount,lambda,ratio,taum,omega,tau_real,om_real,rad_rayl,
     & n_real,k_real,e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio,
     *dd_new,n_iter,err_dd,meas_par )

      call system ('clean')

      
      do i=1,icount	! icount < 50
     	if (e_dir_ray(i) .eq. 0.0) go to 100
	e_dir(i)=e_dir_ray(i)*e_dir_ratio(i)
        e_dif(i)=e_dif_ray(i)*e_dif_ratio(i)
       	if (e_dir(i) .lt. 0.00001 ) then
      dd_calc(i)=  -9999.999       
	else
      dd_calc(i)=e_dif(i)/e_dir(i)
	endif

      enddo
      
 	if(mie_only) go to 101

C------------open radiance output file

	if ( no_radiance_output ) go to 101

       if(jheader.eq.1) then
      jheader=2
      if(iaermod.lt.10) then
        write(aers,'(a,i1,''t'',f4.2)') '0',iaermod,tau440
      else
        write(aers,'(i2,''t'',f4.2)') iaermod,tau440
      endif
      write(surfs,'(i1)') int(aermaxh)
      environm=ozprof//'_'//aers//'_'//surfs
      write(fout,'(''DATA/faer_en'',a,''.dat'')') environm
	WRITE(*,*) 'output filename',fout
      open(11,file=fout,status='unknown')

C-------------write a header

      write(11,'(a)') 'ENVIRONMENTAL VARIABLES'
      write(11,'(a,a)') 'TOMS OZONE & TEMPERATURE PROFILES ',ozprof
      write(11,'(a)') 'umkehr layer,   ozone,   temperature'
      write(11,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11) 
      write(11,'(a,i2,a,f6.3)') 'AEROSOL MODEL',iaermod,' tau440=',
     &tau440
      write(11,'(a,f6.2,a,f6.2,a,e8.2,a,f6.2,a,f6.2,a,e8.2)') 'log-normal 
     & distr: rmean1, ',rmean1,' sigma1 ',sigma1,' Nc1 ',Nc1,
     &' rmean2 ',rmean2,' sigma2 ',sigma2,' Nc2 ',Nc2
      write(11,'(a)') 'wavelength (mu)  real,  imager,           optical 
     & thickness,   omega0'
      write(11,'(f8.3,4e12.3,2f12.3)') (lam(i),refrr1(i),refri1(i),
     &refrr2(i),refri2(i),taum(i),omega(i),i=1,12)      
      write(11,'(f8.3,2f14.5)') (lambda(i),
     &tau_real(i),om_real(i),i=1,12)      

      write(11,'(a,f5.1,a,f5.1)') 'PROFILE: height=',aermaxh,' scale=',     
     &aerwidthh 
      write(11,'(a,f8.1)') 'SURFACE PRESSURE, mb ',surfpres
      write(11,'(a,i2)') 'SURFACE MODEL ',isurfmodel
      write(11,'(a)') ' wavelength, mu      albedo'
      write(11,'(f8.3,f12.3)') (lamalb(i),surfalb(i),i=1,12)
      write(11,*)

      endif

      do j=1,nvza
      do k=1,nphi
      write(11,'(a)') 'GEOMETRY'
      write(11,'(a,f7.2,a,f7.2,a,f9.2)') ' sza =',sza,' vza =',vza(j),
     &' phi =',phi(k)
      write(11,'(a)') 'WAVE (A),  sky_rad_rayl 		aer/ray ratio' 

C--------------write data

      do i=1,icount		!  icount < 50
        write(11,'(3e13.5)') lambda(i),rad_rayl(i,j,k),ratio(i,j,k)
      enddo

      enddo
      enddo

      enddo	! end sza loop

 101	continue	! skip radiance calculations


c---	write flux and MFRSR results
      write(6,*) 'DEBUG!!!: icount,mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn
     *,mfrsr_v0raw'
      write(6,*)icount,mfrsr_tot(1),mfrsr_dif(1),mfrsr_dirn,mfrsr_v0raw

	write(25,8)
     &oleg_dd,oleg_month,oleg_yyyy, 		! UT day #, month #      
     &oleg_hh,oleg_mm,oleg_ss,oleg_dayofyear, 	! UT hh, min,sec, decimal doy   #      
     &aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,aot340m, ! measured aot 
c    &oleg_aot340,oleg_aot380,
     &oleg_aot440,oleg_aot670,oleg_aot870,
     &oleg_aot1020, 						! OLeg aot 
     &oleg_w440,oleg_w670,oleg_w870,oleg_w1020,			! OLeg W 
     &rr440,rr670,rr870,rr1020,					! OLeg Real refr
     &ri440,ri670,ri870,ri1020,				 	! OLeg Im refr
     & vf,vc, rvf,rvc, lnsf,lnsc  				! PSD: fine and coarse mode
     *,int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec) 
     *,mfrsr_sza,mfrsr_ozone,1000.*mfrsr_no2_atmcm,surfpres 
     *,(0.1*lambda(i),n_real(i),k_real(i), 
     *   tau_real(i),om_real(i),aeronet_aot368,
     * mfrsr_tr,mfrsr_toz,mfrsr_ldir,
     * mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw,	    ! mfrsr measurements of tot/V0, dif/V0 and dir_norm/V0 components
     *rad_rayl(i,1,1),ratio(i,1,1),e_dir_ray(i),e_dir_ratio(i),
     *e_dif_ray(i),e_dif_ratio(i),
     *dd_new(i),meas_par,n_iter(i),i=1,icount),
     *mfrsr_errno2DU,mfrsr_dtno2 


c23456789012345678901234567890123456789012345678901234567890123456789012
 8    format(             
     &       2(1x,I2), 		!UT day #,   UT month #
     &         1x, I4,         ! UT year  #
     &       3(1x, I2),         ! UT hour  #
     &         1x, f10.6,      ! decimal UT day of year
     &       7(1x,f9.6),2x,    ! aeronet measured aot @ 1020, 870nm,670nm,500nm,440nm,380nm,340nm
c    &       2(1x,f5.3),2x,    ! OLeg aot @ [340, 380,]                            
     &       4(1x,f5.3),2x,    ! OLeg aot @ 440, 670, 870 and 1020                           
     &       4(1x,f4.2),2x,    ! OLeg W @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg n @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg k @ 1020, 870nm, 670,440                                   
     &       6(1x,f9.6),2x,   !  Log-Norm PSD parameters
     &       3(i2,1x),         ! int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec)
     &       f5.2,1x,2(f5.1,1x),f7.2,2x,  ! mfrsr_sza, mfrsa_ozone, 1000.*mfrsr_no2_atmcm, surfpres
c      lam       (n,k)  (mt,w,aer_t368)
     & f6.1,1x,2(f8.6,1x),3(f5.3,1x),2x,2(f8.6,1x),f8.4,2x,
c      t_tot,t_dif,t_dir V0
     & 3(f7.5,1x),1(f7.2,1x),3x,			!   mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw
     & 6(e13.5,1x), 2(f9.4,1x),3x,i2,2(1x,f5.2))    

	write(6,*) 'mfrsr measurement processed: '

 113    continue        ! mfrsr measurement not processed because mfrsr_sza> 78deg

        enddo 		! continue mfrsr data loop for single AERONET inversion

	write(6,*) 'one aeronet inversion done !'

     
      go to 100   	! return to read new AERONET retrieval


 210  continue        ! end of AERONET file encountered 
      write(6,*) 'end of AERONET file encountered !!'

C------------close all files       
     
      close(17)
      close(11)
      close(25)
      close(28)

      stop

c-- error handling
 
  900  continue
       write(6,*) 'front_end: could not open merged AERONET/MFRSR file '
       stop

  901  continue
       write(6,*) 'front_end: error while reading data in 
     &         AERONET file '
       stop
  902  continue
       write(6,*) 'front_end: error while reading data in 
     &         MFRSR file '
       stop
 
       end
