      program  FRONT_AERONET           
C***********************************************************************
C	AUTHOR: Nickolay Krotkov and Alexander P. Vasilkov
C
C	FUNCTION:
c     To fit imaginary refr index 
C     using AERONET PSD, real refr index and MFRSR diffuse and total irradiance  measurenets
C	
C       WARNING
c       8) MFRSR AOT need to be corrected for the difference between AERONET assumed NO2 and actual measured NO2
c       7) Temperature and NO2 profiles are not correct                         
c       6) The program can not handle SZA=??.?5 because of filename trucation
c	temporal fix:   put fill values in the output: SSA=-999
c       5) Surface albedo at all wavelengths = AERONET V2 albedo at 440nm !! (underestimate SSA)
C       1) Only one wavelength version (lambda(1) and icount=1 ONLY !!! )                      
C       re-calculated at each wavelength !
C       2) only bi-modal log-normal PSD
C       3) Refractive index is the same for both modes (one component model)
C        4) surface pressure should be 1 atm=1013.25 in both flat.in and pressure.prf
C           (otherwise tau_aer is changed)! Pressure in flat.in can not be larger than
C          surface pressure in pressure.prf file.
C
C	HISTORY: 
c     Modified jul 14   , Increase diffuse transmittance by 30%                                                                                
c				parameter (mtau_corr=.false.,mdif_corr=.true.)    ! correct measured diffuse transmittance DIF ( for sensitivity study)
c     				parameter (mean_lnk=+0.08,mdif_corr_cos=1.30)
c     Modified jul 09   , Increase diffuse transmittance by 20%                                                                                
c     Modified aug 25   , New AERONET retrieval on 09/27                                                                                       
c     Modified aug 09   , surf albedo 0.01 at 317nm  2008 Santa Cruz , Bolivia                                                                 
c     Modified aug 04   , 2008 Santa Cruz , Bolivia                                                                 
c     Modified jul 15   , 2008 Decrease AERONET surf. albedo by 0.02 for sensitivty study (need removal)
c     Modified jul 11   , 2008 Remove unnecessary pressure scaling for tau_rau                      
c     Modified may 22    , 2008 To process Mexico campaign at 368nm                           
c     Modified Feb 13    , 2008 To re-process Santa Cruz campaign at 368nm                           
c     Modified Dec 13    , 2007 To re-process Santa Cruz campaign at 325nm                           
c     Modified Oct 15    , 2007 include AERONET clim NO2 corrections                                 
c     Modified Oct 13    , 2007 to re-process aeronet version 2 inversions at 440nm in Santa Cruz without NO2 corrections                                 
c     Modified Oct 13    , 2007 scale t_ray with surface pressure                         
c     Modified Oct 13    , 2007 set NO2=0 for fill values                                 
c     Modified May 11    , 2007 to re-process all aeronet version 2 inversions at 368nm with NO2 corrections                                 
c     Modified May 8     , 2007 to re-process all aeronet version 2 inversions at 440nm with NO2 corrections                                 
c     Modified July 15   , 2005 to re-process all inversions with NO2 corrections                                 
c     Modified January 9, 2005 to include ozone absorption and replace O2-O2 by NO2 in  water vapor absorption
c     Modified January 5, 2005 get rid of coe.dat file ( read lambda and x-sections from MFRSR input file)
c     Modified January 4, 2005 reduce AOT according to tau_NO2 at 368nm                          
c     Modified January 4, 2005 change ozone (NO2) profile to 1st Umkehr layer only               
c     Modified January 4, 2005 change total ozone to total NO2 (still ozone profile)              
c     Modified January 4, 2005 to add NO2 correction (using Brewer direct sun NO2 measurements)
c     Modified March 19, 2004 to add interpolated AERONET AOT to the output
c     Modified March 18, 2004 to compensate for ARIZONA code diffuse flux overestimation by 0.3%
c                             which is equivalent to enhancing measured diffuse by the same amount *1.003
c     Modified March 16, 2004 to fix bug with new format input file reading                      
c     Modified March 15, 2004 to re-calculate total after diffuse correction            
c     Modified March 14, 2004 to change diffuse due to additional diffuse cosine correction!
c     Modified March 11, 2004 to use CT fit: t_fit=.true.                          
c     Modified March 10, 2004 to  use dt fit tolerance=0.0001   and ipseudo=1                            
c     Modified March 10, 2004 to use diffuse/total fraction fit: dt_fit=.true.                          
c     Modified March 2, 2004 to  use absolute DD fit tolerance=  0.001                                  
c     Modified March 2, 2004 to read new input data format (AERONET+20mfrsr)                             
c     Modified Sep  28, 2003 to double   AERONET  k                                                
c     Modified Sep  26, 2003 to use only AERONET data  and no fitting                              
c     Modified July 30, 2003 to use MFRSR AOT correction at 368nm only !!!           
c     Modified July 30, 2003 to fix output format for RVC>10micron                    
c     Modified July 29, 2003 to use AERONET interpolated AOT at 368nm for SSA at 368nm
c     Modified July 18, 2003 to read new format AERONET/MFRSR data                         
c     Modified July 10, 2003 restrict fitting to SZA<70deg                                 
c     Modified July 8 , 2003 to use absolute DD fit tolerance=  0.01                       
c     Modified July 6 , 2003 to constrain number of k iterations: max_k_iter              
c     Modified July 6 , 2003 to process missing MFRSR AOT data for zero direct irradiance)
c     Developed June - July, 2003
C     template program: front_end_TOMRAD_GS_V6_440nm.f Developed by AV in June, 2000-June 2003
C***********************************************************************
      implicit none

      integer :: imfrsr_ldir
      integer aeronet_dump_length
      parameter (aeronet_dump_length=155)	! V2 AERONET inversions: c:\abstrats\AGU_JA07\memo_v2
      real aeronet_dump(aeronet_dump_length)	! AERONET web data only: 
						! it's also affects get_aer_mf_v2.f and get_aeronet subs !!!
      real c0b,c1b,c2b,rhob	! Ozone, and depolarization values are defined here and passed to gs_fit to get rid of coe.dat gas2.coe files
				! Rayleigh optical thickness is red from MFRSR input file
            
      logical no2_corr
c     parameter (no2_corr=.false.)	! no no2 correction
      parameter (no2_corr=.true.)	! no2 correction: adding tau_no2 without changing V2 AOT
      real xno2,tau_no2

      logical real_refr_const	! .TRUE. to use constant real refractive index (ignore AERONET retrieval)
      real real_refr_value
      parameter (real_refr_const=.false.,real_refr_value=1.4)	! set constant value for real refractive index
      logical dd_fit		! .T. use diffuse/direct fit of k
      logical dt_fit		! .T. use diffuse/total  fit of k
      logical  t_fit		! .T. use total          fit of k
      logical  d_fit		! .T. use  diffuse transmittance fit of k
      logical log_fit		! .T. use logarithms of dd and  k
      logical no_radiance_output

c---  choose fitting parameter:
c     parameter (dd_fit=.true.,dt_fit=.false.,t_fit=.false.)
      parameter (dd_fit=.false.,dt_fit=.false.,t_fit=.true.)
      parameter (d_fit=.false.,log_fit=.false.)
      parameter (no_radiance_output=.true.) 

c---  choose first multilpier for the first iteration 
      real firstmult_k
      parameter (firstmult_k=2.0)    	!  332nm and 368nm Initial iAERONET k multiplyer to start 1st iteration
c     parameter (firstmult_k=0.1)    	!  440nm and 368nm Initial iAERONET k multiplyer to start 1st iteration

c---  choose what corrections are applied to MFRSR measurements 
      logical    cimelaot,mtau_corr,mdif_corr
      parameter (mtau_corr=.false.,mdif_corr=.false.)	! do not correct MFRSR AOT and/or measured diffuse transmittance DIF 
c     parameter (mtau_corr=.false.,mdif_corr=.true.)	! correct MFRSR AOT and/or measured diffuse transmittance DIF ( for sensitivity study)
      real mean_lnk,mdif_corr_cos
      parameter (mean_lnk=+0.08,mdif_corr_cos=1.30)	! dtau=myu*<lnk>, increase measured diffuse irradiance to account for uncorrected exces sky radiance blocking by shadowband
      parameter (cimelaot=.false.)	! Not use CIMEL AOT in the flux forward calculation
c     parameter (cimelaot=.true.)	! use CIMEL AOT in DD fit
      real alpha

c---  corection for gaseous absorption
      real gas2maxh,gas2width,gas2tot   ! gas2tot (other than ozone) is obtained from the input UV-MFRSR-CIMEL merged file
      parameter (gas2maxh=0.0)		! Gas2 (NO2) constant mixing profile in the boundary layer
      parameter (gas2width=800.0)	! top of the boundary layer [mb], constant mixing profile

      logical mie_only
      parameter (mie_only=.false.) 
      logical gsfc_psd_clim,gsfc_refr_clim,ref_440

      parameter (gsfc_psd_clim=.false.,gsfc_refr_clim=.false.)	! .T. to use AERONET climatological values for PSD and refr. index
c---  if gsfc_refr_clim=.true. then ref_440 does not matter
      parameter (ref_440      =.true. )  ! Use constant value of ref_440 (real)  to extrapolate into UV wavelengths
                                         ! use linear exrapolation (440,670)

      integer iskip,max_k_iter,n_iter(3)
      parameter (iskip=0,max_k_iter=11)	! skip 0 raws in addition to 5 header raws; do max 6 iterations of k
c     parameter (iskip=0,max_k_iter=0)	! No mfrsr fitting                        
c     parameter (iskip=16880,max_k_iter=6)	! skip 0 raws in addition to 5 header raws
      real fit_tolerance
      parameter (fit_tolerance=0.004)  ! absolute parameter fit tolerance
c     parameter (fit_tolerance=0.001)  ! absolute parameter fit tolerance
c     parameter (fit_tolerance=0.0001)  ! absolute parameter fit tolerance
      real err_dd(3),dd_new(3)  

      integer i,j,k,nstr,maxnum,iaertype,nlayer
      integer iaermod,isurfmodel,icount,iozpr,nozpr
      integer jj,jheader,nvza,nphi
c                vert   azimuth 
      parameter (nvza=6,nphi=7)	!  max dimensions of vza and phi arrays
      parameter (icount=1)	!  just one wavelength calculation (determined by MFRSR input file)

      real sza,szab(10),vza(nvza),phi(nphi),vzatom,phitom,vzar,phir
      real lambda(5000),wave
      real aermaxh,aerwidthh,lat
      real wavea,lamalb(12),surfalb(12),salb440v2
      real*8 surfpres,ozone(11),ozoneb(11),temp(11)
      real refrr1(12),refri1(12),refrr2(12),refri2(12),lam(12)
      real sigma1,rmean1,Nc1,sigma2,rmean2,Nc2
      real tau440,totoz,taumd,taur,tauc,pi,dt      
      real tau_real(1000),om_real(1000)
      real n_real(1000),k_real(1000)
      real ratio(5000,nvza,nphi),taum(12),omega(12)
      real rad_rayl(5000,nvza,nphi)
      real e_dir_ray(50),e_dir_ratio(50),e_dif_ray(50),e_dif_ratio(50)
      real e_dir(50),e_dif(50),dd_calc(50)

c---    aeronet AOT measurements
      integer iaeronet                          
      real aeronet_aotchan             
      real aot340m,aot380m,aot440m,aot500m,aot670m,aot870m,aot1020m
c---    aeronet retrievals       
      real oleg_aot340,oleg_aot380,oleg_aot440,oleg_aot670,oleg_aot870
      real oleg_aot1020
      real oleg_w440,oleg_w670,oleg_w870,oleg_w1020    
      real oleg_dayofyear
      integer oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss
c---   aeronet size distribution parameters appoximated as bi-modal log-normal distribution
      real lnsf,rvf,vf,lnsc,rvc,vc  ! Volumne size distribution parameters (AERONET)
      real      rnf,nf,     rnc,nc  ! Number  size distribution parameters (ARIZONA RT)
      real  rr440,rr670,rr870,rr1020 ! Oleg's refractive index
      real  ri440,ri670,ri870,ri1020 ! Oleg's refractive index

c---   MFRSR measurements
      integer imfrsr,n_meas		! number of mfrsr measurements for a given AERONET inversion
      real mfrsr_dump(30)	! mfrsr_dump with added Brewer NO2 and AERONET NO2 information
      real mfrsr_aot(3),
     *mfrsr_tot(3),mfrsr_dif(3),mfrsr_dirn,mfrsr_v0raw
      real mfrsr_dd(3),mfrsr_dt(3)
      real meas_par
c123456789012345678901234567890123456789012345678901234567890123456789012
      real mfrsr_tr,mfrsr_sza,mfrsr_ozone,md12_pressure
      real aeronet_no2					! AERONET NO2 climatology
      real mfrsr_no2_atmcm,mfrsr_errno2DU,mfrsr_dtno2,
     *     mfrsr_toz,mfrsr_ldir
      real mfrsr_hour,mfrsr_min,mfrsr_sec      

      character*256 aeronethead
      character*580 header
      character*90 fin,fout   
      character*90 str(24),buf(8)
      character*15 fdata
      character*25 buff,faerosol
      character*37 buftom
      character*4 ozprof
      character*7 aers
      character*1 surfs,ozlat
      character*14 environm
      
      parameter (nstr=18,nozpr=26)	! number of dump rows in arizona output files

      data vza/0.0,15.0,30.0,45.0,60.0,75.0/
      data phi/0.0,30.0,60.0,90.0,120.0,150.0,180.0/
      data szab/0.0,30.0,45.0,60.0,70.0,77.0,81.0,84.0,86.0,88.0/
c--   V7 325 ozone & Temp profile
c     data ozoneb/16.0,14.0,26.0,45.0,74.7,66.9,41.7,24.5,11.1,3.7,1.4/
c     data temp/273.0,239.0,219.1,216.6,217.0,220.8,227.6,239.4,
c    &253.6,263.9,262.6/
c--   V7 275 ozone & Temp profile
      data ozoneb/16.0,12.0,15.0,29.0,58.0,63.7,40.6,24.5,11.1,3.7,1.4/ !	 default TOMS V7 clim ozone profile
c     data ozoneb/265.0,1.,1.,1., 1.,1.,1., 1.,1.,1.,1. /
      data temp/273.0,239.0,217.1,212.2,214.9,220.4,227.6,239.4,	!	default TOMS V7 clim temp profile
     &253.6,263.9,262.6/


      jheader=1

c     -- open output file
      open(28,file='./convergence.t332nm_gsfc',status='unknown')
c     open(28,file='./convergence.t440nm_gsfc',status='unknown')
c     open(28,file='./convergence.t440nm_gsfc_test',status='unknown')
c     open(28,file='./convergence.d440nm_gsfc_test',status='unknown')
c     open(28,file='./convergence.d440nm_0723_d1.3',status='unknown')
c     open(28,file='./convergence.dd332nm_0723',status='unknown')
c     open(28,file='./convergence.dt440nm_salb.01_927',status='unknown')
c     open(28,file='./convergence.dt317nm_salb.01',status='unknown')
c     open(28,file='./convergence.dt317nm_salbAER',status='unknown')
c     open(28,file='./convergence.dt368nm_salbAER',status='unknown')
c     open(28,file='./convergence.dt440nm_salbAER',status='unknown')

c     open(28,file='./convergence.dd311nm_salb020',status='unknown')
c     open(28,file='./convergence.dd317nm_salb020',status='unknown')
c     open(28,file='./convergence.dd325nm_salb020',status='unknown')
c     open(28,file='./convergence.dd332nm_salb025',status='unknown')
c     open(28,file='./convergence.dd332nm_0319',status='unknown')
c     open(28,file='./convergence.dd368nm_0319',status='unknown')

      open (25,file='DATA/output.t332nm_gsfc',            
c     open (25,file='DATA/output.t440nm_gsfc',            
c     open (25,file='DATA/output.t440nm_gsfc_test',            
c     open (25,file='DATA/output.d440nm_gsfc_test',            
c     open (25,file='DATA/output.d440nm_0723_d1.3',            
c     open (25,file='DATA/output.dt440.bolivia.v2.l15_salb.01_0927',        
c     open (25,file='DATA/output.dt317.bolivia.v2.l15_salb.01',        
c     open (25,file='DATA/output.dt317.bolivia.v2.l15',        
c     open (25,file='DATA/output.dt368.bolivia.v2.l15',        
c     open (25,file='DATA/output.dt440.bolivia.v2.l15',        

c     open (25,file='DATA/output.dd311.t1_salb_020',            
c     open (25,file='DATA/output.dd317.t1_salb_020',            
c     open (25,file='DATA/output.dd325.t1_salb_020',            
c     open (25,file='DATA/output.dd332.t1_salb_025',            
c     open (25,file='DATA/output.dd332.t1_0319',            
c     open (25,file='DATA/output.dd332.t1_l10_0319',            
c     open (25,file='DATA/output.dd368.t1_0319_1516UT',            
c     open (25,file='DATA/output.dd325.t1',        
c     open (25,file='DATA/output.dd332.t1',        
c     open (25,file='DATA/output.dd368.t1_salb_02',        
c     open (25,file='DATA/output.dd368.t1_0312',            
c     open (25,file='DATA/output.368nm.bolivia.v2.l15',        
c     open (25,file='DATA/output.317nm.bolivia.v2.l15',        
c     open (25,file='DATA/output.325nm.bolivia.v2.l15',        
c     open (25,file='DATA/output.dt440.bolivia.v2.l15',        
c     open (25,file='DATA/output.368nm.bolivia.v2.l15_20070911',        
c     open (25,file='DATA/output.368nm.bolivia.v2.l15_test',        
c     open (25,file='DATA/output.440nm.bolivia.v2.l15_20070911',        
c     open (25,file='DATA/output.440nm.bolivia.v2.l15_test',        
c     open (25,file='DATA/output.dt368.no2_v2',           
c     open (25,file='DATA/output.dt440.no2_v2',           
c     open (25,file='DATA/output.dt440.no2_20060724_v2',           
c     open (25,file='DATA/output.dt440.no2_20060724_v2_test',           
c     open (25,file='DATA/output.dt440.no2_20060724_v1_test2',           
c     open (25,file='DATA/output.dt440.no2_20060724',           
     *status='unknown')

c23456789012345678901234567890123456789012345678901234567890123456789012
      header='oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss,oleg_! V2 AERONET retrieval
     *dayofyear,aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,aot340m
     *,oleg_aot440,oleg_aot670,oleg_aot870,oleg_aot1020,oleg_w440,oleg_w
     *670,oleg_w870,oleg_w1020,rr440,rr670,rr870,rr1020,ri440,ri670,ri87
     *0,ri1020,vf,vc, rvf,rvc, lnsf,lnsc,hh,mm,sec,sza,ozone,surfpr,lam,
     *n_real,k_real,aot_mfrsr,ssa_mfrsr,aot_aer,tau_ray,tau_ozn,lam_dir,
     *trans_tot,trans_dif,trans_dirn,v0_raw, zenith_ray, zenith_ratio,
     *e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio,calc_par,meas_par,nit,
     *errno2DU,no2_delta_time'

        write(25,*) header
      
C----------set up orbit and environment models


        iaertype=4   ! only bi-modal log-normal PSD accepted with AERONET data


      open(17,file='./match30.332nm.gsfc.v2.l2_test',     ! 332nm:  GSFC : test                 
c     open(17,file='./match30.332nm.gsfc.v2.l2',     ! 332nm:  GSFC : test                 
c     open(17,file='./match30.440nm.gsfc.v2.l2',       	! 440nm:  GSFC : ALL                  
c     open(17,file='./match30.368nm.gsfc.v2.l2',       	! 440nm:  GSFC : ALL                  
c     open(17,file='./match30.440.gsfc_test',         	! 440nm:  GSFC : test                 
c     open(17,file='./match30.440.gsfc',         	! 440nm:  GSFC : test                 
c     open(17,file='./match30.440.scout03_20060723' ,	! 332nm:  SCOUT: increase Tdif by 20%

c     open(17,file='./match30.440nm.santa_cruz_utepsa_v2_l15_0927_1211',	! 368:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.317nm.santa_cruz_utepsa_v2_l15',	! 368:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.368nm.santa_cruz_utepsa_v2_l15',	! 368:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.440nm.santa_cruz_utepsa_v2_l15',	! 440:  AERONET V2 l15 data at Santa Cruz, Bolivia            

c     open(17,file='./match30.311nm.t1_max_mex_v2_l15',           ! 311nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.317nm.t1_max_mex_v2_l15',           ! 317nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.325nm.t1_max_mex_v2_l15',           ! 325nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.332nm.t1_max_mex_v2_l15',           ! 332nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.332nm.t1_max_mex_v2_l15_0319',           ! 332nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.332nm.t1_max_mex_v2_l10_0319',           ! 368nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.368nm.t1_max_mex_v2_l10_0319',           ! 368nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.368nm.t1_max_mex_v2_l15_0319',           ! 368nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.368nm.t1_max_mex_v2_l15_0312',           ! 368nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.368nm.t1_max_mex_v2_l15_0321_test',	! 368nm:  AERONET V2 l15 data at Mexico, T1 site                
c     open(17,file='./match30.368nm.santa_cruz_utepsa_v2_l15',	! 368nm:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.317nm.santa_cruz_utepsa_v2_l15',	! 317nm:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.325nm.santa_cruz_utepsa_v2_l15',	! 325nm:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.440nm.santa_cruz_utepsa_v2_l15',	! 440:  AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.368nm.santa_cruz_utepsa_v2_l15_20070911', 	! 20070911:368nm AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.368nm.santa_cruz_utepsa_v2_l15_test', 	! TEST:368nm AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.440nm.santa_cruz_utepsa_v2_l15_20070911',	! DAY: AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.440nm.santa_cruz_utepsa_v2_l15_test', 	! TEST: AERONET V2 l15 data at Santa Cruz, Bolivia            
c     open(17,file='./match30.440nm.gsfc.v2.l15_20060823', 	! AERONET V2 data at GSFC                                                                       
c     open(17,file='./match30.368.txt',          		!  368nm All ERONET V2 data from SCOUT03 campaign in Thessaloniki, Greece (~/UVMFRSR527/THES )            
c     open(17,file='./match30.440.txt',          		!  All ERONET V2 data from SCOUT03 campaign in Thessaloniki, Greece (~/UVMFRSR527/THES )            
c     open(17,file='./clear440_20060724_v2',     		!  All ERONET V2 data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
c     open(17,file='./clear440_20060724test_v2', 		!  AERONET V2 test data on Jul 24 2006 from Thessaloniki, Greece SCOUT03 campaign (~/UVMFRSR527/THES )            
     &     form='formatted',status='old',err=900)
      write(6,*)              'Reading AERONET-MFRSR retrievals'  ! reading header
c     do iaeronet=1,iskip+5
c      read(17,"(1a256)",ERR=901) aeronethead
c     enddo
c      write(6,*) aeronethead


  100   continue        !   MAIN LOOP THOUGH ALL AERONET-MFRSRS DATA
        write(*,*)
        write(*,*)
        write(*,*)
	write(6,*) 'start reading new AERONET retrieval'

        read(17,*,err=901,end=210) aeronet_dump
c       write(6,*) aeronet_dump

	oleg_dd=aeronet_dump(1)
	oleg_month=aeronet_dump(2)
	oleg_yyyy=aeronet_dump(3)
	oleg_hh=aeronet_dump(4)
	oleg_mm=aeronet_dump(5)
	oleg_ss=aeronet_dump(6)
        oleg_dayofyear=aeronet_dump(7)
C
        aot1020m =aeronet_dump(9)
        aot870m  =aeronet_dump(10)
        aot670m  =aeronet_dump(11)
        aot500m  =aeronet_dump(17)
        aot440m  =aeronet_dump(20)
        aot380m  =aeronet_dump(22)
        aot340m  =aeronet_dump(23)

        oleg_aot440   =aeronet_dump(25)
        oleg_aot670   =aeronet_dump(26)
        oleg_aot870   =aeronet_dump(27)
        oleg_aot1020   =aeronet_dump(28)

        oleg_w440     =aeronet_dump(38)
        oleg_w670     =aeronet_dump(39)
        oleg_w870     =aeronet_dump(40)
        oleg_w1020     =aeronet_dump(41)

        rr440		=aeronet_dump(47)
        rr670		=aeronet_dump(48)
        rr870		=aeronet_dump(49)
        rr1020		=aeronet_dump(50)

        ri440		=aeronet_dump(51)
        ri670		=aeronet_dump(52)
        ri870		=aeronet_dump(53)
        ri1020		=aeronet_dump(54)

        vf    		=aeronet_dump(94)
        vc    		=aeronet_dump(98)
        rvf    		=aeronet_dump(96)
        rvc    		=aeronet_dump(100)
        lnsf    		=aeronet_dump(97)
        lnsc    		=aeronet_dump(101)

      call get_aer_mf_v2(real_refr_const,real_refr_value,
     *gsfc_psd_clim,gsfc_refr_clim,ref_440,   		!  =.true. )  ! Use constant value of ref_440 (real)  to extrapolate into UV wavelengths
     *rr440,rr670,ri440,ri670,
     * vf,vc,rvf,rvc,lnsf,lnsc,
     * aot440m,
     *lam,refrr1,refri1,refrr2,refri2,
     *sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

	 write(*,*) 'Nc1 Nc2',Nc1,Nc2
	 write(*,*) 'sigma1,rmean1,sigma2,rmean2',sigma1,rmean1,sigma2,rmean2
	 write(*,*) 'lambda,  real,   imaginary index'
	 write(*,'(5f10.4)') (lam(i),refrr1(i),refri1(i),refrr2(i),
     &refri2(i),i=1,12)

c---   Surface albedo model
c       salb440v2  = 0.057                   !  Fix surf albedo at 368nm for T1 Max Mex site according to Sasha Madr. !!!! remove
c       salb440v2  = 0.010                   !  Fix surf albedo at 317nm for T1 Max Mex site according to Sasha Madr. !!!! remove
c       salb440v2  = 0.025                   !  Fix surf albedo at 332nm for T1 Max Mex site according to Sasha Madr. !!!! remove
c       salb440v2  = aeronet_dump(151)-0.02  !  decrease surf. albedo   for sensitivity study !!!! remove
        salb440v2  = aeronet_dump(151)  !  AERONET V2 albedo ( as function of SZA) 
      	! albedo=0.0543 at Thessaloniki at 440nm from AERONET albedo on 07/24/2006

      isurfmodel=6 !	this default surface albedp is modified using AERONET V2 albedo ( as function of SZA) 
      call surfmodel(salb440v2,isurfmodel,lamalb,surfalb) ! too low surf albedo at GSFC
         write(*,*) 'lambda,  spec albedo'
          write(*,'(2f8.3)') (lamalb(i),surfalb(i),i=1,12)


c---	mfrsr measuremets
	read(17,*) n_meas		! get number of MFRSR measurements for a given almucantar retrieval
        write(6,*) n_meas

      do imfrsr=1,n_meas		! main cycle of MFRSR retrievals

 	read(17,*,err=902) mfrsr_dump
      write(6,*) mfrsr_dump

c		Aerosol optical Thickness

      mfrsr_aot(1) 	=mfrsr_dump(21)		! 368nm

      if(cimelaot) then	!	Using interpolated CIMEL AOT instead of MFRSR AOT
        write(6,*) 'Using CIMEL AOT !!!'
        write(28,*) 'Using CIMEL AOT !!!'
      mfrsr_aot(1) =mfrsr_dump(18)              ! interpolated to 368nm AERONET AOT
      mfrsr_aot(2) =aot340m*1.02410**alpha      ! 332nm      
      mfrsr_aot(3) =aot340m*1.04620**alpha      ! 325nm      
      endif

	if (mfrsr_aot(1) .lt. 0.0) then
	write(6,*) 'front_mfrsr: NEGATIVE MFRSR AOT'
        write(6,*) 'Skip to the next measurement:'
        goto 101
	endif

      pi=4.0*atan(1.0)
      dt=pi/180.0

      mfrsr_hour  	=mfrsr_dump(5)    	! hour of the  3 min averaging interval
      mfrsr_min   	=mfrsr_dump(6)    	! minute of 3 min averaging interval
      mfrsr_sec   	=mfrsr_dump(7)    	! sec of 3 min averaging interval
      mfrsr_sza   	=mfrsr_dump(13)   	! SZA at the end of 3 min averaging interval, [deg]
      mfrsr_v0raw	=mfrsr_dump(11)			! V0_raw at selected channel

c--- Calculate MFRSR total and diffuse transmittances

      mfrsr_tot(1)	=mfrsr_dump(8)/mfrsr_v0raw		! USDA Total Transmittance
      mfrsr_dif(1)	=mfrsr_dump(9)/mfrsr_v0raw		! Diffuse Transmittance  
      mfrsr_dirn 	=mfrsr_dump(10)/mfrsr_v0raw		! Direc_normal transmittance

      if(mdif_corr) then			! Using corrected MFRSR diffuse 
      write(28,*)  'Diff transmittance correction: tdif*',
     * mdif_corr_cos
      mfrsr_dif(1)=mdif_corr_cos*mfrsr_dif(1)		! increase diffuse Voltage           
      mfrsr_tot(1)	=mfrsr_dirn*cos(dt*mfrsr_sza)+mfrsr_dif(1)	! re-calculate total transmittance mfrsr_dif correction !!!
      endif



c				Effective monochromatic wavelength and spectral constants for the channel (former coe.dat and gas2.coe)
      mfrsr_ldir	=mfrsr_dump(25)		!   mfrsr_ldir is radiatively effective wavelength (direct irradiance)
      lambda(1) =  10.*mfrsr_ldir		!  dynamic mfrsr_ldir from mfrsr_dump(25)

      SELECT CASE (INT(mfrsr_ldir))
	CASE (438:442)
      		xno2=1.3380e+01				! NO2 x-section at 440nm [atm_cm-1] rectangular slit FWHM=10nm, T=294K
      		rhob= 2.9100e-02				! Rayleigh depolarization factor at 440nm 
	CASE (364:370)
      		xno2=1.4765e+01				! NO2 x-section at 368nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0100e-02				! Rayleigh depolarization factor at 368nm 
	CASE (330:334) 
      		xno2=8.8113e+00				! NO2 x-section at 332nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0600e-02				! Rayleigh depolarization factor at 332nm
	CASE (323:327)
      		xno2=7.5192e+00				! NO2 x-section at 325nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.0867e-02				! Rayleigh depolarization factor at 325nm
	CASE (315:319)
      		xno2=6.2748e+00				! NO2 x-section at 317nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1321e-02				! Rayleigh depolarization factor at 317nm
	CASE (309:313)
      		xno2=5.2258e+00				! NO2 x-section at 311nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1628e-02				! Rayleigh depolarization factor at 311nm
	CASE (303:307)
      		xno2=4.2322e+00				! NO2 x-section at 305nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1700e-02				! Rayleigh depolarization factor at 305nm
	CASE (298:302)
      		xno2=3.4081e+00				! NO2 x-section at 300nm [atm_cm-1] rectangular slit FWHM=2nm, T=294K
      		rhob= 3.1726e-02				! Rayleigh depolarization factor at 300nm 
 	CASE DEFAULT                         
 	write(*,*) 'mfrsr_ldir is outside UV-MFRSR spectral channels: ',
     * mfrsr_ldir
      	stop
      END SELECT

c					O3 optical thickness and effective dynamic x-section
c	define effective ozone x-sec (c0b) that will produce exactly the same tau_ozone (mfrsr_toz) subtraction used in calibration: mfrsr_dump(23)  
c        c0b,c1b=0,c2b=0,rhob and pass them to gs_fit routine
      mfrsr_ozone 	=mfrsr_dump(16)   	! total ozone , Brewer [DU] ( fill in with EP/TOMS data): !! no fill in values allowed !!
      mfrsr_toz		=mfrsr_dump(23)		!   mfrsr_toz is ozone tau                                
      c0b=1000.*mfrsr_toz/mfrsr_ozone		!  effective dynamic ozone x-sections [atm-cm-1]
      c1b=0.0000  				! neglect Temperature dependence of ozone absorption
      c2b=0.000 				! neglect Temperature dependence of ozone absorption

c					NO2 optical thickness
c	mfrsr_no2_atmcm and xno2 are passed to gs_fit to add as gas2 absorber
      mfrsr_no2_atmcm 	=0.001*mfrsr_dump(27)	!  NO2 total amount [atm*cm]
 	if (mfrsr_no2_atmcm .lt. 0.0) then	! Process NO2 fill values      
              mfrsr_no2_atmcm=0.0	! treat fill values as no NO2 !
c            mfrsr_no2_atmcm=0.001*0.14	! AERONET clim NO2 value [atm*cm] at Santa Cruz, Bolivia, Sep 14
	endif
      mfrsr_errno2DU 	=mfrsr_dump(28)	!  NO2 expanded uncertainty [DU] total amount [atm*cm]
      mfrsr_dtno2	=mfrsr_dump(29)	!  minimal time difference between NO2 and MFRSR measurements [min]

      aeronet_no2	=0.001*mfrsr_dump(30)	!  AERONET assumed NO2 climatology [atm-cm]                            


      tau_no2= mfrsr_no2_atmcm*xno2		!  NO2 effective optical thickness (rectangular, FWHM=2nm, T=294K )
      write(6,*) 'NO2 [DU], x_no2',1000.0*mfrsr_no2_atmcm, xno2

c     Version 2 AERONET AOT is  already corrected for climatological tau NO2
c  !!! Additional correction is needed for the difference between actual and AERONET NO2
c     if (no2_corr) then 			!   Obsolete in Version 2: reduce V1 AOT by tau_NO2           
      mfrsr_aot(1)=mfrsr_dump(21)- 
     *             xno2*(mfrsr_no2_atmcm - aeronet_no2)                ! reduce AOT by difference between Brewer  and AERONET tau_NO2         

	if (mfrsr_aot(1) .lt. 0.0) then
	write(6,*) 'no2_corr: tau_no2 > mfrsr_aot',mfrsr_aot(1),tau_no2
       write(6,*) 'Skip to the next measurement:'
       goto 101
	endif


        if (mfrsr_sza   .gt. 78.0) then
        write(6,*) mfrsr_hour,mfrsr_min,mfrsr_sec,mfrsr_sza
        write(6,*) 'front_mfrsr: mfrsr_sza>78 deg'
        write(6,*) 'Skip to the next MFRSR measurement:'
        goto 113		! skip to the next MFRSR record
        endif		! end check SZA>78

c---	V0 correction: V0_corr=V0*k at 368nm => dtau=myu*ln(k), ln(k) from CIMEL comparisons

      if(mtau_corr) then	!	Using corrected MFRSR AOT
        write(6,*) 'Using corrected mfrsr 368 AOT: dtau=myu*ln(k), 
     * ln(k)=',mean_lnk
        write(28,*) 'Using corrected mfrsr 368 AOT: dtau=myu*ln(k), 
     * ln(k)=',mean_lnk
      mfrsr_aot(1) =mfrsr_aot(1)     +mean_lnk*cos(pi*mfrsr_sza/180.) 
      endif


c---   Calculation of fitting parameter
      do i=1,1

	if (abs(mfrsr_dirn   ) .lt. 0.0001) then
        mfrsr_dd(i)= - 9999.999
        mfrsr_dt(i)= - 9999.999
        mfrsr_tot(i) = - 9999.999
        mfrsr_dif(i) = - 9999.999

        else
	mfrsr_dd(i) = mfrsr_dif(i)/mfrsr_dirn    ! normalize to dir_normal
	mfrsr_dt(i) = mfrsr_dif(i)/mfrsr_tot(i)  ! normalize to total_hor 
        endif
      enddo

c--- Calculate MFRSR total and diffuse transmittances
c     mfrsr_v0raw	=mfrsr_dump(11)			! V0_raw at 368nm
c     mfrsr_tot(1)	=mfrsr_dump(8)/mfrsr_v0raw		! Total transmittance
c     mfrsr_dif(1)	=mfrsr_dump(9)/mfrsr_v0raw		! Diffuse  transmittance
c     mfrsr_dirn        =mfrsr_dump(10)/mfrsr_v0raw	! direct normal transmittance

	if(mie_only) go to 102		! no irradiance calculation

c---	vertical structure model
      nlayer=100
      aerwidthh=0.5		! Gaussian LAyer StDev
c     aermaxh=3.0		! Gaussian Layer height above surface [km] (Santa Cruz, Bolivia)
      aermaxh=0.0		! anthropogenic Mexico city 

      aeronet_aotchan	=mfrsr_dump(18)     	!  interpolated AERONET AOT to the UV-MFRSR channel      

      md12_pressure	=mfrsr_dump(15)     	!  Pressure measurement [mbars] - not used as input to Arizona code
      mfrsr_tr		=mfrsr_dump(22)		!   mfrsr_tr is rayleigh tau for  the measured surface pressure(md12_pressure) and l_dir, used for determining V0 during calibration
c     mfrsr_tr		=mfrsr_dump(22)*md12_pressure/1013.25	!! No additional scaling !!! mfrsr_tr is already scaled for the actual surface pressure and l_dir

c!!!  Keep surface pressure to 1 atm as Arizona code input, so not to trucate Rayleigh and aerosol optical thicknesses
      surfpres=1013.25

c---    ozone vert profile model
	iozpr=0		!iozpr=0; if iozpr=1 then enhanced trop. ozone
      totoz= 0.001*mfrsr_ozone    ! Arizona input total ozone [atmcm] or NO2 (substitution)

      do i=1,11
      ozone(i)=ozoneb(i) 	! ozone profile in DU/Umkehr
      enddo

      if(iozpr.eq.1) then
c     totoz=totoz+0.5*ozone(1)/1000.0
      ozone(1)=1.5*ozone(1)
      endif
      write(ozprof,'(i3,''M'')') int(totoz*1000.0)
	write(*,*) 'ozone profile  ',ozprof
c      call ozonemodel(ozprof,ozone,temp)
	write(*,*) 'umkehr layer,   ozone,   temperature'
	write(*,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11)

      
      
C----------call a subrotine interpolating the TOMRAD ozone and temperature
C----------profiles

      call profiles(1,surfpres,ozone,temp)

C----------call a subroutine adding an aerosol profile and gas2 profile to be used for
C----------Gauss-Seidel computations

c     gas2tot=0.001		! gas2  column amount [atm-cm] : gas2tot=0.001 for 1DU of SO2
      gas2tot=mfrsr_no2_atmcm	! NO2  column amount [atm-cm] 

      call aerprof(aermaxh,aerwidthh,gas2maxh,gas2width,gas2tot,nlayer)


c---	Cloud parameters
      tauc=0.0		!  NO CLOUDS

      
C-----------end the orbit and environmental models

 102  continue		! skip here if Mie only calculations

c     do jj=1,10		!  loop through SZA
      do jj=1,1 
c     sza=szab(jj)
      sza=mfrsr_sza              ! MFRSR SZA 

      tau440=aeronet_dump(20)      !  AERONET measurement at 441nm
      write(*,*) 'tau440=',tau440


      call gs_fit(firstmult_k,xno2,c0b,c1b,c2b,rhob,
     *max_k_iter,dd_fit,dt_fit,t_fit,d_fit,
     *fit_tolerance,log_fit,mie_only,
     *totoz,gas2tot,surfpres,mfrsr_tr,   
     &mfrsr_dd,mfrsr_dt,mfrsr_tot,mfrsr_dif,mfrsr_aot,
     &tau440,tauc,sza,vza,phi,nvza,nphi,
     &iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
     &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,isurfmodel,lamalb,surfalb,
     &icount,lambda,ratio,taum,omega,tau_real,om_real,rad_rayl,
     & n_real,k_real,e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio,
     *dd_new,n_iter,err_dd,meas_par )

      call system ('clean')

      
      do i=1,icount	! icount < 50
     	if (e_dir_ray(i) .eq. 0.0) go to 100
	e_dir(i)=e_dir_ray(i)*e_dir_ratio(i)
        e_dif(i)=e_dif_ray(i)*e_dif_ratio(i)
       	if (e_dir(i) .lt. 0.00001 ) then
      dd_calc(i)=  -9999.999       
	else
      dd_calc(i)=e_dif(i)/e_dir(i)
	endif

      enddo
      
 	if(mie_only) go to 101

C------------open radiance output file

	if ( no_radiance_output ) go to 101

       if(jheader.eq.1) then
      jheader=2
      if(iaermod.lt.10) then
        write(aers,'(a,i1,''t'',f4.2)') '0',iaermod,tau440
      else
        write(aers,'(i2,''t'',f4.2)') iaermod,tau440
      endif
      write(surfs,'(i1)') int(aermaxh)
      environm=ozprof//'_'//aers//'_'//surfs
      write(fout,'(''DATA/faer_en'',a,''.dat'')') environm
	WRITE(*,*) 'output filename',fout
      open(11,file=fout,status='unknown')

C-------------write a header

      write(11,'(a)') 'ENVIRONMENTAL VARIABLES'
      write(11,'(a,a)') 'TOMS OZONE & TEMPERATURE PROFILES ',ozprof
      write(11,'(a)') 'umkehr layer,   ozone,   temperature'
      write(11,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11) 
      write(11,'(a,i2,a,f6.3)') 'AEROSOL MODEL',iaermod,' tau440=',
     &tau440
      write(11,'(a,f6.2,a,f6.2,a,e8.2,a,f6.2,a,f6.2,a,e8.2)') 'log-normal 
     & distr: rmean1, ',rmean1,' sigma1 ',sigma1,' Nc1 ',Nc1,
     &' rmean2 ',rmean2,' sigma2 ',sigma2,' Nc2 ',Nc2
      write(11,'(a)') 'wavelength (mu)  real,  imager,           optical 
     & thickness,   omega0'
      write(11,'(f8.3,4e12.3,2f12.3)') (lam(i),refrr1(i),refri1(i),
     &refrr2(i),refri2(i),taum(i),omega(i),i=1,12)      
      write(11,'(f8.3,2f14.5)') (lambda(i),
     &tau_real(i),om_real(i),i=1,12)      

      write(11,'(a,f5.1,a,f5.1)') 'PROFILE: height=',aermaxh,' scale=',     
     &aerwidthh 
      write(11,'(a,f8.1)') 'SURFACE PRESSURE, mb ',surfpres
      write(11,'(a,i2)') 'SURFACE MODEL ',isurfmodel
      write(11,'(a)') ' wavelength, mu      albedo'
      write(11,'(f8.3,f12.3)') (lamalb(i),surfalb(i),i=1,12)
      write(11,*)

      endif

      do j=1,nvza
      do k=1,nphi
      write(11,'(a)') 'GEOMETRY'
      write(11,'(a,f7.2,a,f7.2,a,f9.2)') ' sza =',sza,' vza =',vza(j),
     &' phi =',phi(k)
      write(11,'(a)') 'WAVE (A),  sky_rad_rayl 		aer/ray ratio' 

C--------------write data

      do i=1,icount		!  icount < 50
        write(11,'(3e13.5)') lambda(i),rad_rayl(i,j,k),ratio(i,j,k)
      enddo

      enddo
      enddo

      enddo	! end sza loop

 101	continue	! skip radiance calculations


c---	write flux and MFRSR results
      write(6,*) 'DEBUG!!!: icount,mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn
     *,mfrsr_v0raw'
      write(6,*)icount,mfrsr_tot(1),mfrsr_dif(1),mfrsr_dirn,mfrsr_v0raw

	write(28,81)
     &oleg_dd,oleg_month,oleg_yyyy, 		! UT day #, month #      
     &oleg_hh,oleg_mm,oleg_ss,oleg_dayofyear, 	! UT hh, min,sec, decimal doy   #      

     *int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec) 
     *,mfrsr_sza,mfrsr_ozone,1000.*mfrsr_no2_atmcm,surfpres 
     *,(0.1*lambda(i),n_real(i),k_real(i), 
     *   tau_real(i),om_real(i),aeronet_aotchan,
     * mfrsr_tr,mfrsr_toz,mfrsr_ldir,
     * mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw,	    ! mfrsr measurements of tot/V0, dif/V0 and dir_norm/V0 components
     *rad_rayl(i,1,1),ratio(i,1,1),e_dir_ray(i),e_dir_ratio(i),
     *e_dif_ray(i),e_dif_ratio(i),
     *dd_new(i),meas_par,n_iter(i),i=1,icount),
     *mfrsr_errno2DU,mfrsr_dtno2 
      
      write(6,*) (0.1*lambda(i),n_real(i),k_real(i),
     *   tau_real(i),om_real(i),i=1,icount)

	write(25,8)
     &oleg_dd,oleg_month,oleg_yyyy, 		! UT day #, month #      
     &oleg_hh,oleg_mm,oleg_ss,oleg_dayofyear, 	! UT hh, min,sec, decimal doy   #      
     &aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,aot340m, ! measured aot 
c    &oleg_aot340,oleg_aot380,
     &oleg_aot440,oleg_aot670,oleg_aot870,
     &oleg_aot1020, 						! OLeg aot 
     &oleg_w440,oleg_w670,oleg_w870,oleg_w1020,			! OLeg W 
     &rr440,rr670,rr870,rr1020,					! OLeg Real refr
     &ri440,ri670,ri870,ri1020,				 	! OLeg Im refr
     & vf,vc, rvf,rvc, lnsf,lnsc  				! PSD: fine and coarse mode
     *,int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec) 
     *,mfrsr_sza,mfrsr_ozone,1000.*mfrsr_no2_atmcm,surfpres 
     *,(0.1*lambda(i),n_real(i),k_real(i), 
     *   tau_real(i),om_real(i),aeronet_aotchan,
     * mfrsr_tr,mfrsr_toz,mfrsr_ldir,
     * mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw,	    ! mfrsr measurements of tot/V0, dif/V0 and dir_norm/V0 components
     *rad_rayl(i,1,1),ratio(i,1,1),e_dir_ray(i),e_dir_ratio(i),
     *e_dif_ray(i),e_dif_ratio(i),
     *dd_new(i),meas_par,n_iter(i),i=1,icount),
     *mfrsr_errno2DU,mfrsr_dtno2 
      


c23456789012345678901234567890123456789012345678901234567890123456789012
 8    format(             
     &       2(1x,I2), 		!UT day #,   UT month #
     &         1x, I4,         ! UT year  #
     &       3(1x, I2),         ! UT hour  #
     &         1x, f10.6,      ! decimal UT day of year
     &       7(1x,f9.6),2x,    ! aeronet measured aot @ 1020, 870nm,670nm,500nm,440nm,380nm,340nm
c    &       2(1x,f5.3),2x,    ! OLeg aot @ [340, 380,]                            
     &       4(1x,f5.3),2x,    ! OLeg aot @ 440, 670, 870 and 1020                           
     &       4(1x,f4.2),2x,    ! OLeg W @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg n @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg k @ 1020, 870nm, 670,440                                   
     &       6(1x,f9.6),2x,   !  Log-Norm PSD parameters
     &       3(i2,1x),         ! int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec)
     &       f5.2,1x,2(f5.1,1x),f7.2,2x,  ! mfrsr_sza, mfrsa_ozone, 1000.*mfrsr_no2_atmcm, surfpres
c      lam       (n,k)  (mt,w,aer_t368)
     & f6.1,1x,2(f9.6,1x),3(f6.3,1x),2x,2(f8.6,1x),f8.4,2x,
c      t_tot,t_dif,t_dir V0
     & 3(f7.5,1x),1(f7.2,1x),3x,			!   mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw
     & 6(e13.5,1x), 2(f9.4,1x),3x,i2,2(1x,f5.2))    

 81   format(             
     &       2(1x,I2), 		!UT day #,   UT month #
     &         1x, I4,         ! UT year  #
     &       3(1x, I2),         ! UT hour  #
     &         1x, f10.6,5x,   ! decimal UT day of year

     &       3(i2,1x),         ! int(mfrsr_hour),int(mfrsr_min),int(mfrsr_sec)
     &       f5.2,1x,2(f5.1,1x),f7.2,2x,  ! mfrsr_sza, mfrsa_ozone, 1000.*mfrsr_no2_atmcm, surfpres
     & f6.1,1x,2(f9.6,1x),3(f6.3,1x),2x,2(f8.6,1x),f8.4,2x,
     & 3(f7.5,1x),1(f7.2,1x),3x,			!   mfrsr_tot(i),mfrsr_dif(i),mfrsr_dirn,mfrsr_v0raw
     & 6(e13.5,1x), 2(f9.4,1x),3x,i2,2(1x,f5.2))    

	write(6,*) 'mfrsr measurement processed: '

 113    continue        ! mfrsr measurement not processed because mfrsr_sza> 78deg

        enddo 		! continue mfrsr data loop for single AERONET inversion

	write(6,*) 'one aeronet inversion done !'

     
      go to 100   	! return to read new AERONET retrieval


 210  continue        ! end of AERONET file encountered 
      write(6,*) 'end of AERONET file encountered !!'

C------------close all files       
     
      close(17)
      close(11)
      close(25)
      close(28)

      stop

c-- error handling
 
  900  continue
       write(6,*) 'front_end: could not open merged AERONET/MFRSR file '
       stop

  901  continue
       write(6,*) 'front_end: error while reading data in 
     &         AERONET file '
       stop
  902  continue
       write(6,*) 'front_end: error while reading data in 
     &         MFRSR file '
       stop
 
       end
