      program TOMRAD_GS_COMPUTATION
C***********************************************************************
C	AUTHOR: Alexander P. Vasilkov
C
C	FUNCTION:
C	
C       WARNING
C       1) current version does not allow refr index changes with wavelength
C       to implement spectral varions in refr index the file miecalc.dat should be
C       re-calculated at each wavelength !
C       2) only bi-modal log-normal PSD
C
C	HISTORY: Developed June, 2000
C	Modified September 5, 2000 to include another absorbing gas 
C	Modified October 12, 2000 to include aerosol various models 
C	Modified October 20, 2000 to include surface reflectance models
C	Modified 05/16/2001 to eliminate common blocks
C	Modified 05/22/2001 to enable multiple SZA calculations
C	Modified 05/25/2001 to change the output of the Ring effect
C	Modified 08/07/2001 to enable multiple VZA & PHI calculations
C     Modified 05/31/2002 to use aot @ 500nm instead of 550nm  
C     Modified 06/12/2002 to use 275M ozone and temp profile
C     Modified 01/08/2003 to calculate only one sza         
C     Modified 01/31/2003 to calculate only 4 wavelengths: 340nm, 360nm, 380nm, 441nm
C     Modified 02/22/2003 [nk] for aeronet aerosol input: tau_500 and Oleg's retrievals   
C     Modified 03/12/2003 [nk] for new output Oleg's retrievals   
C     Modified 03/20/2003 [nk] To put filter on n spectral dependence
C     Modified 03/21/2003 [nk] to use aot @ 440nm instead of 500nm   
C     Modified 03/22/2003 [nk] to aeronet climatology at GSFC [Dubovik et al, JAs,59,590-,2002]
C     Modified 03/25/2003 [nk] to ref_440=.true. or .false.  (use refr index @440nm)          
C                          gsfc_refr_clim=.false. or .true.                               
C     Modified 06/06/2003 [nk] to read new aeronet data format (web download tool)                           
C     Modified 06/05/2003 [nk] to calculate surface irradiance (dir and diff) @ selected wavelengths         
C***********************************************************************
      implicit none
            
      logical mie_only
      logical gsfc_psd_clim,gsfc_refr_clim,ref_440

      parameter (mie_only=.false.) 
      parameter (gsfc_psd_clim=.false.,gsfc_refr_clim=.false.)
c---  if gsfc_refr_clim=.true. then ref_440 does not matter
      parameter (ref_440      =.true. )  ! Use constant value of ref_440 ?

      integer i,j,k,nstr,maxnum,iaertype,nlayer
      integer iaermod,isurfmodel,icount,iozpr,nozpr
      integer jj,jheader,nvza,nphi
c                vert   azimuth wavelength
      parameter (nvza=6,nphi=7)	!  max dimensions of vza and phi arrays
      parameter (icount=1)	!  number of wavelengths (see coe.dat and gas2.dat files)
c     parameter (nvza=6,nphi=7,icount=12)
      
      real sza,szab(10),vza(nvza),phi(nphi),vzatom,phitom,vzar,phir
      real lambda(5000),wave
      real aermaxh,aerwidthh,lat,aeronet_dump(123)
      real wavea,surfpresrel,lamalb(12),surfalb(12)
      real*8 surfpres,ozone(11),ozoneb(11),temp(11)
      real refrr1(12),refri1(12),refrr2(12),refri2(12),lam(12)
      real sigma1,rmean1,Nc1,sigma2,rmean2,Nc2
      real tau440,totoz,totwat,taumd,taur,tauc,pi,dt      
c     real tau500,totoz,totwat,taumd,taur,tauc,pi,dt      
      real so2maxh,so2width,so2tot
      real tau_real(1000),om_real(1000)
      real n_real(1000),k_real(1000)
      real ratio(5000,nvza,nphi),taum(12),omega(12)
      real rad_rayl(5000,nvza,nphi)
      real e_dir_ray(50),e_dir_ratio(50),e_dif_ray(50),e_dif_ratio(50)

c---    aeronet AOT measurements
      integer iaeronet                          
      real aot340m,aot380m,aot440m,aot500m,aot670m,aot870m,aot1020m
c---    aeronet retrievals       
      real oleg_aot340,oleg_aot380,oleg_aot440,oleg_aot670,oleg_aot870
      real oleg_aot1020
      real oleg_w440,oleg_w670,oleg_w870,oleg_w1020    
      real oleg_dayofyear
      integer oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss
c---   aeronet size distribution parameters appoximated as bi-modal log-normal distribution
      real lnsf,rvf,vf,lnsc,rvc,vc  ! Volumne size distribution parameters (AERONET)
      real      rnf,nf,     rnc,nc  ! Number  size distribution parameters (ARIZONA)
      real  rr440,rr670,rr870,rr1020 ! Oleg's refractive index
      real  ri440,ri670,ri870,ri1020 ! Oleg's refractive index

      character*256 aeronethead
      character*480 header
      character*90 fin,fout   
      character*90 str(24),buf(8)
      character*15 fdata
      character*25 buff,faerosol
      character*37 buftom
      character*4 ozprof
      character*7 aers
      character*1 surfs,ozlat
      character*14 environm
      
c               number of dump rows in arizona outout files
      parameter (nstr=18,nozpr=26)

      data vza/0.0,15.0,30.0,45.0,60.0,75.0/
      data phi/0.0,30.0,60.0,90.0,120.0,150.0,180.0/
      data szab/0.0,30.0,45.0,60.0,70.0,77.0,81.0,84.0,86.0,88.0/
c--   V7 325 ozone & Temp profile
c     data ozoneb/16.0,14.0,26.0,45.0,74.7,66.9,41.7,24.5,11.1,3.7,1.4/
c     data temp/273.0,239.0,219.1,216.6,217.0,220.8,227.6,239.4,
c    &253.6,263.9,262.6/
c--   V7 275 ozone & Temp profile
      data ozoneb/16.0,12.0,15.0,29.0,58.0,63.7,40.6,24.5,11.1,3.7,1.4/
      data temp/273.0,239.0,217.1,212.2,214.9,220.4,227.6,239.4,
     &253.6,263.9,262.6/


      pi=4.0*atan(1.0)
      dt=pi/180.0
      jheader=1

c     -- open output file
      open (25,file='DATA/aeronet_nk440_radiance',status='unknown')
c     open (25,file='DATA/aeronet_nk440670',status='unknown')
c     open (25,file='DATA/aeronet_nkclim',status='unknown')
c     open (25,file='DATA/aeronet_all.output',status='unknown')

c23456789012345678901234567890123456789012345678901234567890123456789012
      header='oleg_dd,oleg_month,oleg_yyyy,oleg_hh,oleg_mm,oleg_ss,
     *oleg_dayofyear,aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,
     *aot340m,oleg_aot340,oleg_aot380,oleg_aot440,oleg_aot670,
     *oleg_aot870,oleg_aot1020,
     *oleg_w440,oleg_w670,oleg_w870,oleg_w1020,rr440,rr670,rr870,rr1020,
     *ri440,ri670,ri870,ri1020,vf,vc, rvf,rvc, lnsf,lnsc,
     *(lambda(i),n_real(i),k_real(i),tau_real(i),om_real(i),i=1,icount)'
        write(25,*) header
      
C----------set up orbit and environment models

c---   Surface model

c!!!  change to actual pressure level at roof bld 33 on a given day
      surfpresrel=1.0
      surfpres=surfpresrel*1013.25

c---   Surface albedo model
      isurfmodel=6		! albedo=0.02
      call surfmodel(isurfmodel,lamalb,surfalb)
	write(*,*) 'lambda,  spec albedo'
	write(*,'(2f8.3)') (lamalb(i),surfalb(i),i=1,12)


c---	AERONET aerosol model
c     open(10,file='inp.dat',status='old')
c     read(10,*) iaermod,tau440,aermaxh
c     read(10,*) iozpr
c     close(10)

c	iaermod=2
        iaertype=4   !only bi-modal log-normal PSD accepted with AERONET data


      open(13,file='../aeronet_GSFC_web.10022002.125235',   ! open AERONET web file 
c     open(13,file='../aeronet_dump_updated.gsfc',        ! open AERONET file
     &     form='formatted',status='old',err=900)
      write(6,*)              'Reading AERONET retrievals'  ! reading header
      do iaeronet=1,4
       read(13,"(1a256)",ERR=901) aeronethead
      enddo

c------                        MAIN LOOP THOUGH ALL AERONET RETRIEVALS

  100   continue        !   start reading new AERONET retrieval
	write(6,*) 'start reading new AERONET retrieval'

        read(13,*,err=901,end=210) aeronet_dump
	oleg_dd=aeronet_dump(1)
	oleg_month=aeronet_dump(2)
	oleg_yyyy=aeronet_dump(3)
	oleg_hh=aeronet_dump(4)
	oleg_mm=aeronet_dump(5)
	oleg_ss=aeronet_dump(6)
        oleg_dayofyear=aeronet_dump(7)
        aot1020m  =aeronet_dump(8)
        aot870m  =aeronet_dump(9)
        aot670m  =aeronet_dump(10)
        aot500m  =aeronet_dump(11)
c       tau500  =aot500m
        aot440m  =aeronet_dump(12)
        tau440  =aot440m
        aot380m  =aeronet_dump(13)
        aot340m  =aeronet_dump(14)
        oleg_aot340   =aeronet_dump(15)
        oleg_aot380   =aeronet_dump(16)
        oleg_aot440   =aeronet_dump(18)
        oleg_aot670   =aeronet_dump(19)
        oleg_aot870   =aeronet_dump(20)
        oleg_aot1020   =aeronet_dump(21)

        oleg_w440     =aeronet_dump(31)
        oleg_w670     =aeronet_dump(32)
        oleg_w870     =aeronet_dump(33)
        oleg_w1020     =aeronet_dump(34)

        rr440		=aeronet_dump(56)
        rr670		=aeronet_dump(57)
        rr870		=aeronet_dump(58)
        rr1020		=aeronet_dump(59)

        ri440		=aeronet_dump(60)
        ri670		=aeronet_dump(61)
        ri870		=aeronet_dump(62)
        ri1020		=aeronet_dump(63)

        vf    		=aeronet_dump(102)
        vc    		=aeronet_dump(106)
        rvf    		=aeronet_dump(104)
        rvc    		=aeronet_dump(108)
        lnsf    		=aeronet_dump(105)
        lnsc    		=aeronet_dump(109)


      write(6,*) oleg_dayofyear

      call get_aeronet(gsfc_psd_clim,gsfc_refr_clim,
     *ref_440,aeronet_dump,  ! calc parameters for Arizona code
     *lam,refrr1,refri1,refrr2,refri2,
     *sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

c     iaertype=4
c     if(iaermod.eq.17) iaertype=2
c     call aermodel(iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
c    &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2)

	 write(*,*) 'Nc1 Nc2',Nc1,Nc2
	 write(*,*) 'sigma1,rmean1,sigma2,rmean2',sigma1,rmean1,sigma2,rmean2
	 write(*,*) 'lambda,  real,   imaginary index'
	 write(*,'(5f10.4)') (lam(i),refrr1(i),refri1(i),refrr2(i),
     &refri2(i),i=1,12)

	if(mie_only) go to 102
c---	vertical structure model
      nlayer=100
      aerwidthh=1.5
      aermaxh=0.0

c---    ozone vert profile model
	iozpr=0		!iozpr=0; if iozpr=1 then enhanced trop. ozone
      totoz=0.275
      do i=1,11
      ozone(i)=ozoneb(i)
      enddo
      if(iozpr.eq.1) then
      totoz=totoz+0.5*ozone(1)/1000.0
      ozone(1)=1.5*ozone(1)
      endif
      write(ozprof,'(i3,''M'')') int(totoz*1000.0)
	write(*,*) 'ozone profile  ',ozprof
c      call ozonemodel(ozprof,ozone,temp)
	write(*,*) 'umkehr layer,   ozone,   temperature'
	write(*,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11)
      so2tot=0.0
      so2maxh=0.0
      so2width=1.5
      tauc=0.0
      
      
C----------call a subrotine interpolating the TOMRAD ozone and temperature
C----------profiles

      call profiles(1,surfpres,ozone,temp)

C----------call a subroutine adding an aerosol profile to be used for
C----------Gauss-Seidel computations

      call aerprof(aermaxh,aerwidthh,so2maxh,so2width,so2tot,nlayer
     &,totwat)
      write(*,*) 'total o2o2 ',totwat
      
C-----------end the orbit and environmental models

 102  continue

c     do jj=1,10		!  loop through SZA
      do jj=1,1 
c     sza=szab(jj)
      sza=70.3      ! AERONET retrieval on Oct 2 2002, 12:52:35

      call gauss_seidel(mie_only,totoz,totwat,surfpresrel,
     &tau440,tauc,sza,vza,phi,nvza,nphi,
     &iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
     &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,isurfmodel,lamalb,surfalb,
     &icount,lambda,ratio,taum,omega,tau_real,om_real,rad_rayl,
     & n_real,k_real,e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio)
      
 	if(mie_only) go to 101

C------------open output file

       if(jheader.eq.1) then
      jheader=2
      if(iaermod.lt.10) then
        write(aers,'(a,i1,''t'',f4.2)') '0',iaermod,tau440
      else
        write(aers,'(i2,''t'',f4.2)') iaermod,tau440
      endif
      write(surfs,'(i1)') int(aermaxh)
      environm=ozprof//'_'//aers//'_'//surfs
      write(fout,'(''DATA/faer_en'',a,''.dat'')') environm
	WRITE(*,*) 'output filename',fout
      open(11,file=fout,status='unknown')


C-------------write a header

      write(11,'(a)') 'ENVIRONMENTAL VARIABLES'
      write(11,'(a,a)') 'TOMS OZONE & TEMPERATURE PROFILES ',ozprof
      write(11,'(a)') 'umkehr layer,   ozone,   temperature'
      write(11,'(i2,2f9.2)') (i,ozone(i),temp(i),i=1,11) 
      write(11,'(a,i2,a,f6.3)') 'AEROSOL MODEL',iaermod,' tau440=',
     &tau440
      write(11,'(a,f6.2,a,f6.2,a,e8.2,a,f6.2,a,f6.2,a,e8.2)') 'log-normal 
     & distr: rmean1, ',rmean1,' sigma1 ',sigma1,' Nc1 ',Nc1,
     &' rmean2 ',rmean2,' sigma2 ',sigma2,' Nc2 ',Nc2
      write(11,'(a)') 'wavelength (mu)  real,  imager,           optical 
     & thickness,   omega0'
      write(11,'(f8.3,4e12.3,2f12.3)') (lam(i),refrr1(i),refri1(i),
     &refrr2(i),refri2(i),taum(i),omega(i),i=1,12)      
      write(11,'(f8.3,2f14.5)') (lambda(i),
     &tau_real(i),om_real(i),i=1,12)      

      write(11,'(a,f5.1,a,f5.1)') 'PROFILE: height=',aermaxh,' scale=',     
     &aerwidthh 
      write(11,'(a,f8.1)') 'SURFACE PRESSURE, mb ',surfpres
      write(11,'(a,i2)') 'SURFACE MODEL ',isurfmodel
      write(11,'(a)') ' wavelength, mu      albedo'
      write(11,'(f8.3,f12.3)') (lamalb(i),surfalb(i),i=1,12)
      write(11,*)

      endif

      do j=1,nvza
      do k=1,nphi
      write(11,'(a)') 'GEOMETRY'
      write(11,'(a,f7.2,a,f7.2,a,f9.2)') ' sza =',sza,' vza =',vza(j),
     &' phi =',phi(k)
      write(11,'(a)') 'WAVE (A),  sky_rad_rayl 		aer/ray ratio' 

C--------------write data

      do i=1,icount
        write(11,'(3e13.5)') lambda(i),rad_rayl(i,j,k),ratio(i,j,k)
      enddo

      enddo
      enddo

      enddo	! end sza loop

 101	continue	! skip radiance calculations
	write(25,8)
     &oleg_dd,oleg_month,oleg_yyyy, 		! UT day #, month #      
     &oleg_hh,oleg_mm,oleg_ss,oleg_dayofyear, 	! UT hh, min,sec, decimal doy   #      
     &aot1020m,aot870m,aot670m,aot500m,aot440m,aot380m,aot340m, ! measured aot 
     &oleg_aot340,oleg_aot380,oleg_aot440,oleg_aot670,oleg_aot870,
     &oleg_aot1020, 							! OLeg aot 
     &oleg_w440,oleg_w670,oleg_w870,oleg_w1020,		! OLeg W 
     &rr440,rr670,rr870,rr1020,		! OLeg Real refr
     &ri440,ri670,ri870,ri1020,		! OLeg Im refr
     & vf,vc, rvf,rvc, lnsf,lnsc, 		! PSD: fine and coarse mode
     *(0.1*lambda(i),n_real(i),k_real(i),
     *               tau_real(i),om_real(i),
     *rad_rayl(i,1,1),ratio(i,1,1),e_dir_ray(i),e_dir_ratio(i),
     *e_dif_ray(i),e_dif_ratio(i),i=1,icount)


 8    format(             
     &       2(1x,I2), 		!UT day #,   UT month #
     &         1x, I4,         ! UT year  #
     &       3(1x, I2),         ! UT hour  #
     &         1x, f10.6,      ! decimal UT day of year
     &       7(1x,f9.6),2x,    ! aeronet measured aot @ 1020, 870nm,670nm,500nm,440nm,380nm,340nm
     &       6(1x,f5.3),2x,    ! OLeg aot @ [340, 380,] 440, 670, 870 and 1020                           
     &       4(1x,f4.2),2x,    ! OLeg W @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg n @ 1020, 870nm, 670,440                                   
     &       4(1x,f8.6),2x,    ! OLeg k @ 1020, 870nm, 670,440                                   
     &       6(1x,f8.6),2x,    !  Log-Norm PSD parameters
     &       5(1x,f6.1,1x,f8.6,1x,f8.6,1x,f5.3,1x,f5.3,2x,6(e13.5,1x))  ! Arizona lam, aot , w              
     &)

	write(6,*) 'one aeronet inversion done !'

     
      go to 100   	! return to read new AERONET retrieval


 210  continue        ! end of AERONET file encountered 

C------------close all files       
     
      close(13)
      close(11)
      close(25)

      stop

c-- error handling
 
  900  continue
       write(6,*) 'front_end: could not open AERONET file '
       stop

  901  continue
       write(6,*) 'front_end: error while reading data in 
     &         AERONET file '
       stop
 
       end
