      subroutine gauss_seidel(mie_only,totoz,totwat,surfpresrel,
     &tau440,tauc,sza,vza,phi,nvza,nphi,
     &iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
     &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,isurfmod,lamalb,surfalb,
     &icount,lambda,ratio,tauaer,omega,taum,om,rad,n_real,k_real,
     &e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio)
C***********************************************************************
C	AUTHOR: Alexander P. Vasilkov
C
C	FUNCTION: Compute the ratio R=L(aer)/L(noaer) and
C                 E_dir(aer)/E_dir(noaer) and E_dif(aer)/E_dif(noaer)
C                 using the Gauss-Seidel code
C
C	Modified Oct. 25, 2000 to avoid writing an output file
C	Modified Aug. 20, 2001 to calculate the ratio for 11 wavelengths
C	Modified Nov. 12, 2002 to make output of Rayleigh radiances
C	and save the phase.out files
C	Modified March 12,2003 to make Mie only calculations      [nk]
C	Modified March 21,2003 to normalize extinction at 440nm                  
C	Modified June  5 ,2003 to calculate dir and diffuse downwelling Rayleigh fluxes 
C                                 and (aer)/(noaer) ratio     
C***********************************************************************
      implicit none

	logical mie_only
            
      integer i,j,k,nstr,maxnum,iaertype,icount,inext,n,nang
      integer n2,iaermod,isurfmod,nn,ierr,nf,nalb,m,nvza,nphi
      parameter(nf=12,nalb=12,nang=181)
      
      real n_real(1000),k_real(1000)
      real sza,vza(nvza),phi(nphi)
      real lambda(5000),lambdab,wave
      real c0(5000),c1(5000),c2(5000),beta(5000),rho(5000)
      real c0b,c1b,c2b,betab,rhob,taur,pi
      real wavea,kingf,surfpresrel,lamalb(nalb),surfalb(nalb)
      real refrr1(nf),refri1(nf),refrr2(nf),refri2(nf),lam(nf)
      real tauaer(nf),omega(nf),rIOR1,iIOR1,rIOR2,iIOR2,specalb
      real sigma1,rmean1,Nc1,sigma2,rmean2,Nc2
      real tau440,totoz,totwat,tauc,taumd,surfpres,qext,qext440      
      real rad(5000,nvza,nphi),rad_aer(nvza,nphi)
      real theta,taum(1000),om(1000),ang(nang),phase(nang)
      real rad0,t,sbar,ratio(5000,nvza,nphi)
      real e_dir_ray(5000),e_dir_aer,e_dir_ratio(5000)
      real e_dif_ray(5000),e_dif_aer,e_dif_ratio(5000)
      real delw,w1,w2,wavep
      real lam2(5000),abs2(5000),absgas2,loschmidt,lambuf(1000)
      real interpol
      real*8 abso2o2
      external interpol
 
      character*40 ftop,fout   
      character*90 str(24),buf(12)
      character*15 fname,fdata
      character*25 buff,faerosol
      character*26 buff1
      character*12 fphase
C---------
      parameter (nstr=24,taumd=0.0,maxnum=20)
      data loschmidt/2.6868E19/     !in [#/cm^3]
     
      write(*,*) 'Gauss-Seidel code starts'
      
      surfpres=1013.0*surfpresrel      
      pi=4.0*atan(1.0)
            
C-------calculate aerosol extintion coefficient at reference wavelength: 440nm

      call aerosol(iaermod,iaertype,lam,refrr1,refri1,refrr2,
     &refri2,sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,qext440)
      write(*,*) 'Calculation of the aerosol extintion coefficient at 
     &440 nm completed'
      	write(*,*) 'qext440',qext440

C-------
      fname='flat.in'
      fdata='coe.dat'
      faerosol='aerosol.phs'

C----------open data file containing O3 cross-sections and Rayleigh coeff
      
      open(9,file=fdata,status='old')
 
C----------read in data file 

      do i=1,icount
      read(9,*) lambdab,c0b,c1b,c2b,betab,rhob
      lambda(i)=lambdab
      c0(i)=c0b
      c1(i)=c1b
      c2(i)=c2b
      beta(i)=betab
      rho(i)=rhob
      write(*,*) lambda(i),c0(i),beta(i),rho(i)
      enddo
      close(9)
      
C------------read in O2-O2 absorption coefficients

      open(20,file='gas2.coe',status='old')
      do i=1,icount
         read(20,*) lam2(i),abso2o2
         abs2(i)=loschmidt*abso2o2*loschmidt
      enddo
      close(20)
    
C-----------main loop on wavelength
       
      do i=1,icount
          
      wave=lambda(i)/1.0E4
      kingf=(6.0+3.0*rho(i))/(6.0-7.0*rho(i))
       
C----------aerosol index of refraction

      rIOR1=interpol(nf,lam,refrr1,wave,ierr)
      iIOR1=interpol(nf,lam,refri1,wave,ierr)
      rIOR2=interpol(nf,lam,refrr2,wave,ierr)
      iIOR2=interpol(nf,lam,refri2,wave,ierr)
      n_real(i)=0.5*(rIOR1+rIOR2)
      k_real(i)=0.5*(iIOR1+iIOR2)

C----------spectral surface albedo

      specalb=interpol(nalb,lamalb,surfalb,wave,ierr)

C----------open input file 'flat.in'
            
      open(10,file=fname,status='old')      
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo      
      close(10)
      
      open(10,file=fname,status='unknown')
      
      taur=beta(i)*surfpresrel
      
C-----------modify the input file for Mie calculations at lambda(i)/1.0E4
      
      write(10,'(5e11.4,f10.7)') tau440,taur,tauc,totoz,totwat,wave
      write(10,'(a)') str(2)
      write(10,'(4e11.4)') rIOR1,iIOR1,rIOR2,iIOR2
      write(10,'(a)') (str(k),k=4,15)
      write(10,'(3i3)') -1,4,1
      write(10,'(i2,f7.3)') 1,specalb
      write(10,'(a)') (str(k),k=18,nstr)
      close(10)
      
      call system('rm -f aerosol.phs')
      call system('rm -f miecalc.dat')
      call system('flatpsv.x')

      WRITE(*,*) 'MIE CALCULATIONS COMPLETED'

C----------read in the aerosol extintion coefficient
     
      open(10,file=faerosol,status='old')
      if(iaertype.eq.2) nn=8
      if(iaertype.eq.4) nn=12
      read(10,'(a)') (buf(k),k=1,nn)
      read(10,'(a,e12.4)') buff,qext
      read(10,'(a)') buf(1)
      read(10,'(a,f7.4)') buff1,om(i)
      WRITE(*,*) wave,QEXT
      close(10)
      
      taum(i)=tau440*qext/qext440
      WRITE(*,*) 'I ',i,'TAU AER ',TAUM(i),' OMEGA',om(i)
C      pause

	if(mie_only)  go to 100	! bypass radiance calculations

C-----------save phase.out
      open(10,file='phasf.out',status='old')
      do k=1,nang
      read(10,*) ang(k),phase(k)
      enddo
      close(10)
      write(fphase,'(''phase_'',f6.4)') wave
      open(10,file=fphase,status='unknown')
      do k=1,nang
      write(10,*) ang(k),phase(k)
      enddo
      close(10)

C-----------compute radiances for interpolation wavelengths  

C-----------open an input file

      open(10,file=fname,status='old')
      
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo
      close(10)
      
C-----------modify the input file
            
      open(10,file=fname,status='unknown')      
      write(10,'(5e11.4,f10.7)') taum(i),taur,tauc,totoz,totwat,wave
      write(10,'(a)') (str(k),k=2,5)
      write(10,'(f6.1,f8.5)') surfpres,kingf
      write(10,'(a)') (str(k),k=7,11)
      write(10,'(3e13.5)') c0(i),c1(i),c2(i)
      write(10,'(e13.5)') abs2(i)
      write(10,'(a)') (str(k),k=14,15)
      write(10,'(3i3)') maxnum,3,1
      write(10,'(a)') (str(k),k=17,18)
      write(10,'(f7.3)') sza
      write(10,'(a)') (str(k),k=20,21)
      write(10,'(a,6f8.1)') '.T. ',
     &vza(1),vza(nvza),15.0,phi(1),phi(nphi),30.0
      write(10,'(a)') (str(k),k=23,nstr)
      close(10)
      
      call system('flatpsv.x')

      if(sza.lt. 0.95) then
	 write(ftop,'(''flatpsv.s00'',f2.1,''.bot'')') sza
      else if(sza.lt.9.95) then
         write(ftop,'(''flatpsv.s0'',f3.1,''.bot'')') sza
      else
         write(ftop,'(''flatpsv.s'',f4.1,''.bot'')') sza
      endif
      
C----------open output file and read in TOA radiance
           
      open(10,file=ftop,status='old')      
      do k=1,13
         read(10,'(a)') buff
      enddo
      do j=1,nphi 
      do k=1,nvza     
      read(10,'(f6.2,f6.1,f6.2,f7.2,2e11.4)') theta,theta,theta,theta,
     &rad_aer(k,j),t
      enddo
      enddo
      
      close(10) 
      WRITE(*,*) 'RAD_AER ',RAD_AER(1,1)

C-----Gauss-Sedel computations for the pure Rayleigh scattering case

      open(10,file=fname,status='unknown')
      
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo
      close(10)
      
C---------modify an input file
      
      open(10,file=fname,status='unknown')      
      write(10,'(5e11.4,f10.7)') taumd,taur,tauc,totoz,totwat,wave
      write(10,'(a)') (str(k),k=2,5)
      write(10,'(f6.1,f8.5)') surfpres,kingf
      write(10,'(a)') (str(k),k=7,11)
      write(10,'(3e13.5)') c0(i),c1(i),c2(i)
      write(10,'(e13.5)') abs2(i)
      write(10,'(a)') (str(k),k=14,15)
      write(10,'(3i3)') maxnum,3,1
      write(10,'(a)') (str(k),k=17,18)
      write(10,'(f7.3)') sza
      write(10,'(a)') (str(k),k=20,21)
      write(10,'(a,6f8.1)') '.T. ',
     &vza(1),vza(nvza),15.0,phi(1),phi(nphi),30.0
      write(10,'(a)') (str(k),k=23,nstr)
      close(10)
      
      call system('flatpsv.x')            

      if(sza.lt. 0.95) then
	 write(ftop,'(''flatpsv.s00'',f2.1,''.bot'')') sza
      else if(sza.lt.9.95) then
         write(ftop,'(''flatpsv.s0'',f3.1,''.bot'')') sza
      else
         write(ftop,'(''flatpsv.s'',f4.1,''.bot'')') sza
      endif
           
      open(10,file=ftop,status='old')      
      do k=1,13
         read(10,'(a)') buff
      enddo      
      do j=1,nphi 
      do k=1,nvza     
      read(10,'(f6.2,f6.1,f6.2,f7.2,2e11.4)') theta,theta,theta,theta,
     &rad(i,k,j),t
      enddo
      enddo
      close(10)
      WRITE(*,*) 'RAD_NOAER',RAD(i,1,1)

      WRITE(*,*) 'RT CALCULATIONS COMPLETED'


      do j=1,nphi 
      do k=1,nvza     
      ratio(i,k,j)=rad_aer(k,j)/rad(i,k,j)
      enddo
      enddo

 100	continue	! skip radiance calculations

      call system('rm -f aerosol.phs')

      enddo		! end wavelength loop : lambda(i), i=1,icount
      
C----------interpolation of aerosol optical thickness & single scattering
C----------albedo on fixed wavelengths
      do k=1,icount
         lambuf(k)=lambda(k)/1.0E4
      enddo
      do k=1,12
        tauaer(k)=0.0
        omega(k)=0.0
        if(lam(k).ge.lambuf(1).and.lam(k).le.lambuf(icount)) then
        tauaer(k)=interpol(icount,lambuf,taum,lam(k),ierr)
        omega(k)=interpol(icount,lambuf,om,lam(k),ierr)
	WRITE(*,*) LAM(K),TAUAER(K),OMEGA(K)
        endif    
      enddo

      return
      end
       
      
     
