c $Header: /usr/people/flittner/radtran/flatpsv/src/atmstr.f,v 1.14 1999/08/16 18:57:13 flittner Exp flittner $
C########################################################################

	subroutine atmstr(nz,ttf,ttr,tto3,tth2o,totoz,totwat,
     &wavel,tf,tray,to3,th2o,zmod,presmod,aphasefile,ttc,cphasefile,tc,
     &sfcprs,Fk,o3coe,h2ocoe,top,dtaumax,aerodist,clddist,pcldtop,
     &pcldbot,iatmtype,lgcorrect,daveps,ndl,ierr,lchkdz,aerz0,aersig,
     &deltaoz,lthkcld)

C---------------------------------------------------------------------
C Oct. 4,1995: def; add clouds to model atmosphere.
c
c last modified: Nov. 12, 1996
c purpose: Pass switch for Henyey-Greenstein phase function
c
c last modified: July 22, 1997...def
c purpose: Change switch for H-G phase function to be flag for
c          size dist. type.
c
C
C This program calculates:
C 	the aerosol phase matrix
C	the aerosol single scattering albedo omega0
C	the total optical depth for each layer for aerosol,rayleigh,o3,h2o
C
C This program also computes the elevation for each layer zmod(nz) such that
C the total optical depth in each layer(dtau) is less than a specified 
C maximum optical depth.
C
C Definition of variables:
C	ttf=total aerosol optical depth for the atmosphere
C	ttr=total rayleigh optical depth for the atmosphere
C	tth2o=total water vapor absorption optical depth for the atmosphere
C 	tto3= total ozone absorption optical depth
C       ttc=total cloud optical depth for the atmosphere
C 	wavel=wavelength of interest in microns in free space
C 	rmin,rmax= smallest and biggest particle radii (microns)
C 	anu= power of d(f(r)/d(log r)=r**(-anu)
C 	sigma,rmean =standard deviation and mean radius (microns) of log-normal
C       	distribution for f(r). sigma (microns)
C 	refre,refim=real and imaginary index of refraction of the aerosols
C 	refmed=refractive index of the medium (the atmosphere)  
C
C************************************************************************
C
C-------- INPUT -----------------------------------------------
C input data (in free format):
C NOTE: If there are no aerosols the input data will be only steps 1 - 4
C  1.  ttf,ttr,tto3,tth2o,wavel
C  1a.  if tto3 = -1 then read in o3coe (o3 absorption coe.)
C  1b.  if tth2o = -1 then read in h2ocoe (h2o absorption coe.)
C  2.  dtaumax,top
C  3.  lines 3 through nang+1 should contain the line of sight angles
C  4.  iflag
C  5.  rmin,rmax
C  6.  anu(if iflag=1) or  sigma, rmean(if iflag=2)
C  7.  refre,refim,refmed
C
C------------ OUTPUT DATA AND FILES -----------------------------
C A) Output from this program that is used in 
C    subsequent flat atmosphere programs.
C   1. ttf,ttr,tto3,tth2o,wavel 
C   2. if ttf>0)   phase function file name
C   3. zmod(i),tf(i),tray(i),tabs(i),i=1,nz-1
C B) File 18 is the mie phase matrix output
C   1. cext,nang;   extinction & scattering cross-sections, number
C                         of tabulated angles
C   2. refre,refim,refmed; partical real and imaginary indicies of refraction,
C                          and medium index of refraction (default=1.0).
C   3. iflag
C   4. anu (if iflag=1) nu star for junge distribution
C      or sigma,rmean (if iflag=2) for log-normal distribution
C   5. T1 and T2 from Herman's disseration for scattering angles
C             0 to 180 degrees.
C   6. wavel;  wavelength
C   7. r(1);  min. particle radius
C   8. r(n1+1);  max particle radius
C   9. r(2)-r(1);  particle size increment
C C) File 31 is the output of the program atmstr
c

	implicit none
c
        include 'param.inc'
        include 'catmdata.cmn'
        include 'caerphs.cmn'
        include 'ccldphs.cmn'
c variables that are passed
	integer nz,iatmtype,ierr,ndl
        real*8 ttc,ttf,ttr,tto3,tth2o,wavel,sfcprs,totoz,totwat
        real*8 zmod(maxnz),tf(maxnz-1),tray(maxnz-1),presmod(maxnz-1)
        real*8 th2o(maxnz-1),to3(maxnz-1)
	character aphasefile*100,cphasefile*100
        real*8 Fk,o3coe(3),h2ocoe,top,dtaumax
        real*8 aerodist(22),clddist(22),pcldtop,pcldbot,aerz0,aersig
        real*8 deltaoz
        logical lgcorrect,daveps,lchkdz,lthkcld
c for cloud
        real*8 tc(maxnz-1)
c variables that are local
	real*8 omega0,cext,cextc,omega0c,g
c
        rayleigh=.TRUE.
        aerosol=.TRUE.
        ozone=.TRUE.
        water=.TRUE.
        cloud=.TRUE.
c now change if needed
        if(ttr.eq.0) rayleigh=.FALSE.
        if(ttf.eq.0) aerosol=.FALSE.
        if(ttc.eq.0) cloud=.FALSE.
        if(totoz.eq.0) ozone=.FALSE.
        if(totwat.eq.0) water=.FALSE.
c
c
        open(unit=31,file='atmstr.out',status='unknown') 
	write(31,*)' RESULTS FROM PROGRAM ATMSTR '
	
C WHEN TTF>0 THERE ARE AEROSOLS, FDIST AND PHASF ARE CALLED.
	if(aerosol)then
c call routine to calc the size dist. and phase matrix
          write(6,*)'atmstr. Will compute the aerosol size dist and',
     &' scattering phase matrix'
          write(31,*)'atmstr. Will compute the aerosol size dist and',
     &' scattering phase matrix'
          iflgaer=aerodist(1)
          if(aerodist(1).eq.6)then
            w0aer=aerodist(6)
            gaer=aerodist(7)
            omega0=w0aer
          else
            call cfnp(wavel,aerodist,aphasefile,omega0,cext,g,ierr)
	    if(ierr.ne.0)then
	      write(6,*)'atmstr. Error encountered in cfnp.'
              goto 5000
	    end if
            w0aer=omega0
            gaer=g
          endif
	else
	  omega0=0.0d0
          cext=0.0d0
          iflgaer=0
	endif
C WHEN ttc>0 THERE ARE CLOUDS, fdistcloud AND phascloud ARE CALLED AND
C THE CLOUD PHASE FUNCTION IS CALCULATED FOR GIVEN DROP SIZE DISTRIBUTION.
	if(cloud) then
          write(6,*)'atmstr. Will compute the cloud size dist and',
     &' scattering phase matrix'
          write(31,*)'atmstr. Will compute the cloud size dist and',
     &' scattering phase matrix'
          iflgcld=clddist(1)
          if(clddist(1).eq.6)then
            w0cld=clddist(6)
            gcld=clddist(7)
            omega0c=w0cld
          else
            call cfnp(wavel,clddist,cphasefile,omega0c,cextc,g,ierr)
	    if(ierr.ne.0)then
	      write(6,*)'atmstr. Error encountered in cfnp.'
              goto 5000
	    end if
            w0cld=omega0c
            gcld=g
          endif
        else
          omega0c=0.0d0
          cextc=0.0d0
          iflgcld=0
        endif
C Calculate optical depth in each layer of zmod for all atmospheric component
C using subroutine divide
	call divide(zmod,presmod,ttf,ttr,tto3,o3coe,tth2o,h2ocoe,wavel,
     &		dtaumax,top,omega0,nz,tf,tray,th2o,to3,
     &		ierr,cext,ttc,omega0c,tc,sfcprs,totoz,totwat,pcldtop,
     &          pcldbot,iatmtype,lgcorrect,daveps,ndl,lchkdz,
     &          aerz0,aersig,deltaoz,lthkcld)
	if(ierr.ne.0)then
	  write(6,*)'atmstr. Error encountered in divide.'
	end if


5000	close(31)
	return
        end

C#######################################################################
