c $Header: /usr/people/flittner/radtran/flatpsv/src/cmpsng.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
************************************************************************
      subroutine cmpsng(ndownlos,nlos,np,nz,
     &ipseudo,
     &costh,cossun,dt,PfoFmatrx,ProFmatrx,sflux,
     &esi0,esing0,pa,pal,pasi0,trans,wa,wr,z,
     &wc,PcoFmatrx,rsfc0,isfc,deltmu)
************************************************************************
C This subroutine calculates single scattered intensities in an 
C atmosphere with aerosols, rayleigh and absorption gases (O3,H2O).
C   esing0(i,n,j)=single scattered intensity (without contribution from 
C                 surface reflectivity) at height z(n) for l.o.s at 
C                 theta(i) and phi=(j-1)*df  
C   esi0(i,n,j)=single scattered intensity (including contribution from 
C               surface reflectivity) at height z(n) for l.o.s at 
C               theta(i) and phi=(j-1)*df  
C For a given line of sight angle theta defined by i, and for each
C layer defined by nlev:
C    1) Compute the path length for the path within the layer using
C       the layer optical depth and a 1/mu factor.
C    2) Check for simplified case of top of the atmosphere and looking 
C       up, then intensities are zero.
C    3) Check for simplified case of at the surface and looking down,
C       then intensities are simply the isotropic reflection of the
C       downward solar flux.
C    4) Otherwise do complete single scattering calculations by doing
C       the layer by layer summation along the path using subroutine 
C       integrt to compute pa(nlev) which is the path length from the 
C       sun to height nlev and is used for flux calculations.
C
C pa(i)=tau(i)/mu0
C pasi0=tau total / mu0
C pal(i,n)=dtau(n-1)/mui =optical path length in layer n-1 and view angle i
c trans(ilos,nlev)=exp(-dt(nlev-1)/costh(ilos))
c            =transmitance of nlev-1 layer in ilos direction
C th(I) is the line of sight angle theta in radians
C cossun is the solar angle
C ilos is the index associated with the line of sight angle
C nlev is the level at which the line of sight ends
C sflux is the solar flux at the top of the atmosphere
C ipseudo is the switch for the type of atmospheric attenuation of the
C         solar beam (ipseudo=0, flat; ipseudo=1,pseudo spherical).
C
C There are two cases to differentiate between, the upward and the downward
C case. This refers to the direction in which the radiation is travelling.
C
C last modified: May 5, 1997...def
C purpose: Delete use of refl.
c
      implicit none
      include 'consts.cmn'
      include 'param.inc'
c varibles that are passed as inputs:
      integer ndownlos,nz,nlos,np,ipseudo,isfc
      real*8 costh(mlos),dt(maxnlayr),cossun,sflux(4),
     &PfoFmatrx(4,nfi,mlos),ProFmatrx(4,nfi,mlos),
     &wa(maxnlayr),wr(maxnlayr),z(maxnz)
c for cloud
      real*8 wc(maxnlayr),PcoFmatrx(4,nfi,mlos)
c for surface
      real*8 rsfc0(4,4,nfi,mlos),deltmu(mlos)
c varibles that are passed as outputs: 
      real*8 esi0(4,nfi,mlos,maxnz),esing0(4,nfi,mlos,maxnz),
     &pa(maxnz),pal(mlos,maxnz),pasi0,trans(mlos,maxnz)
c varibles that are local
      integer nlayr,m,nlev,ilos,j,k,isun,n,layer
      real*8  efout(4),um,
     &pl,a1,a2,a3,a4,b,factor,sum2,atten,phase(4),wal,wrl
c for cloud
      real*8 wcl,sum
c
c zero-out the arrays esi0 and esing0
c
      do nlev=1,maxnz
         do ilos=1,mlos
            do k=1,nfi
               esi0(1,k,ilos,nlev)=0.0d0
               esi0(2,k,ilos,nlev)=0.0d0
               esi0(3,k,ilos,nlev)=0.0d0
               esi0(4,k,ilos,nlev)=0.0d0
            enddo
         enddo
      enddo
      do nlev=1,maxnz
         do ilos=1,mlos
            do k=1,nfi
               esing0(1,k,ilos,nlev)=0.0d0
               esing0(2,k,ilos,nlev)=0.0d0
               esing0(3,k,ilos,nlev)=0.0d0
               esing0(4,k,ilos,nlev)=0.0d0
            enddo
         enddo
      enddo
c
      nlayr=nz-1

c compute the solar attenuation to each level on the zenith 
c using either flat atmo, or spherical.
      call cmppa(nz,ipseudo,cossun,dt,z,pa)
      pasi0=pa(nz)
c layer path lengths and transmissions for downward traveling radiation
      nlev=1
      do ilos=1,ndownlos
         pal(ilos,nlev)=0.0d0
         trans(ilos,nlev)=1.0d0
      enddo
      do ilos=1,ndownlos
         um=1.0d0/costh(ilos)
         do nlev=2,nz
            pal(ilos,nlev)=dt(nlev-1)*um
            trans(ilos,nlev)=exp(-pal(ilos,nlev))
         enddo
      enddo
c
c downward radiation
c
c At the top of the atmo. and looking at downward radiation, the
c intensity is zero.  So, will not do this level.
c
      do ilos=1,ndownlos
         um=1.0d0/costh(ilos)
         do k=1,nfi
            do nlev=2,nz
               layer=nlev-1
               pl=pal(ilos,nlev)
               wal=wa(layer)
               wrl=wr(layer)
               wcl=wc(layer)
               isun=layer
               do m=1,np
                  phase(m)=(wal*PfoFmatrx(m,k,ilos)+
     &wrl*ProFmatrx(m,k,ilos)+wcl*PcoFmatrx(m,k,ilos))
c               if((ilos.eq.1).and.(k.eq.1).and.(nlev.eq.2))then
c                 write(6,*)'cmpsng. Pro',Profmatrx(m,k,ilos)
c                 write(6,*)'cmpsng. Pfo',Pfofmatrx(m,k,ilos)
c                 write(6,*)'cmpsng. Pco',Pcofmatrx(m,k,ilos)
c                 write(6,*)'cmpsng. Phs',phase(m)
c               endif
               enddo
               a1=(pa(isun+1)-pa(isun)) - pl
               if(pa(isun).ge.300.0d0)then
                  factor=0.0d0
               else if((a1.le.1.0d-10).and.
     &(a1.ge.-1.0d-10))then
                  b=pl*exp(-pa(isun)-pl)
                  factor=b
               else
                  a2=pl/a1
                  a3=exp(-pa(isun)-pl)
                  a4=1.0d0-exp(-a1)
                  factor=a2*a3*a4
               end if
               do m=1,np
                  sum2=phase(m)
                  efout(m)=sum2*factor
               enddo
               atten=exp(-pl)
               do m=1,np
                  esing0(m,k,ilos,nlev)=efout(m)+
     &esing0(m,k,ilos,nlev-1)*atten
                  esi0(m,k,ilos,nlev)=esing0(m,k,ilos,nlev)
c                  if((ilos.eq.1).and.(k.eq.1).and.(nlev.eq.2))then
c                    write(6,*)'cmpsng. ',efout(m),
c     &esing0(m,k,ilos,nlev-1),atten
c                  endif
               enddo
c               if((ilos.eq.1).and.(k.eq.1).and.(nlev.eq.2))stop
            enddo
         enddo
      enddo
c layer path lengths and transmissions for upward traveling radiation
      do ilos=ndownlos+1,nlos
         nlev=nz
         pal(ilos,nlev)=0.0d0
         trans(ilos,nlev)=1.00d0
         um=-1.0d0/costh(ilos)
         do nlev=nz-1,1,-1
            pal(ilos,nlev)=dt(nlev)*um
            trans(ilos,nlev)=exp(-pal(ilos,nlev))
         enddo
      enddo
c
c upward radiation
c
c At the surface and looking at upward radiation, the intensity
c is isotropic and I3 and I4 are zero.
      nlev=nz
      if(isfc.gt.0)then
        a1=exp(-pasi0)
        do ilos=ndownlos+1,nlos
           do j=1,nfi
              do m=1,np
                 sum=0.0d0
                 do n=1,np
                    sum=rsfc0(m,n,j,ilos)*sflux(n)+sum
                 enddo
                 esi0(m,j,ilos,nlev)=sum*a1
                 esing0(m,j,ilos,nlev)=sum*a1
              enddo
           enddo
        enddo
        sum=0.0d0
        do ilos=ndownlos+1,nlos
           a1=df*deltmu(ilos)
           do m=1,min(2,np)
              sum=sum+esi0(m,1,ilos,nlev)*a1
           enddo
           do m=1,min(2,np)
              sum=sum+esi0(m,nfi,ilos,nlev)*a1
           enddo
           do j=2,nfi-1
              do m=1,min(2,np)
                 sum=sum+esi0(m,j,ilos,nlev)*2.0d0*a1
              enddo
           enddo
        enddo
      endif
      do ilos=ndownlos+1,nlos
         um=-1.0d0/costh(ilos)
         do k=1,nfi
            do nlev=nz-1,1,-1
               layer=nlev
               pl=dt(layer)*um
               wal=wa(layer)
               wrl=wr(layer)
               wcl=wc(layer)
               isun=layer
               do m=1,np
                  phase(m)=(wal*PfoFmatrx(m,k,ilos)+
     &wrl*ProFmatrx(m,k,ilos)+wcl*PcoFmatrx(m,k,ilos))
               enddo
               a1=-(pa(isun+1)-pa(isun))-pl
               if(pa(isun).ge.300.0d0)then
                  atten=exp(-pl)
                  do m=1,np
                     esi0(m,k,ilos,nlev)=atten*
     &esi0(m,k,ilos,nlev+1)
                     esing0(m,k,ilos,nlev)=atten*
     &esing0(m,k,ilos,nlev+1)
                  enddo
               else
                  a2=pl/a1
                  a3=dexp(-pa(isun))
                  a4=1.0d0-dexp(a1)
                  factor=-a2*a3*a4
                  do m=1,np
                     sum2=phase(m)
                     efout(m)=sum2*factor
                  enddo
                  atten=exp(-pl)
                  do m=1,np
                     esi0(m,k,ilos,nlev)=efout(m)
     &+esi0(m,k,ilos,nlev+1)*atten
                     esing0(m,k,ilos,nlev)=efout(m)
     &+esing0(m,k,ilos,nlev+1)*atten
                  enddo
               end if
            enddo
         enddo
      enddo

      write(*,*)'cmpsng. DONE WITH SINGLE SCATTERING CALCULATIONS'
c
      return
      end

c ---------------------------------------------------------------------
