c $Header: /usr/people/flittner/radtran/flatpsv/src/cmpsngr.f,v 1.14 1999/08/16 18:59:08 flittner Exp flittner $
c ---------------------------------------------------------------------

        subroutine cmpsngr(ndownlos,nlos,np,nz,
     &	   costh,dt,Fu,esi0r,esingr)
C THIS SUBROUTINE CALCULATES SINGLE SCATTERING WITH AEROSOL, RAYLEIGH      
C AND ABSORPTION GASES (O3,H2O) for isotropic intensity incident 
c from the surface.
c______________________________________________________________________
C   ESING0(I,N,J)=SINGLE SCATTERING (WITHOUT CONTRIBUTION FROM SURFACE
C   REFLECTIVITY) AT HEIGHT Z(N) FOR L.O.S AT THETA(I) AND PHI=(J-1)*DF  
C   ESI0R(I,N,J)=SINGLE SCATTERING (INCLUDING CONTRIBUTION FROM SURFACE
C   REFLECTIVITY) AT HEIGHT Z(N) FOR L.O.S AT THETA(I) AND PHI=(J-1)*DF  
C FOR A GIVEN LINE OF SIGHT ANGLE THETA DEFINED BY I, AND FOR EACH
C LAYER DEFINED BY NLEV:
C    1) COMPUTE THE PATH LENGTH FOR THE PATH WITHIN THE LAYER USING
C       THE LAYER OPTICAL DEPTH AND A 1/MU FACTOR
C    2) CHECK FOR SIMPLIFIED CASE OF TOP OF THE ATMOSPHERE AND LOOKING 
C       UP, THEN INTENSITIES ARE ZERO
C    3) CHECK FOR SIMPLIFIED CASE OF AT THE SURFACE AND LOOKING DOWN,
C       THEN INTENSITIES ARE SIMPLY THE ISOTROPIC REFLECTION OF THE
C       DOWNWARD SOLAR FLUX.
C    4) OTHERWISE DO COMPLETE SINGLE SCATTERING CALCULATIONS BY DOING
C       THE LAYER BY LAYER SUMMATION ALONG THE PATH USING SUBROUTINE 
C	INTEGRT COMPUTE PA(NLEV) WHICH IS THE PATH LENGTH FROM THE 
C       SUN TO HEIGHT NLEV AND IS USED FOR FLUX CALCULATIONS
C
C Ig   =isotropic intensity reflected from sfc @ sfc.
C ILOS IS THE INDEX ASSOCIATED WITH THE LINE OF SIGHT ANGLE
C NLEV IS THE LEVEL AT WHICH THE LINE OF SIGHT ENDS
C SFLUX IS THE SOLAR FLUX AT THE TOP OF THE ATMOSPHERE
C
C THERE ARE TWO CASES TO DIFFERENTIATE BETWEEN, THE UPWARD AND THE DOWNWARD
C CASE. THIS REFERS TO THE DIRECTION IN WHICH THE RADIATION IS TRAVELLING

      implicit none
      include 'param.inc'
      include 'consts.cmn'
c varibles that are passed as inputs:
	integer ndownlos,nz,nlos,np
	real*8 costh(mlos), dt(maxnlayr)
	real*8 Fu
c varibles that are passed as outputs: 
	real*8 esi0r(4,mlos,maxnz),esingr(4,mlos,maxnz)
c varibles that are local
	integer m,nlev,ilos
	real*8  tautot,um,Ig,e
c


        if(np.gt.1)then
           Ig=Fu*0.5d0/pi
        else
           Ig=Fu/pi
        endif
c
c downward radiation have no direct radiation when source is upward at the
c surface.  Will compute the scattered radiation using downit. (I know I'm
c being lazy.)  Will set all downward radiation to zero.
c
        do nlev=1,nz
	   do ilos=1,ndownlos
              do m=1,np
                 esi0r(m,ilos,nlev)=0.0d0
              enddo
           enddo
        enddo
c
c upward radiation
c
c At the surface and looking at upward radiation, the intensity
c is isotropic and I3 and I4 are zero.
	do ilos=ndownlos+1,nlos
c m=1
           esi0r(1,ilos,nz)=Ig
c m=2
           if(np.gt.1)then
             esi0r(2,ilos,nz)=Ig
             do m=3,np
                esi0r(m,ilos,nz)=0.0d0
             enddo
           endif
	enddo
        tautot=0.0d0
        do nlev=nz-1,1,-1
           tautot=tautot+dt(nlev)
	   do ilos=ndownlos+1,nlos
	      um=-1.0d0/costh(ilos)
c m=1
              e=Ig*exp(-tautot*um)
              esi0r(1,ilos,nlev)=e
	      if(np.gt.1)then
c m=2
                esi0r(2,ilos,nlev)=e
                do m=3,np
                   esi0r(m,ilos,nlev)=0.0d0
                enddo
              endif
	   enddo
	enddo
        do nlev=1,nz
	   do ilos=1,nlos
              do m=1,np
                 esingr(m,ilos,nlev)=esi0r(m,ilos,nlev)
              enddo
           enddo
        enddo

      write(*,*)' DONE WITH SINGLE SCATTERING CALCULATIONS'

      return
      end
c ---------------------------------------------------------------------
