c $Header: /home/loughman/flatpsv/src/crsfc.f,v 1.3 2000/04/04 16:08:23 loughman Exp loughman $
c ---------------------------------------------------------------------
      subroutine crsfc(nlos,mlos,nfi,ndownlos,np,isfc,thd,dmu,deltmu,
     &phi,sun,i,phi0,refl,thdbnd,phibnd,rsfc,idirect,ierr)
c
c Compute the surface reflectance matrix light incident from the direction 
c theta=sun, phi=phi0 and the output direction theta=thdj, phi=phik.
c This is related to the brdf by cos(sun).  For the case of idirect=1,
c the incident light is plane parallel sunlight or flux.  When
c idirect=0, the incident light field is diffuse intensity and the 
c reflectance matrix is averaged over the incident solid angle bin.
c This is similar to the scattering matrix for aerosols, clouds, and
c molecules.  For this case rsfc is equal to 
c
c rsfc = int brdf*cos(theta_in)*domega / int domega.
c
c In the case of a Lambertain surface, the brdf = 1/pi and the 
c rsfc=(cos(theta_in1)^2-cos(theta_in2)^2)/(cos(theta_in1)-cos(theta_in2))/pi/2
c rsfc=deltmu(theta_in)/dmu(theta_in)/pi
c
c last modified: April, 15, 1997...def
c purpose: add the rough ocean reflecing option
c
c last modified: April 17, 1997...def
c purpose: add mirror reflecting surface
c
c last modified: May 5, 1997...def
c purpose: Use refl in rsfc explicitly.
c
c last modified: May 6, 1997...def
c purpose: Average the brdf for ice model.
c
c last modified: July 31, 1997...def
c purpose: pass info for ocean model thru the cocean common blk
c
c last mod. Mar. 27, 2000...def
c purpose: replace brdfice with brdfice routine.
c
c 
      implicit none
      include 'consts.cmn'
      include 'cocean.cmn'
c varibles that are passed
      integer ndownlos,nlos,mlos,nfi,np,isfc,i,idirect,ierr
      real*8 thd(mlos),sun,rsfc(4,4,nfi,mlos),phi0,dmu(mlos)
      real*8 deltmu(mlos),phi(nfi),refl
      real*8 thdbnd(nlos+1),phibnd(nfi+1)
c variables that are local
      integer mode,k,j,n,m,ip
      real*8 c2,c1,sum,domega
      real*4 rho,tht,tht0
      real*8 V,refi,refr,thtout,phiout,phiin
      integer maxthdp,maxphip
      parameter (maxthdp=10,maxphip=30)
      real*8 thdp(maxthdp),phip(maxphip),tmp1(maxthdp,maxphip)
      real*8 sum1,sum2,sum3,sum4,sum5,dy,dx,mup(maxthdp),dphip
      real*8 dthdp,sum6,sum7,sum8,sum9,sum10,sum11,sum12,sum13
      real*4 thtp,thedelphi
      integer ii,jj
      logical just1ang,debug
c
      debug=.FALSE.
      if((nlos.eq.1).and.(nfi.eq.1))then
        just1ang=.TRUE.
      else
        just1ang=.FALSE.
      endif
c
      if(debug)write(6,*)'crsfc. sun phi0= ',sun,phi0
      ip=min(2,np)
      if(isfc.eq.1)then
c lambertian surface
        c1=dcos(sun*dr)
c calc the integration over upward flux (this should be equal to pi) if
c the number of theta angles is greater than 1
        if(nlos.gt.1)then
          sum=0.0d0
          do j=ndownlos+1,nlos
             sum=sum-deltmu(j)
          enddo
          sum=sum*dble(2*nfi-2)*df
        else
          sum=pi
        endif
        if((c1.gt.0.0d0))then
          if(idirect.eq.1)then
c direct solar beam
            c2=c1/sum*refl
          else
c diffuse field incident (must be normalized to diffuse flux after
c multiplying by incident intensity and then integrating over all angles.
c So multiply by interval for flux and divide by the interval for solid angle.
c cos(incident polar angle)*dmu(i) is close to deltmu(i), but not quite.)
            c2=deltmu(i)/sum/dmu(i)*refl
          endif
        else
          c2=0.0d0
        endif
        if(np.gt.1)c2=c2*0.5d0
        do k=1,nfi
           do j=ndownlos+1,nlos
              if(np.gt.1)then
                rsfc(1,1,k,j)=c2
                rsfc(1,2,k,j)=c2
                rsfc(2,1,k,j)=c2
                rsfc(2,2,k,j)=c2
                do m=1,np
                   do n=3,np
                      rsfc(m,n,k,j)=0.0d0
                   enddo
                   if(m.gt.2)then
                     rsfc(m,3,k,j)=0.0d0
                     rsfc(m,4,k,j)=0.0d0
                   endif
                enddo
              else
                rsfc(1,1,k,j)=c2
              endif
           enddo
        enddo
      else if(isfc.eq.2)then
        sum11=0.0
        sum12=0.0
        sum13=0.0
c use jaross's avhrr-ice brdf
        if(idirect.eq.1)then
c direct solar beam
          c1=dcos(sun*dr)*refl
        else
c diffuse field incident (must be normalized to diffuse flux after
c multiplying by incident intensity and then integrating over all angles.
c So multiply by interval for flux and divide by the interval for solid angle.
c cos(incident polar angle)*dmu(i) is close to deltmu(i), but not quite.)
          c1=deltmu(i)/dmu(i)*refl
        endif
        tht0=real(sun)
        do k=1,nfi
c create phi averaging grid for this bin
           call mkbingrid(phibnd(k+1),phibnd(k),phip,maxphip)
           do j=ndownlos+1,nlos
              if(k.eq.1.or.k.eq.nfi)then
                domega=dmu(j)*df
              else
                domega=dmu(j)*2.0*df
              endif
              mode=1
              tht=180.0-real(thd(j))
c
c for each output bin centered on (thd(j),phi(k)), average over the bin
c
c create theta averaging grid for this bin
              call mkbingrid(thdbnd(j+1),thdbnd(j),thdp,maxthdp)
              do ii=1,maxthdp
                 mup(ii)=cos(thdp(ii)*dr)
              enddo
c
c get the brdf values for the first thdp value
c
              mode=1
              sum=0.0d0
              do jj=1,maxphip
                 if(just1ang)then
                   thedelphi=real((acos(cos((phi(1)-phip(jj))*dr)))/dr)
                 else
                   thedelphi=real((acos(cos((phip(jj)-phi0)*dr)))/dr)
                 endif
                 do ii=1,maxthdp
                    if(just1ang)then
                      tht0=real(thdp(ii))
                      thtp=180.0-real(thd(j))
                    else
                      tht0=real(sun)
                      thtp=180.0-real(thdp(ii))
                    endif
                    call brdfice(mode,rho,tht0,thtp,thedelphi)
                    if(mode.eq.2)then
                      write(6,*)'crsfc. error calling brdfice. will',
     &' stop now'
                      ierr=1
                      return
                    endif
                    tmp1(ii,jj)=dble(rho)
                 enddo
              enddo
c sum1 & sum2=integral of brdf over sin*dtheta
c sum3=integral of brdf over solid angle
c
c sum4=integral of 1.0  over sin*dtheta
c sum5=integral of 1.0  over solid angle
c
c sum6=integral of cos  over sin*dtheta
c sum10=integral of cos  over solid angle
c
c sum7 & sum8=integral of cos*brdf over sin*dtheta
c sum9=integral of cos*brdf over solid angle
              sum1=0.0d0
              sum4=0.0d0
              sum6=0.0d0
              sum7=0.0d0
              do ii=1,maxthdp-1
                 dx=mup(ii+1)-mup(ii)
                 sum1=sum1+0.5d0*dx*(tmp1(ii,1)+tmp1(ii+1,1))
                 sum4=sum4+dx
                 sum6=sum6+0.5d0*dx*(mup(ii+1)+mup(ii))
                 sum7=sum7+0.5d0*dx*(mup(ii+1)*tmp1(ii+1,1)+
     &                mup(ii)*tmp1(ii,1))
              enddo
c
              sum3=0.0d0
              sum5=0.0d0
              sum9=0.0d0
              sum10=0.0d0
              do jj=2,maxphip
                 sum2=0.0d0
                 sum8=0.0d0
                 dy=dr*(phip(jj)-phip(jj-1))
                 do ii=1,maxthdp-1
                    dx=mup(ii+1)-mup(ii)
                    sum2=sum2+0.5d0*dx*(tmp1(ii,jj)+tmp1(ii+1,jj))
                    sum8=sum8+0.5d0*dx*(mup(ii+1)*tmp1(ii+1,jj)+
     &                mup(ii)*tmp1(ii,jj))
                 enddo
                 sum3=sum3+0.5d0*dy*(sum1+sum2)
                 sum1=sum2
                 sum5=sum5+dy*sum4
                 sum10=sum10+dy*sum6
                 sum9=sum9+0.5d0*dy*(sum7+sum8)
                 sum7=sum8
              enddo
              sum11=sum11+sum10
              sum12=sum12+sum5
              sum13=sum13+sum9
c                 
c compare average to center value
              tht0=real(sun)
              tht=180.0-real(thd(j))
              thedelphi=real((acos(cos((phi(k)-phi0)*dr)))/dr)
              call brdfice(mode,rho,tht0,tht,thedelphi)
              if(debug)write(6,4000)'crsfc. avg rho',thdbnd(j),
     &thdbnd(j+1),phibnd(k),phibnd(k+1),sum3,sum5,sum3/sum5,rho,
     &sum9/sum10
c
c will average with mu weighting
c
              c2=c1*sum9/sum10
4000  format(a13,4f6.1,1P5E9.2)
              if(np.gt.1)then
                c2=c2*0.5d0
                rsfc(1,1,k,j)=c2
                rsfc(1,2,k,j)=c2
                rsfc(2,1,k,j)=c2
                rsfc(2,2,k,j)=c2
                do m=1,np
                   do n=3,np
                      rsfc(m,n,k,j)=0.0d0
                   enddo
                   if(m.gt.2)then
                     rsfc(m,3,k,j)=0.0d0
                     rsfc(m,4,k,j)=0.0d0
                   endif
                enddo
              else
                rsfc(1,1,k,j)=c2
              endif
           enddo
        enddo
c        write(6,'(a8,3e12.3)')'sum11 =',sum11*2./pi,sum12*2./pi,sum13*2.
      else if((isfc.eq.3).or.(isfc.eq.4))then
c use rough-ocean surface model or mirror surface model
c
        sum=0.0d0
        phiin=phi0*dr
c set the index of refraction of the frenel reflector
        refr=refrocn
        refi=refiocn
        do k=1,nfi
           phiout=phi(k)*dr
           do j=ndownlos+1,nlos
              thtout=180.0-thd(j)
              if(isfc.eq.3)then
c use rough ocean
c
c              domega=dmu(j)*df/cos(sun*dr)
                domega=1.0d0
c
                call brdfsea(thtout,phiout,sun,phiin,refr,refi,
     &vocean,pheevocean,rsfc(1,1,k,j),domega)
              else
c use mirror
                if(idirect.eq.1)then
c for direct solar beam must divide the energy into the whole
c solid angle bin of the reflected direction
                  domega=dmu(j)*df
                else
c for diffuse skylight the solid angle bins are the same for
c input and output, but the output intensity is calculated
c by integrating input intensity times rsfc over solid angle
c for all input directions.  So, must still divide this energy
c into the whole solid angle bin.
                  domega=dmu(j)*df
                endif
c
                call brdfmir(thtout,phiout,sun,phiin,refr,refi,
     &rsfc(1,1,k,j),domega)
              endif
              if(k.eq.1.or.k.eq.nfi)then
                sum=sum-deltmu(j)*df*
     &0.5d0*(rsfc(1,1,k,j)+rsfc(1,2,k,j)+
     &rsfc(2,1,k,j)+rsfc(2,2,k,j))
              else
                sum=sum-deltmu(j)*df*2.0*
     &0.5d0*(rsfc(1,1,k,j)+rsfc(1,2,k,j)+
     &rsfc(2,1,k,j)+rsfc(2,2,k,j))
              endif
              if(np.eq.1)then
                rsfc(1,1,k,j)=0.5d0*(rsfc(1,1,k,j)+rsfc(1,2,k,j)+
     &rsfc(2,1,k,j)+rsfc(2,2,k,j))
                rsfc(1,2,k,j)=0.0d0
                rsfc(2,1,k,j)=0.0d0
                rsfc(2,2,k,j)=0.0d0
              endif
           enddo
        enddo
        write(6,*)'crsfc. sum= ',sum,sun
      else
        write(6,*)'crsfc. wrong surface type. must have isfc=0-4'
        ierr=1
      endif
      return
      end

      subroutine mkbingrid(phihi,philo,phip,maxphip)
c
c create phi averaging grid for this bin
c
      implicit none
c passed
      integer maxphip
      real*8 phihi,philo,phip(maxphip)
c local
      integer ii
      real*8 dphip
c
      dphip=(phihi-philo)/dble(maxphip-1)
      phip(1)=philo
      do ii=2,maxphip-1
         phip(ii)=phip(ii-1)+dphip
      enddo
      phip(maxphip)=phihi
      return
      end
