c $Header: /home/vasilkov/GSCODE/divide.f,v 1.2 2000/05/12 14:07:28 loughman Exp $
c***********************************************************************

       subroutine divide(zmod,presmod,ttf,ttr,tto3,o3coe,tth2o,h2ocoe,
     &wavel,dtaumax,top,omega0,nz,tf,tray,th2o,to3,
     &ierr,cext,
     &ttc,omega0c,tc,sfcprs,totoz,totwat,pcldtop,pcldbot,iatmtype,
     &lgcorrect,daveps,ndl,lchkdz,aerz0,aersig,deltaoz,lthkcld)
C-----------------------------------------------------------------------
C  this subroutine determines the zmod(i) elevation array using the
C  following criterion:
C
C 1.  A total scatter optical depth in a layer no greater than dtaumax
C 2.  Or uses the Dave pressure scheme 
C 3.  Thick clouds with either of the above 2 outside the cloud layer
C
C--VARIABLES--
C    INPUT:
C zmod=starting altitude scale in km. zmod(nz)=0.0   zmod(1)=top
C maxnz=max. dimension of altitude array
C ttf=total aerosol optical depth for atmosphere 
C ttr=total rayleigh optical depth 
C tth2o=total water vapor absorption optical depth (= -1 if unknown)
C tto3=total ozone absorption optical depth (= -1 if unknown)
C dtaumax=maximum optical depth per layer
C top=height of atmosphere
C omega0=aerosol single scatter albedo
C
C    OUTPUT:
C nz=number of layers needed to divide atmosphere
C tf=aerosol optical depth per layer
C tray=rayleigh optical depth per layer
C th2o=water vapor optical depth per layer
C to3=ozone optical depth per layer
c ierr=error flag equal to 1 if problems encountered in divide.
c           This will cause atmstr to quit without producing catalog of results.
C
C last modified: Nov. 22, 1996...def
C purpose: allow sfc to be in the cloud. fix some problems with the
C calculation of zmax.
C
C last modified: April 15, 1997...def
C purpose: check for zero optical depths in o3 and h2o and zero-out
C          appropriate quantities if so.
C
C last modified: June 17, 1997...def
C purpose: add atmo type umkehr so can use umkehr profile generated by 
C          C. Mateer.  Also fix the call to spline aerosol conc.
C
C last modified: Oct. 7, 1997...def
C purpose: Check for scale heights lt 0.0
C
C last modified: Apr. 29, 1998...def
C purpose: Correct check for too many levels for daveps option.
c
c last mod. Oct. 22, 1998...def
c purpose: Initialize sums to 0.
c
c last mod. July, 15, 1999...def
c purpose: read atmosphere files created by modtran.
C
C-----------------------------------------------------------------------
       implicit none

       include 'consts.cmn'
       include 'param.inc'
       include 'catmdata.cmn'
       include 'cstdatm.cmn'
c
c passed
       integer nz,ierr,iatmtype,ndl
       logical lgcorrect,daveps,lchkdz,lthkcld
       real*8 zmod(maxnz),presmod(maxnz-1),tf(maxnz-1),tray(maxnz-1)
       real*8 to3(maxnz-1),th2o(maxnz-1),tc(maxnz-1),ttf,ttr,tto3,tth2o
       real*8 o3coe(3),h2ocoe,wavel,dtaumax,top,omega0
       real*8 cext,ttc,omega0c
       real*8 sfcprs,totoz,totwat,aerz0,aersig,deltaoz
       real*8 pcldtop,pcldbot
c
c local
       logical sort,prtf
c NN=number of data values for aerosol, water vapor concentration,
c ozone concentration and pressure
       integer NN
       PARAMETER(NN=140)
       integer npr,naer,noz,nh2o,ntemp,usemode
       integer k,nlayers,mode,j,i
       logical monoton,luseintdtau,lusezstd
       real*8 zpres(NN),pres(NN),zaer(NN),aer(NN),zoz(NN),oz(NN)
       real*8 zwat(NN),wat(NN),ztemp(NN),temp(NN),y2(NN),yp1,ypn,zsfc
       real*8 tautot,root,lastdtau,tautots
       real*8 tftop,prestop,tau,aeravg,sum
       real*8 sumt,dtdelz,tfbot,sumf,sumr
       real*8 sumo3,sumh2o,doz,dwat,watavg,ozavg,delz,coeff,tempavg
       real*8 dtau,dtt,dta,dts
c local for cloud
       integer ncld
       real*8 zc(NN),cld(NN),cavg,sumc,tctop,tcbot
       real*8 zcldtop,zcldbot
       real*8 intcld(N),intpres(N),intaer(N)
       real*8 intoz(N),intwat(N),inttmp(N)
       real*8 y2b(N)
       real*8 z1,rjunk1,rjunk2,ttr0
       integer iextra,one,ndavelay,nlay1,nlay2,nlay3
c
       real*8 zmax,zmin,beta,gratio,ttcldtop,ttcldbot
       real*8 dttcdz,oneatm,ztop
       real*8 zlev(6)
       integer ivelzd(6),ien,ist
c for the input
       character*100 prsfname,aerfname,o3fname,h2ofname,tmpfname
       common /cprofname/prsfname,aerfname,o3fname,h2ofname,tmpfname
       real*8 dzlay
       logical lmorelays
       common /catmmisc/dzlay,lmorelays
       save /cprofname/,/catmmisc/
c functions
       real*8 davedps
       external davedps
c
       integer imodt
c**************************************************************
       ierr=0
       dzlay=1.0d0
       luseintdtau=.false.
       usemode=4
       sort=.FALSE.
       one=1
       prtf=.FALSE.
       oneatm=1013.25d0
       zmax=100.0d0
       zmin=0.0d0
       ztop=100.0d0
       lusezstd=.FALSE.
c
c initialize the optical depth arrays to zero
c
       i=maxnz-1
       call setzeros(tray,i)
       call setzeros(tf,i)
       call setzeros(to3,i)
       call setzeros(th2o,i)
       call setzeros(tauray,N)
       call setzeros(tauf,N)
       call setzeros(tauo3,N)
       call setzeros(tauh2o,N)
c
       if(rayleigh)then
         beta=ttr
       else
         beta=0.0d0
       endif
c create zstd array
c alt.   dz
c 1-0    @0.05
c 3-1    @0.1
c 7-3    @0.2
c 55-7   @0.5
c 85-55  @0.5
c 100-85 @1
       zlev(6)=100.0d0
       zlev(5)=85.0d0
       zlev(4)=55.0d0
       zlev(3)=7.0d0
       zlev(2)=3.0d0
       zlev(1)=1.0d0
       ivelzd(6)=1
       ivelzd(5)=2
       ivelzd(4)=2
       ivelzd(3)=5
       ivelzd(2)=10
       ivelzd(1)=20
c
       ist=1
       zstd(1)=0.0d0
       do j=1,6
          if(j.eq.1)then
            ien=ivelzd(1)*nint(zlev(1))+1
            ist=2
          else
            ien=ien+ivelzd(j)*nint(zlev(j)-zlev(j-1))
          endif
          do i=ist,ien,1
             zstd(i)=zlev(j)-dble(ien-i)/dble(ivelzd(j))
             if(zstd(i).ge.zmax)then
               nstd=i
               goto 25
             endif
          enddo
          nstd=ien
          ist=ien+1
       enddo
25     zstd(nstd)=zmax
c
c       do 20 i=1,N-1
cc 0.0 to 1.0 km
c          if(i.le.21)then
c            zstd(i)=(i-1)*0.05d0
cc 1.0 to 3.0 km
c          else if(i.le.41)then
c            zstd(i)=(i-11)*0.1d0
cc 3.0 to 7.0 km
c          else if(i.le.61)then
c            zstd(i)=(i-26)*0.2d0
cc 7.0 to 85.0 km
c          else if(i.le.217)then
c            zstd(i)=(i-47)*0.5d0
cc 85.0 to 100.0 km
c          else if(i.le.232)then
c            zstd(i)=(i-132)*1.0d0
c          end if
c          if(zstd(i).ge.zmax)goto 25
c20     continue
c25     zstd(i)=zmax
c       nstd=i
       do i=1,nstd-1
          dzstd(i)=zstd(i+1)-zstd(i)
       end do
c open constituent files
       if(iatmtype.eq.1)then
         call rdproname(zpres,pres,npr,ztemp,temp,ntemp,zoz,oz,noz,
     &zaer,aer,naer,NN)
         if(zmax.gt.zpres(npr))zmax=zpres(npr)
         if(zmin.lt.zpres(1))zmin=zpres(1)
c zero-out the water vapor values
         totwat=0.0d0
       else if(iatmtype.eq.4)then
c convert Krotkov's disort atmo into gauss-seidel atmo
	 call disort2gs(zmod,presmod,ttf,ttr,tto3,tth2o,
     &dtaumax,nz,tf,tray,th2o,to3,ierr,ttc,tc)
         goto 200
       else if(iatmtype.eq.5)then
         call useumkehr(
     &NN,npr,zpres,pres,aer,oz,y2,zmax,intpres,
     &totoz,totwat,sfcprs,ttc,ttf,ttr,tto3,tth2o)
         lusezstd=.FALSE.
         goto 70
       else if(iatmtype.eq.6)then
c read modtran 35 layer atmosphere
         imodt=16
         open(unit=imodt,file='def.dat',status='old')
         call rdmodtrnatm(imodt,sfcprs,top,prtf,
     &NN,npr,naer,ntemp,noz,nh2o,ncld,zpres,ztemp,zaer,zoz,zwat,zc,
     &pres,temp,oz,aer,cld,wat,
     &y2,intpres,totoz,totwat,ttc,ttf,ttr,tto3,tth2o,oneatm)
         i=maxnz-1
         call setzeros(tc,i)
         call setzeros(tauc,N)
         goto 70
       else
c will use cubic spline on log of concentration for aerosol, water vapor,
c and ozone.
c         if(rayleigh)then
           call rdin(zpres,pres,NN,npr,prsfname,ierr)
           if(ierr.ne.0)goto 3000
           if(zmax.gt.zpres(npr))zmax=zpres(npr)
           if(zmin.lt.zpres(1))zmin=zpres(1)
c         end if
  
         if(aerosol)then
           if(iatmtype.eq.2.or.iatmtype.eq.3)then
           else
             call rdin(zaer,aer,NN,naer,aerfname,ierr)
             if(ierr.ne.0)goto 3000
             if(zmax.gt.zaer(naer))zmax=zaer(naer)
             if(zmin.lt.zaer(1))zmin=zaer(1)
           endif
         end if

         if(ozone)then
           call rdin(zoz,oz,NN,noz,o3fname,ierr)
           if(ierr.ne.0)goto 3000
           if(zmax.gt.zoz(noz))zmax=zoz(noz)
           if(zmin.lt.zoz(1))zmin=zoz(1)
         else
           totoz=0.0d0
         end if
  
         if(water)then
           call rdin(zwat,wat,NN,nh2o,h2ofname,ierr)
           if(ierr.ne.0)goto 3000
           if(zmax.gt.zwat(nh2o))zmax=zwat(nh2o)
           if(zmin.lt.zwat(1))zmin=zwat(1)
         else
           totwat=0.0d0
         end if

         if(ozone)then
           call rdin(ztemp,temp,NN,ntemp,tmpfname,ierr)
           if(ierr.ne.0)goto 3000
         end if
       endif
c will use "natrual" spline
       yp1=1.0d30
       ypn=1.0d30
       mode=2
c what is the pressure at the top of this profile (@ z=top)?
       call spline(zpres,pres,npr,yp1,ypn,y2,mode)
       call splint(zpres,pres,y2,npr,top,prestop,mode)
       write(6,2000)prestop
c what is the pressure at the bottom of this profile (@ z=0)?
       write(6,2010)zpres(1),pres(1)
c will normalize pressure profile to bottom of the input profile,
c so will set oneatm equal to pres(1)
       oneatm=pres(1)
c what is the alt. of the reflecting surface?
       if(abs(pres(1)-sfcprs).gt.smallest)then
         mode=1
         call spline(pres,zpres,npr,yp1,ypn,y2,mode)
         call splint(pres,zpres,y2,npr,sfcprs,zsfc,mode)
       else
         zsfc=zpres(1)
       endif
       write(6,2020)zsfc,sfcprs
c
       if(cloud) then
c will use "natrual" spline
         yp1=1.0d30
         ypn=1.0d30
         mode=1
c what is the pressure at the top of this profile (@ z=top)?
         call spline(pres,zpres,npr,yp1,ypn,y2,mode)
         call splint(pres,zpres,y2,npr,pcldtop,zcldtop,mode)
         call splint(pres,zpres,y2,npr,pcldbot,zcldbot,mode)
c check to see if the cloud bottom is below the ground surface (zsfc>zcldbot)
c if so, then addjust the total cloud optical depth assuming constant
c drop density.
         if(zsfc.gt.zcldbot)then
           ttc=ttc*(zcldtop-zsfc)/(zcldtop-zcldbot)
           zcldbot=zsfc
         endif
         zc(npr)=zpres(npr)
         cld(npr)=0.0d0
         ncld=npr
         call mkcld(zcldtop,zcldbot,ttc,zc,cld,NN,ncld)
c         call rdin(zc,cld,NN,ncld,fname,ierr)
c         if(ierr.ne.0)goto 3000
       else
         i=maxnz-1
         call setzeros(tc,i)
         call setzeros(tauc,N)
       endif
c readjust the top of the zstd array
       i=1
       do while(zstd(i).lt.zmax)
          i=i+1
       enddo
       nstd=i
c adjust the bottom of the zstd array match the input files
       if(zstd(1).lt.zmin)then
         zstd(1)=zmin
         i=2
C         do while(zstd(i).lt.zstd(1))
C*****************************************************
         do while(zstd(i).le.zstd(1))
C******************************************************
            i=i+1
         enddo
         if(i.gt.2)then
           do j=i,nstd
              zstd(2+(j-i))=zstd(j)
           enddo
           nstd=nstd-(i-2)
         endif
       endif
c Now interpolate profiles to find values at zstd.
       do i=1,nstd
          ttotal(i)=0.0d0
       enddo
c Then will integrate profiles to get tau aerosol, rayleigh, ozone, water
c vapor, and total versus height.
c       if(rayleigh)then
         mode=2
         call spline(zpres,pres,npr,yp1,ypn,y2,mode)
         do i=1,nstd
            call splint(zpres,pres,y2,npr,zstd(i),intpres(i),mode)
         enddo
c scale height for alt. above zstd(nstd)
         Hp=(dzstd(nstd-1))/log(intpres(nstd-1)/intpres(nstd))
         write(6,'(a31,1pE13.4,a6)')'divide. Pressure scale height=',
     &Hp,' (km).'
         if(prtf)then
           open(92,file='intpres.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),intpres(i)
           enddo
           close(92)
         endif
       if(rayleigh)then
c beta is the rayleigh scattering per atmosphere, but since the number
c density is not constant with alt. due to change in gravity with
c alt., we must adjust the scattering coef. by the ratio of gravity
c at the surface to gravity at this alt.  No correction is done for 
c alt. below 0.0.
         do 30 k=1,nstd
            if(lgcorrect.and.(zstd(k).gt.0.0d0))then
              gratio=(1.0d0+zstd(k)/Re)**2
            else
              gratio=1.0d0
            endif
            tauray(k)=beta*intpres(k)*gratio/oneatm
30       continue
         if(ttr.ne.tauray(1)) then
           write(6,'(a36,1PE11.4,a4,1PE11.4)')
     &'divide. must normalize tauray from ',tauray(1),' to',ttr
           call normalize(ttr,tauray,nstd,tauray(1))
         endif
         do i=1,nstd
            ttotal(i)=tauray(i)+ttotal(i)
         end do
         ttr0=ttr
c adjust ttr for elevated surface
         gratio=1.0d0
         if(lgcorrect.and.(zsfc.gt.0.0d0))gratio=(1.0d0+zsfc/Re)**2
         ttr=beta*sfcprs/oneatm*gratio
         write(6,*)'divide. ttr =',ttr
       end if

       if(aerosol)then
         if(iatmtype.eq.2.or.iatmtype.eq.3)then
           rjunk1=1.0d0/sqrt(2.0d0*pi*aersig*aersig)
           rjunk2=0.5d0/(aersig*aersig)
           do i=1,nstd
              intaer(i)=rjunk1*exp(-rjunk2*(zstd(i)-aerz0)**2)
           enddo
         else
c cubic interpolation, but don't over or under shoot the data points
           mode=0
           monoton=.true.
           sort=.false.
           call smoothb(zaer,aer,naer,NN,sort,zstd,intaer,nstd,N,
     &ierr,mode,monoton)
         endif
         if(prtf)then
           open(92,file='intaer.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),intaer(i)
           enddo
           close(92)
         endif
c use scale height for aerosol loading above level zstd(nstd)
         if(intaer(nstd).ne.intaer(nstd-1))then
           Hf=(dzstd(nstd-1))/log(intaer(nstd-1)/intaer(nstd))
           if(Hf.lt.0.0)Hf=0.0
         else
           Hf=0.0d0
         endif
         write(6,'(a30,1pE13.4,a6)')'divide. aerosol scale height=',
     &Hf,' (km).'
         tauf(nstd)=Hf*intaer(nstd)
         do 40 k=nstd-1,1,-1
            aeravg=(0.5d0*(intaer(k+1)+intaer(k)))
            tauf(k)=tauf(k+1)+aeravg*dzstd(k)
40       continue
         if(ttf.ne.tauf(1)) then
           write(6,'(a36,1PE11.4,a4,1PE11.4)')
     &'divide. must normalize tauf from   ',tauf(1),' to',ttf
           call normalize(ttf,tauf,nstd,tauf(1))
         endif
         do i=1,nstd
            ttotal(i)=tauf(i)+ttotal(i)
         end do
c adjust ttf for elevated surface
         mode=0
         call spline(zstd,tauf,nstd,yp1,ypn,y2b,mode)
         call splint(zstd,tauf,y2b,nstd,zsfc,ttf,mode)
         mode=0
         monoton=.true.
         sort=.false.
         call smoothb(zstd,tauf,nstd,nstd,sort,zsfc,ttf,one,one,
     &ierr,mode,monoton)
         if(prtf)then
           open(92,file='tauf.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),tauf(i)
           enddo
           close(92)
         end if
       end if

       if(cloud)then
c cubic interpolation, but don't over or under shoot the data points
         mode=4
         monoton=.true.
         sort=.false.
         call smoothb(zc,cld,ncld,NN,sort,zstd,intcld,nstd,N,
     &ierr,mode,monoton)
c set cloud amount above zstd(nstd) equal to zero
         tauc(nstd)=0.0d0
         do 45 k=nstd-1,1,-1
            cavg=(0.5d0*(intcld(k+1)+intcld(k)))
            tauc(k)=tauc(k+1)+cavg*dzstd(k)
C            write(6,*)'z tauc ',zstd(k),intcld(k)
45       continue
         if(ttc.ne.tauc(1)) then
           write(6,'(a36,1PE11.4,a4,1PE11.4)')
     &'divide. must normalize tauc from   ',tauc(1),' to',ttc
c           write(6,*)'factor =',ttc/tauc(1)
c           call normalize(ttc,intcld,nstd,tauc(1))
           call normalize(ttc,tauc,nstd,tauc(1))
         endif
         if(prtf)then
           open(92,file='tauc.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),tauc(i)
           enddo
           close(92)
         endif
       endif

       if(ozone)then
         call integoz(zoz,oz,temp,noz,o3coe,tto3,sumo3,Ho3)
         write(6,'(a28,1pE13.4,a6)')'divide. ozone scale height=',
     &Ho3,' (km).'
         write(6,'(a52,1pe13.4,a7)')
     &'divide. the total o3 for the input distribution is ',sumo3,
     &' atm-cm'
         write(6,'(a50,a15,1pe13.4,a7)')
     &'divide. the total o3 optical depth for the input ',
     &'distribution is ',tto3
         if(tto3.eq.0.0d0)then
           write(6,*)'hey the ozone od is 0'
c           totoz=0.0d0
           i=maxnz-1
           call setzeros(to3,i)
           call setzeros(tauo3,N)
c           ozone=.FALSE.
           sumo3=0.0d0
           goto 60
         endif
c adjust the concentration & total od to match the input total amount
         tto3=tto3*totoz/sumo3
         do i=1,noz
            oz(i)=totoz/sumo3*oz(i)
         enddo
c now for the interpolated profile
         mode=0
         sort=.FALSE.
         monoton=.TRUE.
         call spline(zoz,oz,noz,yp1,ypn,y2,mode)
         do i=1,nstd
              call smoothb(zoz,oz,noz,noz,sort,zstd(i),intoz(i),
     &one,one,ierr,mode,monoton)
c            call splint(zoz,oz,y2,noz,zstd(i),intoz(i),mode)
         enddo
         if(iatmtype.eq.3)then
c subtract gaussian bump from the input distribution
c total amount to subtract is in terms of atm-cm, but the 
c distribution is in terms of #/cm^3.  So, need conversion factor of
           rjunk1=deltaoz*Nl*1.0d-05/sqrt(2.0d0*pi*aersig*aersig)
           rjunk2=0.5d0/(aersig*aersig)
           do i=1,nstd
              intoz(i)=intoz(i)-rjunk1*exp(-rjunk2*(zstd(i)-aerz0)**2)
           enddo
         endif
         if(prtf)then
           open(92,file='intoz.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),intoz(i)
           enddo
           close(92)
         endif
         call spline(ztemp,temp,ntemp,yp1,ypn,y2,mode)
         do i=1,nstd
            call splint(ztemp,temp,y2,ntemp,zstd(i),inttmp(i),mode)
         enddo
         if(intoz(nstd).ne.intoz(nstd-1))then
c use scale height for ozone loading above level zstd(nstd)
           Ho3=(dzstd(nstd-1))/log(intoz(nstd-1)/intoz(nstd))
           if(Ho3.lt.0.0)Ho3=0.0
         else
           Ho3=0.0d0
         endif
         tempavg=inttmp(nstd)-273.0d0
         coeff=(o3coe(1)+o3coe(2)*tempavg+o3coe(3)*tempavg*tempavg)
         tauo3(nstd)=coeff*Ho3*intoz(nstd)/Nl*1.0d05
         sumo3=Ho3*intoz(nstd)/Nl*1.0d05
         do 50 k=nstd-1,1,-1
            ozavg=0.5d0*(intoz(k+1)+intoz(k))/Nl
            doz=ozavg*dzstd(k)*1.0d05
            sumo3=sumo3+doz
            tempavg=0.5d0*(inttmp(k+1)+inttmp(k))-273.0d0
            coeff=(o3coe(1)+o3coe(2)*tempavg+o3coe(3)*tempavg*tempavg)
            tauo3(k)=tauo3(k+1)+doz*coeff
50       continue
         tto3=tauo3(1)
         write(6,'(a30,a30,f6.3,a7)')'divide. the total o3 for the ',
     &'interpolated distribution is ',sumo3,' atm-cm'
c adjust the concentration & total od to match the input total amount
         if(iatmtype.eq.3)then
           tto3=tto3*(totoz-deltaoz)/sumo3
           rjunk1=totoz-deltaoz
         else
           tto3=tto3*totoz/sumo3
           rjunk1=totoz
         endif
c normalize the standard profile
         call normalize(rjunk1,intoz,nstd,sumo3)
c
         if(tto3.ne.tauo3(1)) then
           write(6,'(a36,1PE11.4,a4,1PE11.4)')
     &'divide. must normalize tauo3 from  ',tauo3(1),' to',tto3
           call normalize(tto3,tauo3,nstd,tauo3(1))
         endif
         if(prtf)then
           open(92,file='tauo3.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),tauo3(i)
           enddo
           close(92)
         endif
         do i=1,nstd
            ttotal(i)=tauo3(i)+ttotal(i)
         end do
c adjust tto3 for elevated surface
         mode=0
         call spline(zstd,tauo3,nstd,yp1,ypn,y2b,mode)
         call splint(zstd,tauo3,y2b,nstd,zsfc,tto3,mode)
         write(6,*)'divide. tto3 = ',tto3
c what is total o3 at elevated surface
c         if(zsfc.gt.zstd(nstd))then
       end if
60     continue
       if(water)then
c must integrate over distribution to get water vapor optical depth.
c h2o is input in terms of number density [#/cubic cm]
c absorption coef. is in terms of [1/atm-cm]
c optical depth = h2o [#/cc] * dz [km] * 1.0e5 [cm/km] * 
c                 coeff [1/atm-cm] / Nl [1/(#/atm-cm)]
c Nl=Locshmidt's number = 2.6867e16 #/cm^2
         tth2o=0.0d0
         sumh2o=0.0d0
         coeff=h2ocoe
         do j=2,nh2o
            delz=(zwat(j)-zwat(j-1))*1.0d05
            watavg=(0.5d0*(wat(j)+wat(j-1)))/Nl
            dwat=watavg*delz
            sumh2o=sumh2o+dwat
            tth2o=tth2o+dwat*coeff
         enddo
c scale height for alt. above zwat(nh2o)
         if(wat(nh2o).ne.wat(nh2o-1))then
           Hh2o=delz/log(wat(nh2o-1)/wat(nh2o))
           if(Hh2o.lt.0.0)Hh2o=0.0
         else
           Hh2o=0.0d0
         endif
         write(6,'(a30,1pE13.4,a6)')'divide. H20 scale height=',
     &Hh2o,' (km).'
         sumh2o=sumh2o+Hh2o*wat(nh2o)/Nl*1.0d05
         tth2o=tth2o+coeff*Hh2o*wat(nh2o)/Nl*1.0d05
        write(6,'(a52,1PE9.3,a7)')
     &'divide. the total h2o for the input distribution is ',sumh2o,
     &' atm-cm'
         if(tth2o.eq.0.0d0)then
           write(6,*)'hey the water vapor od is 0'
           totwat=0.0d0
           i=maxnz-1
           call setzeros(th2o,i)
           call setzeros(tauh2o,N)
           water=.FALSE.
           sumh2o=0.0d0
           goto 70
         endif
c adjust the concentration and total od to match the input total amount
         do i=1,nh2o
            wat(i)=wat(i)*totwat/sumh2o
         enddo
         tth2o=tth2o*totwat/sumh2o
c now for the interpolated profile
C         mode=0
C---------------MODE CHANGED TO 4 (LINEAR INTERPOLATION) BY A. VASILKOV
      MODE=4
         call spline(zwat,wat,nh2o,yp1,ypn,y2,mode)
         do i=1,nstd
            call splint(zwat,wat,y2,nh2o,zstd(i),intwat(i),mode)
         enddo

c use scale height for ozone loading above level zstd(nstd)
         if(intwat(nstd).ne.intwat(nstd-1))then
           Hh2o=(dzstd(nstd-1))/log(intwat(nstd-1)/intwat(nstd))
           if(Hh2o.lt.0.0)Hh2o=0.0
         else
           Hh2o=0.0d0
         endif
         coeff=h2ocoe
         sumh2o=Hh2o*intwat(nstd)/Nl*1.0d05
         tauh2o(nstd)=coeff*Hh2o*intwat(nstd)/Nl*1.0d05
         do k=nstd-1,1,-1
            watavg=(0.5*(intwat(k+1)+intwat(k)))/Nl
            dwat=watavg*dzstd(k)*1.0d05
            sumh2o=sumh2o+dwat
            tauh2o(k)=tauh2o(k+1)+dwat*coeff
         enddo
         write(6,'(a29,a30,f6.3,a7)')'divide. the total h2o for the ',
     &'interpolated distribution is ',sumh2o,' atm-cm'
         if(tth2o.ne.tauh2o(1)) then
           write(6,'(a35,1PE11.4,a4,1PE11.4)')
     &'divide. must normalize tauh2o from ',tauh2o(1),' to',tth2o
           call normalize(tth2o,tauh2o,nstd,tauh2o(1))
         endif
         do i=1,nstd
            ttotal(i)=tauh2o(i)+ttotal(i)
         end do
c adjust tth2o for elevated surface
         mode=0
         call spline(zstd,tauh2o,nstd,yp1,ypn,y2b,mode)
         call splint(zstd,tauh2o,y2b,nstd,zsfc,tth2o,mode)
       end if
70     continue
       if(prtf)then
         open(92,file='ttotal.dat',status='unknown')
         do i=1,nstd
            write(92,*)zstd(i),ttotal(i)
         enddo
         close(92)
       endif
       tautot=ttr+ttf+tto3+tth2o+ttc
       tautots=ttr+omega0*ttf+ttc*omega0c
       write(6,'(a16,E11.4,a10,E11.4)')' divide. tautot=',tautot,
     &' ;tautots=',tautots
       
c now interpolate to find zmod

       k=1
c
c check to see if top is less than or equal to zstd(1)
c
       if(top.gt.zstd(nstd))then
         write(6,1010)zstd(nstd)
1010   format(" divide. The bottom of the Chapman layer is above ",
     &"the heighest",/,"         standard alt.  So, will set to ",
     &"the value of ",f8.3,".")
         top=zstd(nstd)
       endif
       zmod(1)=top
       mode=0
       call spline(zstd,ttotal,nstd,yp1,ypn,y2b,mode)
       call splint(zstd,ttotal,y2b,nstd,zmod(1),tau,mode)
c calculate the number of layers needed
       if(cloud.and.lthkcld)then
c thick cloud in atmosphere
         call splint(zstd,ttotal,y2b,nstd,zcldtop,ttcldtop,mode)
         call splint(zstd,ttotal,y2b,nstd,zcldbot,ttcldbot,mode)
c estimate the number of layers needed
         if(daveps)then
           nlay1=(maxdavepslay+ndl+(7.456454-
     &sqrt(55.5987-4358.08*log(pcldtop/oneatm))))/ndl
           nlay2=(maxdavepslay+ndl)/ndl-
     &(maxdavepslay+ndl+(7.456454-
     &sqrt(55.5987-4358.08*log(pcldbot/oneatm))))/ndl
         else
           nlay1=nint((ttcldtop-tau)/dtaumax+0.49)
           if(nlay1.lt.1)nlay1=1
           nlay2=nint((tautot-ttc-ttcldbot)/dtaumax+0.49)
           if(nlay2.lt.1)nlay2=1
         endif
         if(daveps)then
           call czdave(intpres,zmod(k),zcldtop,zmod,nz,ndl,k,ierr)
           nlay1=k
           write(6,*)'divide. using daveps nlay1=',nlay1
         else
c do the atmo above the cloud in equal dtaumax values
           mode=0
           monoton=.true.
           sort=.false.
           dtau=(ttcldtop-tau)/dble(nlay1)
           do k=2,nlay1
              tau=tau+dtau
              call smoothb(ttotal,zstd,nstd,nstd,sort,tau,root,
     &one,one,ierr,mode,monoton)
              write(6,*)'divide. from spline tau ',tau,' root',root
              zmod(k)=root
           enddo
         endif
         k=nlay1+1
         zmod(k)=zcldtop
c do the cloud using isotropic drop density in the cloud
         call cncldlay(ttc,dtaumax,zcldtop,zcldbot,zmod,maxnz,k,nlay3)
         nlayers=1+nlay1+nlay2+nlay3
         nz=nlayers
c check nz to see if larger than maxnz
         if(nz.eq.maxnz)then
           write(6,*)'divide. nz is greater that maxnz'
           write(6,*)'divide. you must recompile with maxnz ',
     &'greater than or equal to',nz+1,'.'
           write(6,*)'divide. Look in the file param.inc'
           ierr=1
	   goto 9999
         end if
         k=1+nlay1+nlay3
         zmod(k)=zcldbot
         if(daveps)then
c do the atmo below the cloud using the daveps function
           if(zsfc.lt.zmod(k))then
             call czdave(intpres,zmod(k),zsfc,zmod,nz,ndl,k,ierr)
             if(ierr.ne.0)goto 9999
             nlayers=k+1
           else
             nlayers=k
           endif
         else
c do the atmo below the cloud in equal dtaumax values
           mode=0
           monoton=.true.
           sort=.false.
           tau=ttcldbot
           dtau=(tautot-tau-ttc)/dble(nlay2)
           do k=nlay1+nlay3+2,nlayers-1
              tau=tau+dtau
              call smoothb(ttotal,zstd,nstd,nstd,sort,tau,root,
     &one,one,ierr,mode,monoton)
c              write(6,*)'divide. from spline tau ',tau,' root',root
              zmod(k)=root
           enddo
         endif
c         k=nlayers-1
c         k=k+1
         k=nlayers
         nz=k
       else if(lusezstd)then
         nz=nstd
         do k=1,nstd
            zmod(k)=zstd(nstd-k+1)
         enddo
       else if(daveps)then
         ndavelay=maxdavepslay/ndl
         nlayers=ndavelay
         write(6,*)'divide. Using dave pressure scheme nlayers = ',
     &nlayers,' plus 2 layers above.'
c check nz to see if larger than maxnz
         if(nlayers+3.gt.maxnz)then
           write(6,*)'divide. nz is greater that maxnz'
           write(6,*)'divide. you must recompile with ',
     &'maxnz greater than or equal to',nlayers+2+1,'.'
           write(6,*)'divide. will stop now'
           ierr=4
           goto 9999
         end if
         call czdave(intpres,zmod(1),zsfc,zmod,nz,ndl,k,ierr)
         if(ierr.ne.0)goto 9999
         k=k+1
         nz=k
       else
c
c will use equal delta_tau through-out atmosphere
c
         nlayers=nint((tautot-tau)/dtaumax)
c revise dtaumax to match nz
         dtau=(tautot-tau)/dble(nlayers)
c add 1 to the number of even delta tau layers to account for top layer
         nlayers=nlayers+1
         nz=nlayers
c check nz to see if larger than maxnz
         if(nz.eq.maxnz)then
           write(6,*)'divide. nz is greater that maxnz'
           write(6,*)'divide. you must recompile with maxnz ',
     &'greater than or equal to',nz+1,'.'
           write(6,*)'divide. Look in the file param.inc'
           ierr=4
           goto 9999
         endif
         if(cloud)then
c add the cloud od to ttotal
           do i=1,nstd
               ttotal(i)=tauc(i)+ttotal(i)
           enddo
         endif
c cubic interpolation, but don't over or under shoot the data points
         monoton=.true.
         sort=.false.
         k=1
         tau=tau+dtau
         do while((k.lt.nz-1).and.
     &((tau.lt.tautot).or.(zmod(k)-zsfc.gt.dzlay)))
            if(cloud)then
              mode=4
            else
              mode=1
            endif
            call smoothb(ttotal,zstd,nstd,nstd,sort,tau,root,
     &one,one,ierr,mode,monoton)
            if(lchkdz.and.((zmod(k)-root).gt.dzlay))then
              root=zmod(k)-dzlay
              if(cloud)then
                mode=4
              else
                mode=2
              endif
              call smoothb(zstd,ttotal,nstd,nstd,sort,root,tau,
     &one,one,ierr,mode,monoton)
c revise nz
              nlayers=nint((tautot-tau)/dtaumax+0.49)
              dtau=(tautot-tau)/dble(nlayers)
              nz=k+1+nlayers
            endif
            k=k+1
c              write(6,*)'divide.',k,'tau ',tau,' root',root
            if(k.ge.maxnz-1)then
              write(6,*)'divide. nz is greater that maxnz'
              write(6,*)'divide. you must recompile with maxnz ',
     &'greater than ',k+1,'.'
              write(6,*)'divide. Look in the file param.inc'
              ierr=4
              goto 9999
            endif
            if(root.gt.zmod(k-1))write(6,*)'neg. dz',zmod(k-1),root
            zmod(k)=root
            tau=tau+dtau
         enddo
         do while(lchkdz.and.(zmod(k)-zsfc.gt.dzlay))
            nz=nz+1
            k=k+1
            zmod(k)=zmod(k-1)-dzlay
         enddo
         k=k+1
         nz=k
       endif
       zmod(nz)=zsfc
c
c calculate tau values
c
c add more layers to the top of the atmosphere
       if(lmorelays)then
         z1=dnint(zmod(2))
         iextra=int((zmod(1)-z1)/dzlay+0.5)
         write(6,*)'divide. ',zmod(1),zmod(2),z1,iextra
         write(6,*)'divide. will add ',iextra,' levels'
c shift the old levels to make room for the new levels
         do i=nz,2,-1
            zmod(i+iextra)=zmod(i)
         enddo
c now add the new levels
         do i=2,iextra+1,1
            zmod(i)=zmod(1)-dble(i-1)*dzlay
         enddo
         nz=nz+iextra
       endif
       lastdtau=0.0d0
       if(rayleigh) then
         mode=2
         yp1=1.0d30
         ypn=1.0d30
         call intdtau(zstd,tauray,nstd,yp1,ypn,y2b,mode,nz,
     &ttr,zmod,Hp,tray,sumr,lastdtau)
       else
         sumr=0.0d0
       endif
       if(aerosol)then
         if(luseintdtau)then
           mode=0
           yp1=1.0d30
           ypn=1.0d30
           call intdtau(zstd,tauf,nstd,yp1,ypn,y2b,mode,nz,
     &ttf,zmod,Hf,tf,sumf,lastdtau)
         else
c cubic interpolation, but don't over or under shoot the data points
           mode=0
           monoton=.true.
           sort=.false.
           k=1
           call smoothb(zstd,tauf,nstd,nstd,sort,zmod(k),tfbot,
     &one,one,ierr,mode,monoton)
           tf(k)=tfbot
c get the scale height for the amount above zmod(1)
           root=zmod(k)-1.0d0
           call smoothb(zstd,tauf,nstd,nstd,sort,root,tftop,
     &one,one,ierr,mode,monoton)
           if(tftop.gt.tfbot)then
             Hf=1.0d0/log(tftop/tfbot)
           else
             Hf=0.0d0
           endif
           tftop=tfbot
           sumf=tf(k)
           do k=2,nz-1
              call smoothb(zstd,tauf,nstd,nstd,sort,zmod(k),tfbot,
     &one,one,ierr,mode,monoton)
              tf(k)=tfbot-tftop
              tftop=tfbot
              sumf=sumf+tf(k)
           enddo
           tf(k)=ttf-tftop
           lastdtau=lastdtau+tf(k)
           sumf=sumf+tf(k)
         endif
       else
         sumf=0.0d0
       end if
       if(cloud)then
         if(lthkcld)then
c linear interpolation, but don't over or under shoot the data points
           sumc=0.0d0
           dttcdz=ttc/(zcldtop-zcldbot)
           tc(1)=0.0d0
           do k=2,nlay1+1
              tc(k)=0.0d0
           enddo
           do k=1+nlay1+1,1+nlay1+nlay3
              tc(k)=dttcdz*(zmod(k-1)-zmod(k))
              sumc=sumc+tc(k)
           enddo
           do k=1+nlay1+nlay3+1,1+nlay1+nlay3+nlay2
              tc(k)=0.0d0
           enddo
         else
c cubic interpolation, but don't over or under shoot the data points
           mode=0
           monoton=.true.
           sort=.false.
           k=1
           call smoothb(zstd,tauc,nstd,nstd,sort,zmod(k),tcbot,
     &one,one,ierr,mode,monoton)
           tc(k)=tcbot
           tctop=tcbot
           sumc=tf(k)
           do k=2,nz-1
              call smoothb(zstd,tauc,nstd,nstd,sort,zmod(k),tcbot,
     &one,one,ierr,mode,monoton)
              tc(k)=tcbot-tctop
              tctop=tcbot
              sumc=sumc+tc(k)
           enddo
           k=nz
           tc(k)=ttc-tctop
           lastdtau=lastdtau+tc(k)
           sumc=sumc+tc(k)
         endif
       else
         sumc=0.0d0
       endif
       if(tto3.gt.0.0d0)then
c       if(ozone)then
         mode=0
         yp1=1.0d30
         ypn=1.0d30
         call intdtau(zstd,tauo3,nstd,yp1,ypn,y2b,mode,nz,
     &tto3,zmod,Ho3,to3,sumo3,lastdtau)
       else
         sumo3=0.0d0
       end if
       if(water)then
c         mode=0
C-----------MODE CHANGED TO LINEAR INTERPOLATION BY A. VASILKOV
      MODE=4
         yp1=1.0d30
         ypn=1.0d30
         call intdtau(zstd,tauh2o,nstd,yp1,ypn,y2b,mode,nz,
     &tth2o,zmod,Hh2o,th2o,sumh2o,lastdtau)
       else
         sumh2o=0.0d0
       end if
       
c check to see if total delta tau of last layer is less than 1% of
c dtaumax.  If so, then add this layer to the layer above.  Thus nz=nz-1.

       if(lastdtau.le.dtaumax*0.01.and.(daveps.eqv..FALSE.))then
         write(*,*)'divide. last layer is too thin!',lastdtau
         write(*,*)'divide. Will add the last two layers together.'
         nz=nz-1
         k=nz
         if(rayleigh) tray(k)=tray(k)+tray(k+1)
         if(aerosol) tf(k)=tf(k)+tf(k+1)
         if(cloud) tc(k)=tc(k)+tc(k+1)
         if(tto3.gt.0.0d0) to3(k)=to3(k)+to3(k+1)
c         if(ozone) to3(k)=to3(k)+to3(k+1)
         if(water) th2o(k)=th2o(k)+th2o(k+1)
	 zmod(nz)=zsfc
       end if

c normalize once more to insure that sum of optical depths are equal to 
c input (desired) values.
130    if(rayleigh) then
         if(ttr.ne.sumr)then
           write(6,'(a36,1PE11.4,a4,1PE11.4)')
     &'divide. must renormalize tray from ',sumr,' to ',ttr
           call normalize(ttr,tray,nz,sumr)
         endif
       endif
       if(aerosol) then
         if(ttf.ne.sumf)then
           write(6,'(a34,1PE11.4,a4,1PE11.4)')
     &'divide. must renormalize tf from ',sumf,' to ',ttf
           call normalize(ttf,tf,nz,sumf)
         end if
         if(prtf)then
           open(92,file='tf.dat',status='unknown')
           do i=1,nz
              write(92,*)zmod(i),tf(i)
           enddo
           close(92)
         end if
       end if
       if(cloud) then
         if(ttc.ne.sumc)then
           write(6,'(a34,1PE11.4,a4,1PE11.4)')
     &'divide. must renormalize tc from ',sumc,' to ',ttc
           call normalize(ttc,tc,nz,sumc)
         end if
         if(prtf)then
           open(92,file='tc.dat',status='unknown')
           write(92,*)zmod(1),tc(1)
           do i=2,nz
              write(92,*)zmod(i),tc(i),tc(i)/(zmod(i-1)-zmod(i))
           enddo
           close(92)
           open(92,file='intcld.dat',status='unknown')
           do i=1,nstd
              write(92,*)zstd(i),intcld(i),dttcdz
           enddo
           close(92)
         end if
       endif
       if(ozone) then
         if(tto3.ne.sumo3)then
           write(6,'(a37,1PE11.4,a4,1PE11.4)')
     &'divide. must renormalize tauo3 from ',sumo3,' to ',tto3
           call normalize(tto3,to3,nz,sumo3)
         end if
         if(prtf)then
           open(92,file='to3.dat',status='unknown')
           do i=1,nz
              write(92,*)zmod(i),to3(i)
           enddo
           close(92)
         end if
       endif
       if(water) then
         if(tth2o.ne.sumh2o)then
           write(6,'(a38,1PE11.4,a4,1PE11.4)')
     &'divide. must renormalize tauh2o from ',sumh2o,' to ',tth2o
           call normalize(tth2o,th2o,nz,sumh2o)
         endif
       endif
       mode=2
       call spline(zpres,pres,npr,yp1,ypn,y2,mode)
       do i=1,nz
          call splint(zpres,pres,y2,npr,zmod(i),presmod(i),mode)
       enddo
c
200    continue
       write(31,1001)(i,zstd(i),tauf(i),tauray(i),tauo3(i),tauh2o(i),
     &          tauc(i),ttotal(i),i=1,nstd)
       write(31,'(1x,//)')
       write(31,1000)(i,zmod(i),tf(i),tray(i),to3(i),th2o(i),
     &         tc(i),i=1,nz)

c check to see that NO negative values were generated due to line fits
       do k=1,nz
          if(tray(k).lt.0.0d0)then
            write(6,*)'divide. negative Rayleigh optical depth in ',
     &'layer',k
            stop
          else if(tf(k).lt.0.0d0)then
            write(6,*)'divide. negative aerosol optical depth in ',
     &'layer',k
            stop
          else if(tc(k).lt.0.0d0)then
            write(6,*)'divide. negative cloud optical depth in layer',k
            stop
          else if(to3(k).lt.0.0d0)then
            write(6,*)'divide. negative ozone optical depth in layer',k
            stop
          else if(th2o(k).lt.0.0d0)then
C------------------MODIFIED BY A. VASILKOV
	    TH2O(K)=0.0D0
            write(6,*)'divide. negative h2o optical depth in layer',k
C            stop
          endif
       enddo
       write(31,*)
c the rest of the programs assume that zmod(1) is the top of the atmo
c and that nz is one more than the number of layers
c So, will put the top of the atmosphere at ztop.
       do i=nz,1,-1
          zmod(i+1)=zmod(i)
       enddo
       if(ztop.gt.zmod(1))then
         zmod(1)=ztop
       else
         zmod(1)=zmod(1)+1.0d0
       endif
       nz=nz+1
c
       write(31,1500)nz
       write(6,1500)nz
       write(31,*)
       sum=0.0d0
       do i=1,nz-1
          dts=tray(i)+omega0*tf(i)+omega0c*tc(i)
          dta=(1.0d0-omega0)*tf(i)+th2o(i)+to3(i)+(1.0-omega0c)*tc(i)
          dtt=dts+dta
          sum=sum+dtt
          delz=zmod(i)-zmod(i+1)
          dtdelz=dtt/delz
          write(31,1600) i,zmod(i+1),dtt,dts,dta,dtdelz
       enddo

       write(31,1700)sum
       write(31,1800)wavel,sumf,sumr,sumc,sumo3,sumh2o
       sumt=sumf+sumr+sumo3+sumh2o+sumc
       write(31,1900)sumt

       return
c
1000   format(1x,'LAYER',1x,'BOT (KM)',4x,'TMIE',6x,'TRAY',7x,
     &        'TO3',6x,'THO2',7x,'TC',/,100(i4,5x,0PF6.2,1x,
     &        1P5E10.3,/))
1001   format(1x,'LEVEL',1x,'  Z (KM)',4x,'TauMIE',4x,'TauRAY',4x,
     &        'TauO3',5x,'TauHO2',4x,'TauCld',2x,'TOTAL TAU',/,
     &        100(i4,5x,0PF6.2,1x,1P6E10.3,/))
1500   format(1x,'divide. # OF LEVELS NEEDED IN OTHER PROGRAMS ',
     &'(NZ) IS ',I3)
1600   format(1x,'I=',i4,1x,'ZMOD(I)=',f6.2,2x,'DTT=',f6.4,2x,
     &                'DTS=',f6.4,2x,'DTA=',f6.4,2x,'DT/DZ=',1Pe9.3)
1700   format(' SUM OF ALL DTAU IS ',f8.5)

1800   format(' WAVELENGTH= ',f6.4,' (um)',/,' TAU MIE= ',1PE10.3,
     &'  TAU RAYLEIGH= ',1PE10.3,' TAU CLOUD= ',1PE10.3,/,
     &' TAU O3= ',1PE10.3,' TAU H2O= ',1PE10.3)
1900   format(' TOTAL TAU OF THE ATMOSPHERE IS ',1PE10.3)

2000   format(' divide. The pres. at the top of the input profile is',
     &1PE12.4,' (mb)')
2010   format(' divide. The pres. at the bottom of the input profile ',
     &F5.2,' (km) is ',f7.2,' (mb)')
2020   format(' divide. The pres. at the reflecting surface ',
     &F6.2,' (km) is ',f7.2,' (mb)')
3000   write(6,*)'divide. ERROR...ERROR'
       write(6,*)'divide. ERROR READING FILES'
9999   continue
       return
       end

c********************************************************************
       subroutine intdtau(z,tau,nstd,yp1,ypn,y2b,mode,nz,tt,zmod,
     &H,t,sum,lastdtau)
c
c interpolate the total tau curve for this constiuent and compute the
c delta tau.
c
	implicit none
        include 'consts.cmn'
        integer nstd,mode,nz
        real*8 z(nstd),tau(nstd),yp1,ypn,y2b(nstd)
        real*8 zmod(nz),tt,t(nz),sum,lastdtau
c
	integer k,mode_linear,i
        real*8 root,ttbot,tttop,H,taumax
c
c determine the maximum tau
c
        if(z(1).lt.z(nstd))then
          taumax=tau(1)
        else
          taumax=tau(nstd)
        endif
c
        call spline(z,tau,nstd,yp1,ypn,y2b,mode)
        k=1
        call splint(z,tau,y2b,nstd,zmod(k),ttbot,mode)
        t(k)=ttbot
c get the scale height for the amount above zmod(1)
        root=zmod(k)-1.0d0
        call splint(z,tau,y2b,nstd,root,tttop,mode)
        if(tttop.gt.ttbot)then
          H=1.0d0/log(tttop/ttbot)
        else
          H=0.0d0
        endif
        tttop=ttbot
        sum=t(k)
        do k=2,nz-1
           call splint(z,tau,y2b,nstd,zmod(k),ttbot,mode)
           t(k)=ttbot-tttop
c check to see if negative layer value
           if(t(k).lt.0.0d0)then
c will try lineaer mode
             mode_linear=4
             call spline(z,tau,nstd,yp1,ypn,y2b,mode_linear)
             call splint(z,tau,y2b,nstd,zmod(k),ttbot,mode_linear)
             t(k)=ttbot-tttop
             call spline(z,tau,nstd,yp1,ypn,y2b,mode)
           endif
c check to see that ttbot is .le. taumax
           if(ttbot.gt.taumax)ttbot=taumax
           tttop=ttbot
           sum=sum+t(k)
        enddo
        k=nz
        t(k)=tt-tttop
        lastdtau=lastdtau+t(k)
        sum=sum+t(k)
        return
        end
c**********************************************************************
