c $Header: /usr/people/flittner/radtran/flatpsv/src/extrass.f,v 1.14 1999/08/16 18:59:07 flittner Exp flittner $
c ---------------------------------------------------------------------
        subroutine extrass(np,nz,down,
     &cossun,dt,pa,wa,wr,wc,PaoFov,ProFov,PcoFov,vcosth,ivlev,
     &vsing0,pal,trans,rsfc0,isfc,sflux)
c ---------------------------------------------------------------------
C THIS SUBROUTINE CALCULATES SINGLE SCATTERING WITH AEROSOL, RAYLEIGH      
C AND ABSORPTION GASES (O3,H2O)
C   VSING0(I,N,J)=SINGLE SCATTERING (WITHOUT CONTRIBUTION FROM SURFACE
C   REFLECTIVITY) AT HEIGHT Z(N) FOR L.O.S AT VTHETA AND PHI
C FOR A GIVEN LINE OF SIGHT ANGLE VTHETA, AND FOR EACH LAYER DEFINED BY NLEV:
C    1) COMPUTE THE PATH LENGTH FOR THE PATH WITHIN THE LAYER USING
C       THE LAYER OPTICAL DEPTH AND A 1/MU FACTOR
C    2) CHECK FOR SIMPLIFIED CASE OF TOP OF THE ATMOSPHERE AND LOOKING 
C       UP, THEN INTENSITIES ARE ZERO
C    3) CHECK FOR SIMPLIFIED CASE OF AT THE SURFACE AND LOOKING DOWN,
C       THEN INTENSITIES ARE SIMPLY THE ISOTROPIC REFLECTION OF THE
C       DOWNWARD SOLAR FLUX.
C    4) OTHERWISE DO COMPLETE SINGLE SCATTERING CALCULATIONS BY DOING
C       THE LAYER BY LAYER SUMMATION ALONG THE PATH USING SUBROUTINE 
C	INTEGRT COMPUTE PA(NLEV) WHICH IS THE PATH LENGTH FROM THE 
C       SUN TO HEIGHT NLEV AND IS USED FOR FLUX CALCULATIONS
C
C pa(I)=TAU(I)/mu0
C pal(i,n)=dtau(n-1)/mui =optical path length in layer n-1 and view angle i
c trans(1,nlev)=exp(-dt(nlev-1)/costh(1))
c 		  =transmitance of nlev-1 layer in 1 direction
C COSSUN IS THE SOLAR ANGLE
C NLEV IS THE LEVEL AT WHICH THE LINE OF SIGHT ENDS
C PA IS THE PATH LENGTH ALONG THE SOLAR BEAM
C SFLUX IS THE SOLAR FLUX AT THE TOP OF THE ATMOSPHERE
C
C THERE ARE TWO CASES TO DIFFERENTIATE BETWEEN, THE UPWARD AND THE DOWNWARD
C CASE. THIS REFERS TO THE DIRECTION IN WHICH THE RADIATION IS TRAVELLING
C
C last modifed: May 5, 1997...def
C purpose: Delete use of refl.

      implicit none
        include 'param.inc'
c varibles that are passed as inputs:
	integer nz,np,ivlev,isfc
	real*8 vcosth, dt(maxnlayr),PaoFov(4),ProFov(4),PcoFov(4)
        real*8 wr(maxnlayr),wa(maxnlayr),wc(maxnlayr)
	real*8 cossun,pa(maxnz),rsfc0(4,4,nfi,mlos),sflux(4)
	logical down
c varibles that are passed as outputs: 
	real*8 vsing0(4,maxnz),pal(mlos,maxnz),
     &trans(mlos,maxnz)
c varibles that are local
	integer nlayr,m,nlev,n,isun,layer
	real*8  efout(4),um,pum,pl,wal,wrl,wcl
	real*8 a1,a2,a3,a4,b,factor,sum2,atten
c
	nlayr=nz-1

c
      if(down)then
c
c viewing downward radiation
c layer path lengths and transmissions for downward traveling radiation
         nlev=1
         pal(1,nlev)=0.0d0
         trans(1,nlev)=1.0d0
	 um=1.0d0/vcosth
         do nlev=2,nz
            pal(1,nlev)=dt(nlev-1)*um
            trans(1,nlev)=exp(-pal(1,nlev))
         enddo
c At the top of the atmo. and looking at downward radiation, the
c intensity is zero
	nlev=1
        do m=1,np
              vsing0(m,nlev)=0.0d0
	enddo
	if(ivlev.ne.1) then
           do nlev=2,ivlev
              layer=nlev-1
              pl=dt(layer)*um
              isun=layer
              a1=(pa(isun+1)-pa(isun)) - pl
              if(pa(isun).ge.300.0d0)then
                 factor=0.0d0
                 do m=1,np
                    efout(m)=0.0d0
                 enddo
              else if((a1.le.1.0d-10).and.
     &(a1.ge.-1.0d-10))then
                 b=pl*exp(-pa(isun)-pl)
                 factor=b
                 wal=wa(layer)
                 wrl=wr(layer)
                 wcl=wc(layer)
                 do m=1,np
                    sum2=wal*PaoFov(m)+wrl*ProFov(m)+wcl*PcoFov(m)
                    efout(m)=sum2*factor
                 enddo
              else
                 a2=pl/a1
                 a3=exp(-pa(isun)-pl)
                 a4=1.0d0-exp(-a1)
                 factor=a2*a3*a4
                 wal=wa(layer)
                 wrl=wr(layer)
                 wcl=wc(layer)
                 do m=1,np
                    sum2=wal*PaoFov(m)+wrl*ProFov(m)+wcl*PcoFov(m)
                    efout(m)=sum2*factor
                 enddo
              end if
              atten=exp(-pl)
              do m=1,np
                 vsing0(m,nlev)=efout(m)+vsing0(m,nlev-1)*atten
              enddo
           enddo
	endif
      else
c
c viewing upward radiation
c
c layer path lengths and transmissions for upward traveling radiation
c
	um=-1.0d0/vcosth
        pum=um+1.0d0/cossun
        do nlev=1,nz-1,1
           pal(1,nlev)=dt(nlev)*um
           trans(1,nlev)=exp(-pal(1,nlev))
        enddo
        pal(1,nz)=0.0d0
        trans(1,nz)=1.00d0
c At the surface and looking at upward radiation.
        if(isfc.gt.0)then
          a1=exp(-pa(nz))
          do m=1,np
             sum2=0.0d0
             do n=1,np
                sum2=rsfc0(m,n,1,1)*sflux(n)+sum2
             enddo
             vsing0(m,nz)=sum2*a1
          enddo
        else
          do m=1,np
             vsing0(m,nz)=0.0d0
          enddo
        endif
	if(ivlev.lt.nz) then
           do nlev=nz-1,1,-1
              layer=nlev
              pl=dt(layer)*um
              isun=layer
              a1=-(pa(isun+1)-pa(isun))-pl
              if(pa(isun).ge.300.0d0)then
                 atten=exp(-pl)
                 do m=1,np
                    vsing0(m,nlev)=atten*vsing0(m,nlev+1)
                 enddo
              else
                 wal=wa(layer)
                 wrl=wr(layer)
                 wcl=wc(layer)
                 a2=pl/a1
                 a3=dexp(-pa(isun))
                 a4=1.0d0-dexp(a1)
                 factor=-a2*a3*a4
                 do m=1,np
                    sum2=wal*PaoFov(m)+wrl*ProFov(m)+wcl*PcoFov(m)
                    efout(m)=sum2*factor
                 enddo
                 atten=exp(-pl)
                 do m=1,np
                    vsing0(m,nlev)=efout(m)+vsing0(m,nlev+1)*atten
                 enddo
              end if
           enddo
	endif
      endif
      return
      end
c ---------------------------------------------------------------------
