c $Header: /usr/people/flittner/radtran/flatpsv/src/fdist.f,v 1.14 1999/08/16 18:57:13 flittner Exp flittner $
********************************************************************

	subroutine fdist(f,r,wavel,n1,n,iflag,radmin,radmax,anu,sigma,
     &rmean,Nc,a,b,alpha,gamma,refmed,nmode,levendx)

c-----------------------------------------------------------------------
C This subroutine calculates the aerosol size distribution (junge or
C log-normal) so that the integral of the aerosol size distribution
C between radmin and radmax is 1. 
C Definition of variables:
C---INPUT---
C wavel=wavelength (wavel should be wavel/refmed but 
C	      refmed is nearly one) 
C iflag=1 - f(r) will be junge distribution
C iflag=2 - f(r) will be log-normal distribution.
C iflag=3 - f(r) will be modified gamma distribution.
C iflag=4 - f(r) will be bi-modal log-normal dist.
C iflag=5 - f(r) will be tri-modal log-normal dist.
C radmax= largest aerosol radius (microns)
C radmin= smallest aerosol radius (microns)
C 	      suggested values radmax=5.0 radmin=0.01
C anu= power of power law size-distribution, dN/dlogr = const*r**(-anu)
C sigma= standard deviation of the log-normal distribution.
C rmean= mean radius of log-normal distribution
C a,b,alpha,gamma = size distribution parameters for modified gamma dist.
C refmed  =  refractive index of medium (air)
C---OUTPUT---
C f(i)= numer of particles in unit volume per unit radii interval
C 	      [number/cm**3/um].
C r(i)= radii array
C n1= number of bins needed for the condition drad=0.02
C	    (n1 is less than n)
C drad= delta r interval between radii points, drad=0.02 um
C dx=2*pi*drad/(wavel/refmed)
c-----------------------------------------------------------------------
C iflag=1 compute f(r) for junge distribution assuming dn/dlog(r)=r**(-anu)
C         suggested value anu=3.0
C
C iflag=2 compute f(r) for log-normal
C         sigma= standard deviation of the distribution (not the log standard 
C                                                deviation)
C         suggested values  sigma=log(2.2) rmean=1.2 for soil derived particles
C         (for accurate integration need to use error function)
C iflag=3 compute f(r) for modified gamma distribution
C         suggested values for a water cloud are;
C         a=2.373,b=1.5,alpha=6.0,gamma=1.0
C         The modified gamma distribution is found in Deirmendjian's 
C         "ELECTROMAGNETIC SCATTERING ON SPHERICAL POLYDISPERSIONS" pages 72-78.
C iflag=4 compute f(r) for bi-modal log-normal
C iflag=5 compute f(r) for tri-modal log-normal
C
C last modified: May 23, 1997...def
C purpose: Correct error with power-law distribution.
C
C last modified: July 21, 1997...def
C purpose: Clean-up the calc. of # of radii bins for n1=0 case
C
C
C last modified: May 13, 1998...def
C purpose: Change the first dx increament to 0.01 from 0.05 and clean-up
C the second r value determination for uneven dx case.  Don't forget to 
C change the format 1040 of cfnp if dx intervals are changed again.
C
C last mod. Oct. 28, 1998...def
C purpose: minimum # of radii bins is set to n1min (200).
C
C last mod. Dec. 30, 1998...def
C purpose: for iflag=1, input the maximum radius of constant dn/dlogr
C
C
c-----------------------------------------------------------------------

        implicit none
        include 'consts.cmn'
c passed
	integer n,n1,iflag,nmode
	real*8 f(n+1,3),r(n+1),wavel,radmin(3),radmax(3)
        real*8 anu,sigma(3),rmean(3)
	real*8 a,b,alpha,gamma,refmed,Nc(3)
        logical levendx
c local
	integer i,is,imode,j
	real*8 sumf,dx,drad,a1,a2,power,reff,veff,aeff,mom4,mom2,mom3
        real*8 fmus,xmax,xmin,x,r2x,No,rrmin,rrmax,tol,rJNbrk,voleff
        logical lusedx,prtf,debug,nooverlap
        integer ndx,n1min
        parameter (ndx=4,n1min=200)
        real*8 dxi(ndx),xe(ndx),R1,R2,mom1,mom0
c
        data dxi/0.01d0,0.1d0,0.2d0,0.5d0/
        data xe/10.0d0,20.0d0,30.0d0,100.0d0/
c
        tol=1.0d-06
        lusedx=.true.
        prtf=.true.
        debug=.true.
        dx=0.10d0
c
c zero out the r and f arrays
c
        do i=1,n+1
           r(i)=0.0d0
           do j=1,3
              f(i,j)=0.0d0
           enddo
        enddo
c start and stop radii
c
c for comparision with vpd code min and max for log-normal dist.
c  rrmin= exp(log(rmean) - 4.00d+00*log(sigma))
c  rrmax= exp(log(rmean) + 4.00d+00*log(sigma))
c
        rrmin=radmin(1)
        rrmax=radmax(1)
        if(iflag.eq.4)then
          if(radmin(1).gt.radmin(2))rrmin=radmin(2)
          if(radmax(1).lt.radmax(2))rrmax=radmax(2)
          if(radmax(1).ge.radmin(2))then
            nooverlap=.FALSE.
          else
            nooverlap=.TRUE.
          endif
        endif
        if(lusedx.and.levendx.eqv..FALSE.)then
c will set dx and then calc. drad
c for comparision with dave vpd report use dx=0.20d0 and r(2)=0.5/r2x
          r2x=2.0d0*pi/(wavel/refmed)
	  xmax=r2x*rrmax
	  xmin=r2x*rrmin
	  n1=nint((xmax-xmin)/dx)
          if(n1.lt.n1min)then
            n1=n1min
            dx=(xmax-xmin)/dble(n1)
            levendx=.true.
          endif
	  if(n1.gt.n)then
            if(xmax.gt.100.0d0.and.(490+nint(xmax-100.0d0).le.n))then
              n1=490+nint(xmax-100.0d0)
              levendx=.false.
            else
              n1=n
              dx=(xmax-xmin)/dble(n1)
              levendx=.true.
            endif
          endif
C create radii array
          r(1)=rrmin
c          xmin=0.5d0
          if(levendx)then
c find second x value such that it is greater than xmin and a multiple of dx.
            x=dx
            do while(x.le.xmin)
               x=x+dx
            enddo
            xmin=x
            do i=2,n1
               x=xmin+dx*dble(i-2)
	       r(i)=x/r2x
            enddo
          else
	    x=xmin
            do i=1,ndx
               if((x-xe(i)).le.tol)then
                 dx=dxi(i)
                 goto 100
               endif
            enddo
            dx=1.0d0
100         x=dx
            do while(x.le.xmin)
               x=x+dx
            enddo
            r(2)=x/r2x
            n1=2
            x=x+dx
            do while(x.lt.xmax)
               n1=n1+1
               r(n1)=x/r2x
               do i=1,ndx
                  if((x+dx-xe(i)).le.tol)then
                    dx=dxi(i)
                    goto 200
                  endif
               enddo
               dx=1.0d0
200            x=x+dx
            enddo
          endif
          r(n1+1)=rrmax
        else
          dx=0.1d0
          drad=dx*(wavel/refmed)/(2.0d0*pi)
c for comparison with Herman & browning, 1975 use dr=0.02, ~0.34 for w=0.34 um
          drad=0.02
          n1=int((rrmax-rrmin)/drad+0.001d0)
          if(n1.lt.n1min) n1=n1min
          if(n1.gt.n) n1=n
          drad=(rrmax-rrmin)/n1
          r(1)=rrmin
          do i=2,n1
	     r(i)=rrmin+drad*dble(i-1)
          enddo
          r(n1+1)=rrmax
        endif

        write(31,*) ' input in subroutine fdist'
        write(31,'(3(a8,1PE12.4))')' rmin= ',rrmin,
     &' radmax= ',rrmax,' dx=',dx
        write(31,*) ' number of bins in f(r) is ',n1

        if(iflag.eq.1)then
          write(31,*) ' IFLAG= ',iflag
          write(31,*) ' F(R)=JUNGE DN/DLOG(R)=R**(-ANU)'
          write(31,*) ' ANU= ',anu
          nmode=1
          power=anu+1.0d0
c constant number density from radii=rmin to rmean(1) um
          i=1
          No=2.251d04
          rJNbrk=rmean(1)
          if(rJNbrk.le.0.0)rJNbrk=r(1)
          do while(r(i).lt.rJNbrk)
             f(i,1)=No
             i=i+1
             if(i.gt.n1+1)goto 50
          enddo
          is=i
          do 20 i=is,n1+1
             f(i,1)=No*(rJNbrk/r(i))**(power)
20        continue
        else if(iflag.eq.2)then
          write(31,*) ' IFLAG= ',iflag
          write(31,*) ' F(R)=LOG NORMAL'
          write(31,*) ' SIGMA= ',sigma(1),' RMEAN= ',rmean(1)
          Nc(1)=1.0d0
          nmode=1
          call mklnf(r,f(1,1),n,n1,radmin(1),radmax(1),sigma(1),
     &rmean(1),Nc(1))
        else if(iflag.eq.3)then
C CALCULATE THE SIZE DISTRIBUTION USING THE MODIFIED GAMMA DISTRIBUTION
          write(31,*) ' IFLAG= ',iflag
          write(31,*) ' F(R)=MODIFIED GAMMA'
          write(31,*) ' A= ',a,' B= ',b
          write(31,*) ' ALPHA=',alpha,' GAMMA=',gamma
          nmode=1
	  a1=a*(r(1)**alpha)*exp(-b*r(1)**gamma)
	  do i=1,n1
	     a2=a*(r(i+1)**alpha)*exp(-b*r(i+1)**gamma)
             f(i,1)=a1
	     a1=a2
          enddo
          f(n1+1,1)=a1
        else if(iflag.eq.4)then
          write(31,*) ' IFLAG= ',iflag
          write(31,*) ' F(R)=Bi-MODAL LOG NORMAL'
          nmode=2
          do imode=1,2
             write(31,*) ' SIGMA= ',sigma(imode),' RMEAN= ',
     &rmean(imode),' Nc= ',Nc(imode)
             call mklnf(r,f(1,imode),n,n1,radmin(imode),radmax(imode),
     &sigma(imode),rmean(imode),Nc(imode))
          enddo
        else if(iflag.eq.5)then
          write(31,*) ' IFLAG= ',iflag
          write(31,*) ' F(R)=Tri-MODAL LOG NORMAL'
          nmode=3
          do imode=1,3
             write(31,*) ' SIGMA= ',sigma(imode),' RMEAN= ',
     &rmean(imode),' Nc= ',Nc(imode)
             call mklnf(r,f(1,imode),n,n1,radmin(imode),radmax(imode),
     &sigma(imode),rmean(imode),Nc(imode))
          enddo
	endif

C normalization of f(r)
c integrate over entire range so can normalize to 1 particle
50      sumf=0.0d0
        do i=2,n1+1
           drad=r(i)-r(i-1)
           do imode=1,nmode
              sumf=sumf+(f(i,imode)+f(i-1,imode))*drad*0.5d0
           enddo
        enddo

        write(31,*)' NORMALIZED AEROSOL SIZE DISTRIBUTION'

        write(31,*)'radius bin  radius      number density'
        write(31,*)'   #         (um)          (1/um)' 
	fmus=1.0d0/sumf
c        open(93,file='fdist.out',status='unknown')
        do i=1,n1+1
c           write(93,*)r(i),(f(i,imode)*r(i),imode=1,nmode)
           do imode=1,nmode
              f(i,imode)=f(i,imode)*fmus
           enddo
           write(31,1100) i, r(i),(f(i,imode),imode=1,nmode)
        enddo
1100    format(i7,1P4E14.4)
        write(31,1200)sumf
1200    format("before normalization integrated size distribution",
     &" =",1Pe12.4)
c        close(93)
c calc the effective radius and variance of the distribution
        reff=0.0d0
        veff=0.0d0
        voleff=0.0d0
        aeff=0.0d0
        do i=1,n1+1
c           write(93,*)r(i),(f(i,imode)*r(i),imode=1,nmode)
           sumf=0.0d0
           do imode=1,nmode
              sumf=sumf+f(i,imode)
           enddo
           if(i.eq.1)then
            mom0=0.5*sumf*(r(i+1)-r(i))
            mom1=0.5*sumf*(r(i+1)-r(i))*r(i)
            mom2=0.5*sumf*(r(i+1)-r(i))*r(i)**2.0d0
            mom3=0.5*sumf*(r(i+1)-r(i))*r(i)**3.0d0
            mom4=0.5*sumf*(r(i+1)-r(i))*r(i)**4.0d0
           else if(i.eq.n1+1)then
            mom0=mom0+0.5*sumf*(r(i)-r(i-1))
            mom1=mom1+0.5*sumf*(r(i)-r(i-1))*r(i)
            mom2=mom2+0.5*sumf*(r(i)-r(i-1))*r(i)**2.0d0
            mom3=mom3+0.5*sumf*(r(i)-r(i-1))*r(i)**3.0d0
            mom4=mom4+0.5*sumf*(r(i)-r(i-1))*r(i)**4.0d0
           else
            mom0=mom0+0.5*sumf*(r(i+1)-r(i-1))
            mom1=mom1+0.5*sumf*(r(i+1)-r(i-1))*r(i)
            mom2=mom2+0.5*sumf*(r(i+1)-r(i-1))*r(i)**2.0d0
            mom3=mom3+0.5*sumf*(r(i+1)-r(i-1))*r(i)**3.0d0
            mom4=mom4+0.5*sumf*(r(i+1)-r(i-1))*r(i)**4.0d0
           endif
        enddo
        reff=mom3/mom2
        veff=(mom4-2.0d0*reff*mom3)/(reff*reff*mom2)+1.0d0
        aeff=mom2*pi/mom0
        voleff=mom3*pi*4./3./mom0
        write(6,*)'fdist. the mom0 = ',mom0
        write(6,*)'fdist. the mom2 = ',mom2
        write(6,*)'fdist. the mom3 = ',mom3
        write(6,*)'fdist. the mom4 = ',mom4
        write(6,*)'fdist. the avg area = ',aeff
        write(6,*)'fdist. the avg vol. = ',voleff
        write(6,*)'fdist. the effective radius = ',reff
        write(6,*)'fdist. the effective variance = ',veff
c        call POWER_LAW (reff,veff,R1,R2)
c        write(6,*)'fdist. r1 r2 ',r1,r2
        return
        end
c***********************************************************************
        subroutine mklnf(r,f,n,n1,radmin,radmax,sigma,rmean,Nc)
c
c make a log-normal size dist and add to the f array (DnDr array)
c
        implicit none
        include 'consts.cmn'
c passed in and out
        integer n,n1
        real*8 r(n+1),f(n+1),radmin,radmax,sigma,rmean,Nc
c local
	integer is,ie,i
        real*8 a1,a2,ai,sumf,bipls1,dlogr0,const,a3,drad,vol,area
        real*8 v1,v2
        logical debug
c
        debug=.FALSE.
        if(r(1).ge.radmin)then
          is=1
        else
          is=1
          do while(r(is).le.radmin)
             is=is+1
          enddo
          is=is-1
        endif
        if(r(n1+1).eq.radmax)then
          ie=n1+1
        else
          ie=is+1
          do while((r(ie).lt.radmax).and.(ie.le.n1))
             ie=ie+1
          enddo
        endif
        const=Nc/(sqrt(2.0d0*pi)*sigma)
        a3=-0.50d0/(sigma*sigma)
        dlogr0=dlog(rmean)
c
c compute the normalization constant for this mode by integrating over the 
c range of radii using trapazodal integration.
c
        if(r(is).ge.radmin)then
          ai=(dlog(r(is))-dlogr0)*(dlog(r(is))-dlogr0)
          a1=(const/r(is))*dexp(ai*a3)
          v1=r(is)*r(is)*dexp(ai*a3)
        else
          a1=0.0d0
          v1=0.0d0
        endif
        sumf=0.0d0
        vol=0.0d0
        area=0.0d0
        do i=is+1,ie
           drad=r(i)-r(i-1)
           if((r(i).ge.radmin).and.(r(i).le.radmax))then
             bipls1=(dlog(r(i))-dlogr0)*(dlog(r(i))-dlogr0)
             a2=(const/r(i))*dexp(bipls1*a3)
             v2=r(i)*r(i)*dexp(bipls1*a3)
           else
             a2=0.0d0
             v2=0.0d0
           endif
           sumf=sumf+(a1+a2)*drad*0.5d0
           vol=vol+(v1+v2)*drad*0.5d0
           area=area+(a1*r(i-1)*r(i-1)+a2*r(i)*r(i))*drad*0.5d0
           a1=a2
           v1=v2
        enddo
        vol=vol*4.0d0/3.0d0*pi*const
        area=area*4.0d0*pi*const
        const=const*Nc/sumf
        if(debug)write(6,'(a8,1P2e8.1,1P5e11.4)')'mklnf. ',r(is),r(ie),
     &sumf,area/sumf,vol/sumf,Nc,vol/sumf*Nc
c
c now add this mode to the number density
c
        if(r(is).ge.radmin)then
          a1=(const/r(is))*dexp(ai*a3)
          f(is)=f(is)+a1
        endif
        do i=is+1,ie
           if((r(i).ge.radmin).and.(r(i).le.radmax))then
             bipls1=(dlog(r(i))-dlogr0)*(dlog(r(i))-dlogr0)
             a2=(const/r(i))*dexp(bipls1*a3)
             f(i)=f(i)+a2
           endif
        enddo
        return
        end
c
c
C********************************************************************

C  COMPUTATION OF R1 AND R2 FOR A POWER LAW SIZE DISTRIBUTION WITH
C  EFFECTIVE RADIUS A AND EFFECTIVE VARIANCE B

      SUBROUTINE POWER_LAW (A,B,R1,R2)
      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL F
      COMMON AA,BB
      AA=A
      BB=B
      AX=1D-5
      BX=A-1D-5
      R1=ZEROIN (AX,BX,F,0D0)
      R2=(1D0+B)*2D0*A-R1
      RETURN
      END

C***********************************************************************

      DOUBLE PRECISION FUNCTION ZEROIN (AX,BX,F,TOL)
      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL F
      EPS=1D0
   10 EPS=0.5D0*EPS
      TOL1=1D0+EPS
      IF (TOL1.GT.1D0) GO TO 10
   15 A=AX
      B=BX
      FA=F(A)
      FB=F(B)
   20 C=A
      FC=FA
      D=B-A
      E=D
   30 IF (DABS(FC).GE.DABS(FB)) GO TO 40
   35 A=B
      B=C
      C=A
      FA=FB
      FB=FC
      FC=FA
   40 TOL1=2D0*EPS*DABS(B)+0.5D0*TOL
      XM=0.5D0*(C-B)
      IF (DABS(XM).LE.TOL1) GO TO 90
   44 IF (FB.EQ.0D0) GO TO 90
   45 IF (DABS(E).LT.TOL1) GO TO 70
   46 IF (DABS(FA).LE.DABS(FB)) GO TO 70
   47 IF (A.NE.C) GO TO 50
   48 S=FB/FA
      P=2D0*XM*S
      Q=1D0-S
      GO TO 60
   50 Q=FA/FC
      R=FB/FC
      S=FB/FA
      P=S*(2D0*XM*Q*(Q-R)-(B-A)*(R-1D0))
      Q=(Q-1D0)*(R-1D0)*(S-1D0)
   60 IF (P.GT.0D0) Q=-Q
      P=DABS(P)
      IF ((2D0*P).GE.(3D0*XM*Q-DABS(TOL1*Q))) GO TO 70
   64 IF (P.GE.DABS(0.5D0*E*Q)) GO TO 70
   65 E=D
      D=P/Q
      GO TO 80
   70 D=XM
      E=D
   80 A=B
      FA=FB
      IF (DABS(D).GT.TOL1) B=B+D
      IF (DABS(D).LE.TOL1) B=B+DSIGN(TOL1,XM)
      FB=F(B)
      IF ((FB*(FC/DABS(FC))).GT.0D0) GO TO 20
   85 GO TO 30
   90 ZEROIN=B
      RETURN
      END

C***********************************************************************

      DOUBLE PRECISION FUNCTION F(R1)
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON A,B
      R2=(1D0+B)*2D0*A-R1
      F=(R2-R1)/DLOG(R2/R1)-A
      RETURN
      END

C**********************************************************************

