c $Header: /usr/people/flittner/radtran/flatpsv/src/pavinp.f,v 1.14 1999/08/16 18:59:07 flittner Exp flittner $
******************************************************************
      subroutine pavinp(Tfine,pavg,thet0i,thetj,phik,delphi,
     &		dthjm,dthjp,forward,backward,fsbs,g,iflag)
c
c subroutine to calculate the average phase function over the
c peak, theta inc = theta out. and delta phi=0.0 or 180.0.
c Can also be used to average over any set of input and output angles.
c The averaging is done over the input angle, as opposed to being done 
c over the output angle as is done if subroutine Aerohump.  Other than 
c that, both subroutines are the same.  def  12-16-92
c
c***********************************************************************
c inputs:
c T	= combinations of Herman's T1 & T2 functions
c thet0i= input theta angle
c thetj = output theta angle
c phik	= delta phi angle (phi in - phi out)
c delphi= delta phi of averaged bin
c dthjm+dthjp = delta theta of averaged bin
c 		The phase function of an average over the theta bin 
c		ranging from thet0i-dthjm to thet0i+dthjp.
c
c outputs:
c pavg	= 4 by 4 scattering matrix averaged over solid angle specified by the 
c	  above angles.
c
C THET0I= THETA0(I) ANGLE (IN DEGREES) OF THE INCIDENT RAY.
C THETJ=  THETA(J) ANGLE (IN DEGREES) OF THE SCATTERED RAY.
C PHIK= DELTA PHI (IN DEGREES) BETWEEN THE TWO RAYS; phi in - phi out
C NOTE:
C      1. SYMMETRY OF THE RESULTS (I.E. PAVG WILL BE THE SAME)
C         FOR PHIK=150, PHIK=210
C                  =120       =240
C                  =90        =270
C                  =60        =300
C                  =30        =330
C      2. FOR THET0I,THETJ=THET0I,PHIK=0   PAVG IS THE AVERAGE PHASE 
C         FUNCTION ABOUT THE FORWARD PEAK FOR INCIDENT RAY AT ANGLE THET0I
C      3. FOR THET0I,THETJ=180-THET0I,PHIK=180  PAVG IS THE AVERAGE PHASE
C         FUNCTION ABOUT THE BACKWARD DIRECTION FOR AN INCIDENT RAY AT AN ANGLE
C         THET0I
C PAVG= AVERAGE OF PF(I) FOR THET0I,THETJ,PHIK  IN DELTA THETA=10
C         AND DELTA PHI=30. (I.E. BETWEEN THET0I+-5, AND PHIK+-15)
C         PAVG=(INTEGRAL OF PF(I)*D(SOLID ANGLE))/INTEGRAL OF D(SOLID ANGLE)
C
C DELTHE=DELTA THETA AROUND THETJ
C DELPHI= DELTA PHI AROUND PHIK
C DR= CONVERSION FROM DEGREES TO RADIANS
C CALCULATE SOLID ANGLE DELOMG FOR DELTA THETA AND DELTA PHI AROUND 
C THET0I,PHIK
c---------------------------------------------------------------------
c note that minus signs are used in cos of theta angles because
c phase matrix is computed using a theta angle of 0.0 for a 
c beam traveling straight up and L.O.S. are read-in in the 
c subroutine read (which means that the beam traveling straight
c up is discribed by the LOS angle 180.0).
C	AMU = -cos(output angle)
C	ANU = sin(output angle)
C	AMUP = -cos(incident angle)
C	ANUP =  sin(incident angle)
C	AMUSQ = cos(inc)cos(inc)
C	AMUMU = cos(out)cos(inc)
C	AMUPS = cos(inc)cos(inc)
C	COOSN = cos(phi_output phi_input)
C
c T1	= T1T1*
c T2	= T2T2*
c T3	= real(T1T2*)
c T4	= imag(T1T2*)
*
c last modified: Nov. 18, 1996
c purpose: add switch for Henyey-Greenstein phase function 
c
c last modified: July 22, 1997...def
c purpose: Change the switch for HG phase function to flag for 
c          size dist (iflag).
c
c last modified: Dec. 30, 1997...def
c purpose: Increase second dimension of Tfine from 4 to 7 so that can
c          use phase functions created by tmatrix program and liou.
c
c last modified: Apr. 16, 1998...def
c purpose: Rotate matrix from paero2 from scattering plane to vertical plane
c
c***********************************************************************
	implicit none
        include 'consts.cmn'
        include 'param.inc'
c variables passed to pavinp
        integer iflag
	real*8 thet0i,thetj,phik,delphi,
     &	pavg(4,4),Tfine(nan,7),dthjm,dthjp,g
	logical forward,backward,fsbs
c external
	real*8 phg
	external phg
c variables that are local
	integer idthet,idfi
	parameter (idthet=10,idfi=30)
	integer k,ii,jj,i,ndfi,ndthet
        integer MT,MTP1
	real*8 ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,CFISQ,SIFI,SFISQ,
     &		AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,
     &		AMU,ANU,c1,c2,dmu,s,sumphi(4,4),T1,T2,T3,T4
c
	real*8 dt,th,delomg,ph,phi,
     &		c,theta(idthet),p(4,4),
     &		cosn(idfi),sinn(idfi),dphi,
     &		thetmin,gmoled,thetar,sumphiconst
c
	ndfi=idfi
	ndthet=idthet
	c=1.0d0
	s=1.0d0
c
        dphi=delphi/dble(ndfi)
        if((iflag.eq.6).or.(iflag.eq.7))then
          do jj=1,4
             p(1,jj)=0.0d0
             p(2,jj)=0.0d0
             p(3,jj)=0.0d0
             p(4,jj)=0.0d0
          enddo
        endif
c
        dt=(dthjm+dthjp)/dble(ndthet)
	if(forward)then
          c=2.0d0
          s=-1.0d0
          ndfi=ndfi/2
	else if(backward)then
          c=2.0d0
          s=1.0d0
          ndfi=ndfi/2
	endif
        sumphiconst=c*dphi*dr

        delomg=(dcos((thet0i-dthjm)*dr)-
     &		dcos((thet0i+dthjp)*dr))*delphi*dr
c
	if(delomg.gt.0.0d0)then
          gmoled=1.0d0/delomg

C 2. CALCULATE PHI, SUCH THAT EACH DEGREE FROM -/+ DPHI/2
C FROM PHI IS ACCOUNTED FOR, THAT IS PHI=PHIK-14.5,-13.5,...,
C -0.5+PHIK,...14.5+PHIK FOR DELPHI OF 30 DEGREES

          do k=1,ndfi
	     ph=dble(k)
             phi=phik-s*((delphi+dphi)/2.0d0-ph*dphi)
             cosn(k)=cos(phi*dr)
             sinn(k)=sin(phi*dr)
          enddo

C 1. SET THETA ARRAY THETA(I)=THET0I-DTHJM...,THET0I+DTHJP  
C DELTHE/IDTHET IS THE WIDTH OF THE FINE INTERVAL (THERE ARE IDTHET INTERVALS)
C IN THE RANGE AROUND THETJ SUCH THAT THERE ARE 10 ANGLES ABOUT THETA(I) 
C DTHETA=THETA(I+1)-THETA(I)=CONST=1.
          th=0.0d0
          thetmin=thet0i-dthjm-dt/2.0d0
          do 20 i=1,ndthet
             th=th+1.0d0
             theta(i)=thetmin+dt*th
20        continue
c
c initialize pavg to zero
          do jj=1,4
             pavg(1,jj)=0.0d0
             pavg(2,jj)=0.0d0
             pavg(3,jj)=0.0d0
             pavg(4,jj)=0.0d0
          enddo
          AMU=-cos(thetj*dr)
          ANU=sin(thetj*dr)
          AMUSQ=AMU*AMU
c loop over output theta angle
          do 40 i=1,ndthet
             thetar=theta(i)*dr
             AMUP=-cos(thetar)
             ANUP=abs(sin(thetar))
             AMUPS=AMUP*AMUP
             AMUMU=AMU*AMUP
c initialize sumphi to zero
             do jj=1,4
                sumphi(1,jj)=0.0d0
                sumphi(2,jj)=0.0d0
                sumphi(3,jj)=0.0d0
                sumphi(4,jj)=0.0d0
             enddo
c integrate over azimuthal angles
             do 30 k=1,ndfi
                COOSN=COSN(k)
                SIIN=SINN(k)
                COPSI=ANU*ANUP+AMU*AMUP*COOSN
                SISQ=COPSI*COPSI
                CFISQ=COOSN*COOSN
                SIFI=COPSI*COOSN
                SFISQ=SIIN*SIIN
                cos_captheta=AMUMU+ANUP*ANU*COOSN
                if(iflag.eq.6)then
                  p(1,1)=phg(cos_captheta,g)
                else if(iflag.eq.7)then
                  call paero2(Tfine,p,captheta,cos_captheta,fsbs,
     &ANUP,AMUP,AMU,ANU,COOSN,SIIN)
                else
                  call paero(Tfine,p,ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,
     &CFISQ,SIFI,SFISQ,AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,
     &AMU,ANU,fsbs)
                endif
***********************************************************************
                do ii=1,4
                   do jj=1,4
                      sumphi(ii,jj)=sumphi(ii,jj)+
     &p(ii,jj)*sumphiconst
                   enddo
                enddo
30           continue
                           
C CALCULATE THE INTEGRAL PF(I)*SIN(THETA)DTHETA
C SUMPHI IS THE INTEGRAL OF THE PHASE FUNCTION OVER PHI 
C COS(C1)-COS(C2)=INTEGRAL OF SIN(THETA)DTHETA FOR THE PHASE FUNCTION 
C CALCULATED FOR THETA(I),THET0I       
C INTEGRATE OVER THETA

             c1=(theta(i)-dt/2.0d0)*dr
             c2=(theta(i)+dt/2.0d0)*dr
             dmu=(cos(c1)-cos(c2))
	     if(dmu.lt.0.0)then
               write(6,*)'pavinp. hey what is the deal?'
               write(6,*)'pavinp. domega (solid angle) is zero.'
               write(6,*)'pavinp. dmu=',dmu
               write(6,*)'pavinp. theta from',theta(i)-dt/2.0d0,' to',
     &			theta(i)+dt/2.0d0
               write(6,*)'pavinp. c1=',c1,'c2=',c2
               write(6,*)theta
               stop
             endif
             do ii=1,4
                do jj=1,4
                   pavg(ii,jj)=pavg(ii,jj)+
     &sumphi(ii,jj)*dmu
                enddo
             enddo
40        continue

C SUMPHS IS THE INTEGRAL OF THE PHASE FUNCTION AROUND THATA0,PHI0 WITHIN
C DELTA PHI AND DELTA THETA

          do ii=1,4
             do jj=1,4
                pavg(ii,jj)=pavg(ii,jj)*gmoled
             enddo
          enddo
        else
          write(6,*)'pavinp. hey what is the deal?'
          write(6,*)'pavinp. delomega (solid angle) is zero.'
          stop
	end if
        return
        end
***********************************************************************
