c $Header: /usr/people/flittner/radtran/flatpsv/src/upit.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
c ---------------------------------------------------------------------

      subroutine upit(itest,ndownlos,esi0,esing0,pal,dmu,wa,wr,
     &fmatrx,rmatrx,trans,dnflux,solrflux,nz,
     &nlos,np,convrg,rsfc,wc,cmatrx,isfc,itlmax)

C Performs the upward pass of the Gauss-Seidel iteration scheme
C itest - Used to check for convergence, if intensity change of more 
C         than convrg occurs between the current and last iteration
C         another iteration is performed.
C nlos2  - The index of the line of sight angle closest to but less than
C         90 degrees.
C The level is selected by the do 50 loop:
C For level=nz, this is the surface layer. The incident intensities
C          is just that due to the isotropic intensity reflected from the 
C          Lambertian surface.
C For all other layers there will be a boundary term and a term for 
C          scattering within the layer. In a flat atmosphere model
C          this boundary intensity is simply the intensity from the
C          level directly below in the same direction.
C
C Set the l.o.s such that theta=0,90(ilos=1,nlos2) for downward intensities
C This theta is the output direction for the phase function calculations 
C in the integration of the source function.
C
C jlos is used for the output phi direction
C
C Subroutine source performs the source term integration.
C
C last modified: May 5, 1997...def
C purpose: Delete use of refl and Ig
C
C last modified: Aug. 7, 1997...def
C purpose: Change order of loop over theta, work from nadir to limb.
C

      implicit none
      include 'consts.cmn'
      include 'param.inc'
c varibles that are passed
      integer itest,ndownlos,nz,nlos,np,isfc,itlmax
      real*8 esi0(4,nfi,mlos,maxnz),esing0(4,nfi,mlos,maxnz),
     &pal(mlos,maxnz), dmu(mlos), trans(mlos,maxnz),wr(maxnlayr),
     &wa(maxnlayr),wc(maxnlayr),
     &fmatrx(4,4,nfi,mlos,mlos),rmatrx(4,4,nfi,mlos,mlos),
     &cmatrx(4,4,nfi,mlos,mlos),
     &dnflux,solrflux,convrg
c for surface rsfc
      real*8 rsfc(4,4,nfi,mlos,mlos)
c variables that are local
      integer level,layer,lbelow,ilos,jlos,m,it,itm
      real*8 bndry(4),enew(4),prcntdif,delt,c1
      real*8 cf1,cf2,T,wal,wrl,wcl
	logical debug
        debug=.false.
c
c level = nz and isfc > 0
      level=nz
      if(isfc.gt.0)then
        call cIgdif(ndownlos,nlos,nz,np,esi0,esing0,dmu,rsfc)
      endif
      do 400 level=nz-1,1,-1
         layer=level
         lbelow=level+1
         wal=wa(layer)
         wrl=wr(layer)
         wcl=wc(layer)
         if(wcl.gt.0.5d0)then
           itm=itlmax
         else
           itm=1
         endif
         do it=1,itm
            do 80 ilos=nlos,ndownlos+1,-1
               delt=pal(ilos,level)
               T=trans(ilos,level)
               c1=1.0d0-T
               cf2=(1.0d0-c1/delt)
               cf1=(c1-cf2)
               do 70 jlos=1,nfi
                  call source(level,layer,lbelow,ilos,jlos,esi0,
     &dmu,wal,wrl,wcl,cf1,cf2,fmatrx,rmatrx,cmatrx,nlos,np,enew,
     &debug)
c load the boundary terms
                  do m=1,np
                     bndry(m)=esi0(m,jlos,ilos,lbelow)-
     &esing0(m,jlos,ilos,lbelow)
                  enddo
c m=1
                  enew(1)=enew(1)+bndry(1)*T+esing0(1,jlos,ilos,level)
                  if(itest.ne.100.and.enew(1).ne.0.0) then
                    prcntdif=abs(1.0d0-esi0(1,jlos,ilos,level)/
     &enew(1))*100.0d0
                    if(prcntdif.gt.convrg) then
                      itest=100
                      write(6,'(a25,1pe7.1,a32,i5)')
     &'upit. PERCENT DIFFERENCE >',convrg,
     &'% FOR UPWARD RADIATION AT LEVEL ',level
                      write(*,'(6x,a21,3I5)')'ILOS, JLOS and M ARE',
     &ilos,jlos,1
                    end if
                  end if
                  esi0(1,jlos,ilos,level)=enew(1)
c m=2
                  if(np.gt.1)then
                    m=2
                    enew(2)=enew(2)+bndry(2)*T+esing0(2,jlos,ilos,level)
                    if(itest.ne.100.and.enew(2).ne.0.0) then
                      prcntdif=abs(1.0d0-esi0(2,jlos,ilos,level)/
     &enew(2))*100.0d0
                      if(prcntdif.gt.convrg) then
                        itest=100
                        write(6,'(a26,1pe7.1,a34,i5)')
     &'upit. PERCENT DIFFERENCE >',convrg,
     &'% FOR UPWARD RADIATION AT LEVEL ',level
                        write(*,'(a21,3I5)')'ILOS, JLOS and M ARE',
     &ilos,jlos,2
                      end if
                    end if
                    esi0(2,jlos,ilos,level)=enew(2)
                    if(np.gt.2)then
                      esi0(3,jlos,ilos,level)=enew(3)+bndry(3)*T+
     &esing0(3,jlos,ilos,level)
                      if(np.gt.3)esi0(4,jlos,ilos,level)=enew(4)+
     &bndry(4)*T+esing0(4,jlos,ilos,level)
                    endif
                  endif
70             continue
80          continue
         enddo
400   continue 
      return
      end
c ---------------------------------------------------------------------
