      subroutine get_aeronet(gsfc_psd_clim,gsfc_refr_clim,ref_440,
     * aeronet_dump,
     *lam,rIOR1,iIOR1,rIOR2,iIOR2,
     &sig1,r1,n1,sig2,r2,n2)
C***********************************************************************
C	AUTHOR: Nickolay A. Krotkov  
C
C	WARNING: 
C       1) Aeronet retrieves average effective ref.index for both modes
C        Despite Arizona code allows refr indices for 2 modes to be different,
C        in this program refr. index for both modes should be equal
C
c       INPUT:
C	aeronet_dump(..) r  - aeronet retrievals (aeronet.gsfc.nasa.gov)
C	gsfc_psd_clim    l  - use or not climatological PSD for GSFC
C	gsfc_ref_clim    l  - use or not climatological refractive index for GSFC
C	     ref_440     l  - use or not constant refr index equal to refr index = ref(440nm)
C       if gsfc_ref_clim=.true. then ref_440 does not matter  !!
C
C	OUTPUT:
C	lam,rIOR1,iIOR1,rIOR2,iIOR2 - regular UV wavelength grid [um] and extrapolated refr index
C	sig1,r1,n1,sig2,r2,n2  -Number size distribution
C
C	FUNCTION:
C	
C	HISTORY: Developed 02/22/2003 to process aeronet aerosol retrievals from http://aeronet.gsfc.nasa.gov
C	Modified 03/22/2003 to allow spectral dependence of refr index (n and k)
c	lineraly extrapolate Oleg's refr index from lam1=0.44 and lam2=0.67micron
c	Modified 03/23/03 to add AERONET GSFC climatology
c	if gsfc_psd_clim=.true. then use climatological psd for GSFC
c	if gsfc_refr_clim=.true. then use climatological refr index for GSFC [Dubovik et al., JAS, 59, 590-]
C
C	Modified 03/25/03 to change wavelength refr index interpolation nodes to match aeronet channels:
C       0.30, 0.32, 0.34,0.38, 0.44
C	Modified 06/04/03 to read aeronet web format (aeronet.gsfc.nasa.gov: download tool)             
C
C***********************************************************************
      implicit none
            
      logical gsfc_refr_clim, gsfc_psd_clim,ref_440
      integer i,j      
      real tau_meas440
c     real aeronet_dump(95)
      real aeronet_dump(123)
      real r1,sig1,n1,r2,sig2,n2
      real lam(12),rIOR1(12),iIOR1(12),rIOR2(12),iIOR2(12)
      real lnsf,lnsc,lnsf2,lnsc2,rvf,rvc,vf,vc
	real lam1,lam2,coef,frac
      real dum

      
CD	write(6,*) 'get_aeronet>'

	tau_meas440=aeronet_dump(12)

c---   	get number density size distribution parameters
c        read(30,*) ityp,r1,sig1,n1,r2,sig2,n2

	if (gsfc_psd_clim) then  ! GSFC aeronet summer climatology (tau>0.4, sza>45 deg)

	vf  =     0.15*tau_meas440	! +/-0.03
	vc  =0.01+0.04*tau_meas440	! +/-0.01
	lnsf=0.38			! +/-0.01
	lnsc=0.75			! +/-0.03
	rvf =0.12+0.11*tau_meas440	! +/-0.03
	rvc =3.03+0.49*tau_meas440	! +/-0.21
  
	else			! Actual retrievals on that day

c---   Mode volume concentrations:
       	vf=aeronet_dump(83)	! Volume of fine mode (=0.018833)
       	vc=aeronet_dump(84)	! Volume of coar mode (=0.009983)
c---    read aeronet fitted log-norm. volume PSD parameters
	lnsf=aeronet_dump(92)	! ln(sigma) fine mode (=0.295772) 
	lnsc=aeronet_dump(93)   ! ln(sigma) coar mode (=0.655669)  
c---   Mode radia of the bi-modal log-normal volume PSD
       	rvf=aeronet_dump(89)	! Rvf fine mode (=0.136964)
       	rvc=aeronet_dump(90)	! Rvc coar mode (=3.192300)

	endif

C---		Convert to Number PSD
c---   ln(sigma) is the same for volume and number log-norm. PSD
	lnsf2=lnsf*lnsf
	lnsc2=lnsc*lnsc                                             
c---   Arizona code requires geometrical sigma as input parameter
        sig1=exp(lnsf)
        sig2=exp(lnsc)

c---   Arizona code requires number PSD raduis: rn= exp(alog(rv) - 3.*lns^2)
	r1  = exp(alog(rvf) - 3.*lnsf2)
	r2  = exp(alog(rvc) - 3.*lnsc2)

c---   convert to mode number concentrations: cn=cv*0.75/!pi/rn^3.0*exp(-4.5*lns^2.0)
	n1 = vf*0.238732/(r1**3)*exp(-4.5*lnsf2)
	n2 = vc*0.238732/(r2**3)*exp(-4.5*lnsc2)

c---	get lam and refractive index
      lam1=0.44
      lam2=0.67
      coef=4.3478	! coef=1/(lam2-lam1)

         do j=1,11 
c           read(30,*) lam(j),rIOR1(j),iIOR1(j),rIOR2(j),iIOR2(j)
c        lam(j)=0.270 + 0.02*(j-1)		! Old Sasha's spectral grid in aerosol.dat file
         lam(j)=0.280 + 0.02*(j-1)
	 frac=coef*(lam2-lam(j) )

	if (ref_440) then
        rIOR1(j)=aeronet_dump(40)	!  nr @ 440nm    
 	iIOR1(j) =aeronet_dump(44)	!  k  @ 440nm
        else
c---  extr from n1 @ 440nm(40) and n2 @670nm(41) and  from k1 @ 440nm (44) and k2 @670nm (45)
      rIOR1(j)=frac*aeronet_dump(40)+(1.-frac)*aeronet_dump(41) 
      iIOR1(j)=frac*aeronet_dump(44)+(1.-frac)*aeronet_dump(45)
        endif

      rIOR2(j)=rIOR1(j)         
      iIOR2(j)=iIOR1(j)                                          

         enddo

      lam(12) =0.5
      frac=coef*(lam2-lam(12) )
	if (ref_440) then
        rIOR1(12)=aeronet_dump(40)	!  nr @ 440nm    
 	iIOR1(12) =aeronet_dump(44)	!  k  @ 440nm
        else
c---  extr from n1 @ 440nm(40) and n2 @670nm(41) and  from k1 @ 440nm (44) and k2 @670nm (45)
      rIOR1(12)=frac*aeronet_dump(40)+(1.-frac)*aeronet_dump(41) 
      iIOR1(12)=frac*aeronet_dump(44)+(1.-frac)*aeronet_dump(45) 
        endif

      rIOR2(12)=rIOR1(12)         
      iIOR2(12)=iIOR1(12)                                          


C----	USE REFR INDEX AERONET CLIMATOLOGY FOR GSFC [Dubovik et al , JAS, 59,590-,2002]

	if (gsfc_refr_clim) then
         do j=1,12
      rIOR1(j)=1.41 - 0.03*tau_meas440	! n_clim(gsfc) +/-0.01
      rIOR2(j)=rIOR1(j)
      iIOR1(j)=0.003 	           	! k_clim(gsfc) +/-0.003
      iIOR2(j)=iIOR1(j)
         enddo
 	endif

	
 	write(6,*) 'get_aeronet < '

      return
      end
