      subroutine gs_fit(firstmult_k,xsec_wat,c0b,c1b,c2b,rhob,
     *max_k_iter,dd_fit,dt_fit,t_fit,d_fit,
     *fit_tolerance,log_fit,
     *mie_only,
     *totoz,totwat,surfpres,mfrsr_tr,   
     &mfrsr_dd,mfrsr_dt,mfrsr_tot,mfrsr_dif,mfrsr_aot,
     &tau440,tauc,sza,vza,phi,nvza,nphi,
     &iaermod,iaertype,lam,refrr1,refri1,refrr2,refri2,
     &sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,isurfmod,lamalb,surfalb,
     &icount,lambda,ratio,tauaer,omega,taum,om,asym,rad,
     &n_real,k_real,e_dir_ray,e_dir_ratio,e_dif_ray,e_dif_ratio,
     *calc_new,n_iter,err_par,meas_par,max_k_real,dd_k,dd_w)
C***********************************************************************
C	AUTHOR: Alexander P. Vasilkov and N. Krotkov
C
C	FUNCTION: I) Find imaginary refractive index by
C                 fitting MFRSR measurement (tot, dif, dir/diffuse, dif/total, etc. )  for fixed AOT, n and PSD
c			OR
c                 if max_k_iter =< 0 then compute zenith radiance and fluxes for initial values of k and n
c                 II) Compute the ratio R=L(aer)/L(noaer) and
C                 E_dir(aer)/E_dir(noaer) and E_dif(aer)/E_dif(noaer)
C                 using Ben Herman's  Gauss-Seidel code
c                 totwat xsec_wat variableis are used to describe NO2 absorption instead of water
c       WARNINGS: 1) make sure all RT calculations are with F0=1 (transmittances)
C
c       Modified Sep 22   2012 nk  to output asymetry parameter on last iteration: real 
c       Modified Sep 22   2012 nk  to output Jacobian on last iteration: real dd_ssa=(dd2-dd1)/(k2-k1)/dd1*w1
c       Modified Sep 21   2012 nk  Use relative fit tolerance : fit_err = abs( (calc-meas)/meas )                            
c       Modified Sep 21   2012 nk  to output Jacobian on last iteration: real dd_k=(dd2-dd1)/(k2-k1)/dd1*k1
c       Modified Jul 3    2012 nk  to constrain k_real < max_k_real                                   
c       Modified Jul 20   2009 to fix output after 0 iteration if par_calc(AERONET) = par_meas
c       Modified Jul 20   2009 to fix output after 1 iteration without k fitting
c       Modified Jul 14   2009 to use variable k multiplier for the first iteration
C	Modified Jul   15 2008 to assign fill values to k_real and om_real in case Arizona code stoppage
C	Modified Jan    8 2005 to pass xsec_wat and totwat from the main program to eliminate reading gas2.coe file
C	Modified March 11 2004 to use total transmittance fitting: t_fit=.true.                                 
C	Modified March 10 2004 to include ipseudo parameter for spherical correction of direct beam
C	Modified March 10 2004 to use different fitting parameters:   dd or dt or t or d          
C	                       with mfrsr_dd, mfrsr_dt, mfrsr_tot, mfrsr_dif                      
C	Modified March 2  2004 to use new ratio: diffuse/direct_normal (mfrsr_dd)                          
C	Modified March 2  2004 use dynamic Rayleigh OT: mfrsr_tr (scaled for pressure and l_eff  )                          
C	Modified Sept 26  2003 to supress fitting if max_k_iter <=0                                        
C	Modified July 20 ,2003 to force negative k_iter to zero                                                
C	Modified July  8 ,2003 always initialize negative Rayleigh irradiance before k iterations              
C	Modified July  8 ,2003 to use absolute DD fit tolerance                                         
C	Modified July  7 ,2003 to correct for trucated sza in arizona radiance output                   
C	Modified July  6 ,2003 to constrain number of k iterations n_iter(i)<max_k_iter                 
C	Modified July  1 ,2003 to use actual MFRSR measurements                                
c       Template: gauss_seidel_ratio_v6_440nm.f by A. Vasilkov
C***********************************************************************
      implicit none

	logical mie_only,dd_fit,dt_fit,t_fit,d_fit,log_fit ! logical flags specify which parameter to fit
            
      integer ipseudo,max_k_iter,n_iter(3)      ! no fitting if max_k_iter =< 0
      parameter(ipseudo=1)   			! ipseudo=1 - spherical correction
      real fit_tolerance,k_fit(3),err_par(3)      
      real calc_old(3),calc_new(3)     !  calculated dif/dir_normal ratio at mfrsr wavelengths (368,332,325)

      integer i,j,k,nstr,maxnum,iaertype,icount,inext,n,nang
      integer n2,iaermod,isurfmod,nn,ierr,nf,nalb,m,nvza,nphi
      parameter(nf=12,nalb=12,nang=181)
	
      real dum,dd_w,dd_k,arr_dd(2),arr_w(2),arr_k(2)
      real mfrsr_dd(3),mfrsr_dt(3)
      real meas_par,mfrsr_tot(3),mfrsr_dif(3)
      real mfrsr_aot(3)
      real iIOR1_new,iIOR2_new,firstmult_k 
      
      real n_real(1000),k_real(1000),max_k_real
      real tsza,sza,vza(nvza),phi(nphi)
      real*8 cos_sza
      real lambda(5000),lambdab,wave
      real c0(5000),c1(5000),c2(5000),beta(5000),rho(5000)
      real c0b,c1b,c2b,betab,rhob,taur,pi,tau_aer
      real mfrsr_tr
      real*8 surfpres
      real wavea,kingf,lamalb(nalb),surfalb(nalb)
      real refrr1(nf),refri1(nf),refrr2(nf),refri2(nf),lam(nf)
      real tauaer(nf),omega(nf),rIOR1,iIOR1,rIOR2,iIOR2,specalb
      real sigma1,rmean1,Nc1,sigma2,rmean2,Nc2
      real tau440,totoz,totwat,tauc,taumd,qext,qext440      
      real rad(5000,nvza,nphi),rad_aer(nvza,nphi)
      real theta,taum(1000),om(1000),asym(1000),ang(nang),phase(nang)
      real rad0,t,sbar,ratio(5000,nvza,nphi)
      real e_dir_ray(5000),e_dir_aer,e_dir_ratio(5000)
      real e_dif_ray(5000),e_dif_aer,e_dif_ratio(5000)
      real delw,w1,w2,wavep
      real lam2(5000),abs2(5000),xsec_wat,loschmidt,lambuf(1000)
      real interpol
      real*8 abso2o2
      external interpol
 
      character*40 ftop   
      character*40 fout   
      character*90 str(24),buf(12)
      character*15 fname,fdata
      character*11 faerosol	! faerosol='aerosol.phs'
				!1234567890123456789012345123456789012
      character*25 buff 	!EXTINCTION COEFFICIENT =   1.6979E-09 (cm*cm)
				!SCATTERING COEFFICIENT =   1.5847E-09 (cm*cm)
				!       buff1              f7.4   buff                   f7.4
				!123456789012345678901234561234567123456789012345678901231234567
      character*26 buff1	!SINGLE SCATTERING ALBEDO = 0.9333; ASYMMETRY PARAMETER = 0.6912
      character*23 buff2
      character*12 fphase
C---------
	logical no_mie
      parameter (nstr=24,taumd=0.0,maxnum=20)	! taumd is used for pure Rayleigh normalization run
      data loschmidt/2.6868E19/     !in [#/cm^3]
     
      write(*,*) 'Gauss-Seidel code starts'
      
c                                   ! Arizona code can not handle properly case with surfpres .ne.1013.25mb
c				    ! (trucates input aerosol and ozone tau if Psurf < 1013.25)
c     surfpres=1013.25              ! so we have to fix surfpres=1013.25 in the current version
				    ! and use pressure scaled Rayleigh optical thickness: taur=mfrsr_tr

      write(*,'(f7.2)') surfpres
      pi=4.0*atan(1.0)
      cos_sza=cos(sza*pi/180.)

c--- initialize direct Rayleigh irradiance
	do i=1,icount
	e_dir_ray(i)=-999.99
	e_dir_ratio(i)=-999.99
	e_dif_ray(i)=-999.99
	e_dif_ratio(i)=-999.99
        om(i)= -9.99
	asym(i)= -9.99
        k_real(i)=-9.99999
	enddo
            
C------- Create input file flat.in for given size distribution and 
C------- if no_mie=.false. then calculate aerosol extintion coefficient at reference wavelength: 440nm

C------- tau_440 is not used in MFRSR fitting: instead we use MFRSR measured AOT or input tau                
	no_mie=.true.

      call aerosol(no_mie,iaermod,iaertype,lam,refrr1,refri1,refrr2,
     &refri2,sigma1,rmean1,Nc1,sigma2,rmean2,Nc2,qext440)
c     write(*,*) 'Calculation of the aerosol extintion coefficient at 
c    &440 nm completed'
c     	write(*,*) 'qext440',qext440

C-------
      fname='flat.in'
      fdata='coe.dat'
      faerosol='aerosol.phs'

C----------open data file containing O3 cross-sections and Rayleigh coeff
      
c     open(9,file=fdata,status='old')
 
C----------read in actual wavelengths and x-sections for Mie/radiance calculations 

      do i=1,icount	! spectral loop: only use i=1 in current version`
      c0(i)=c0b
      c1(i)=c1b
      c2(i)=c2b
      beta(i)=mfrsr_tr
      rho(i)=rhob
      enddo
      
    
C-----------main loop in wavelength
       
      do i=1,icount	! spectral loop: only use i=1 in current version
          
      wave=lambda(i)/1.0E4	! lambda(i) is already in [nm] convert to [nm]

       
C---- get initial aerosol index of refraction (interpolated linearly in get_aeronet_mf.f 
c---- from extrapolated AERONET values, nf=12)

      rIOR1=interpol(nf,lam,refrr1,wave,ierr)	! real 1st mode
      iIOR1=interpol(nf,lam,refri1,wave,ierr)	! imag 1st mode
      rIOR2=interpol(nf,lam,refrr2,wave,ierr)	! real 2nd mode
      iIOR2=interpol(nf,lam,refri2,wave,ierr)	! imag 2nd mode
      n_real(i)=0.5*(rIOR1+rIOR2)	! not used in calculation (only output)
      k_real(i)=0.5*(iIOR1+iIOR2)	!  not used in calculation (only output)

C----------spectral surface albedo

      specalb=interpol(nalb,lamalb,surfalb,wave,ierr)

C----------open input file 'flat.in'
            
      open(10,file='flat.in_aer440',status='old') ! fname=flat.in_aer440 was modified in call aerosol(...)
				       		  ! it already has right size distribution
c---   read in the file content
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo      
      close(10)
      
c--- re-write the file fname='flat.in' with new refractive index
      open(10,file=fname,status='unknown')
      
      taur=mfrsr_tr		! use already scaled mfrsr Rayleigh optical thickness (according to pressure and lam_eff )  
      kingf=(6.0+3.0*rho(i))/(6.0-7.0*rho(i))	! Rayleigh depolarization factor
      
C-----------modify the input file for Mie calculations at lambda(i)/1.0E4
      
      write(10,'(5e11.4,f10.7)') 
     *                     mfrsr_aot(i),taur,tauc,totoz,totwat,wave ! MFRSR meas tau at wave
      write(10,'(a)') str(2)	! aero size dist. flag
      write(10,'(4e11.4)') rIOR1,iIOR1,rIOR2,iIOR2	! initial values of refr index
      write(10,'(a)') (str(k),k=4,5)		! aero min max radii, size dist parameters: not changed
      write(10,'(f7.2,1x,f7.5)') surfpres,kingf 
      write(10,'(a)') (str(k),k=7,15)
      write(10,'(3i3)') -1,4,1			! do not perform RT calculations (only Mie)
      write(10,'(i2,f7.3)') 1,specalb
      write(10,'(a)') (str(k),k=18,nstr)
      close(10)
      
      call system('rm -f aerosol.phs')
      call system('rm -f miecalc.dat')
      call system('flatpsv.x')		! -> new miecalc.dat and aerosol.phs files for wave

c     WRITE(*,*) 
c     WRITE(*,*) 'MIE CALCULATIONS COMPLETED AT WAVELENGTH  ',wave
c     WRITE(*,*) 

C----------read in the aerosol extintion coefficient,SSA and asymmetry parameter (asym)at wave from 'aerosol.phs'
     
      open(10,file=faerosol,status='old')
      if(iaertype.eq.2) nn=8
      if(iaertype.eq.4) nn=12
      read(10,'(a)') (buf(k),k=1,nn)
      read(10,'(a,e12.4)') buff,qext
      read(10,'(a)') buf(1)
      read(10,'(a,f7.4,a,f7.4)') buff1,om(i),buff2,asym(i)
      WRITE(28,*) wave,qext,om(i),asym(i)
      close(10)

c---  determine tau_aer at wave:
      taum(i)=mfrsr_aot(i)		! use MFRSR measurenet at wave


	if(mie_only)  go to 100	! bypass radiance calculations

C-----------save phase.out
      open(10,file='phasf.out',status='old')
      do k=1,nang
      read(10,*) ang(k),phase(k)
      enddo
      close(10)
      write(fphase,'(''phase_'',f6.4)') wave
      open(10,file=fphase,status='unknown')
      do k=1,nang
      write(10,*) ang(k),phase(k)
      enddo
      close(10)
	write(6,*) 'write phase.out'

C-----------compute radiances for MFRSR wavelengths  
c     WRITE(*,*) 
c   	write(*,*) 'COMPUTE RADIANCES AT WAVELENGTH ', wave
c     WRITE(*,*) 

C-----------open an input file

      open(10,file=fname,status='old')
      
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo
      close(10)
      
C-----------modify the input file
            
      open(10,file=fname,status='unknown')      
      write(10,'(5e11.4,f10.7)') taum(i),taur,tauc,totoz,totwat,wave
      write(10,'(a)') (str(k),k=2,5)
      write(10,'(f7.2,f8.5)') surfpres,kingf
      write(10,'(a)') (str(k),k=7,11)
      write(10,'(3e13.5)') c0(i),c1(i),c2(i)
      write(10,'(e13.5)') xsec_wat	! now gas2 xsec [atmcm] is passed from the main program
      write(10,'(a)') (str(k),k=14,15)
      write(10,'(3i3)') maxnum,3,ipseudo	! ipseudo=1 - pseudo-spherical attenuation of solar beam
      write(10,'(a)') (str(k),k=17,18)
      write(10,'(f7.3)') sza
      write(10,'(a)') (str(k),k=20,21)
      write(10,'(a,6f8.1)') '.T. ',
     &vza(1),vza(nvza),15.0,phi(1),phi(nphi),30.0
      write(10,'(a)') (str(k),k=23,nstr)
      close(10)
      
      call system('flatpsv.x')

C----------open output file and read in TOA radiance
      tsza=float(int(sza*10.0))/10.	! truncated sza for arizona output
      if(sza.lt. 0.95) then
	 write(ftop,'(''flatpsv.s00'',f2.1,''.bot'')') sza
      else if(sza.lt.9.95) then
         write(ftop,'(''flatpsv.s0'',f3.1,''.bot'')') sza
      else
         write(ftop,'(''flatpsv.s'',f4.1,''.bot'')') sza
      endif

      open(10,file=ftop,status='old',err=900)      
c     write(6,*) 'initial flux: SZA name is correct   ',ftop
c     write(6,*) 'continue calculating initial DD ratio:'

c      write(6,*) ftop
      do k=1,10
         read(10,'(a)') buff
      enddo
      read(10,'(4e13.4)') dum,e_dir_aer,e_dif_aer,dum       
      do k=12,13
         read(10,'(a)') buff
      enddo      
      do j=1,nphi 
      do k=1,nvza     
      read(10,'(f6.2,f6.1,f6.2,f7.2,2e11.4)') theta,theta,theta,theta,
     &rad_aer(k,j),t
      enddo
      enddo
      close(10) 
c--------------------------------------------------

c     WRITE(*,*) 'RAD_AER ',RAD_AER(1,1)
c     write(*,*) 'E_DIR_AER,E_DIF_AER,  DD  AT ',wave         
c     write(*,*)  e_dir_aer,e_dif_aer, e_dif_aer/e_dir_aer/cos_sza
c     write(*,*)  'calc with initial refr index completed at ',wave




c---	Check DD ratio with MFRSR measured ratio:
cnk03 calc_old(i)=e_dif_aer/e_dir_aer
c     calc_old(i)=cos_sza*e_dif_aer/e_dir_aer	! e_dir_aer - driect_horizontal

      if (dd_fit) then
      calc_old(i)=cos_sza*e_dif_aer/e_dir_aer	! e_dir_aer - direct_horizontal
      meas_par =mfrsr_dd(i)
      write(6,*) '			DD_FIT.eq.TRUE '
      else if (dt_fit) then
      calc_old(i)=e_dif_aer/(e_dir_aer + e_dif_aer)
      meas_par =mfrsr_dt(i)
      write(6,*) '			DT_FIT.eq.TRUE '
      else if (t_fit) then
      calc_old(i)=e_dir_aer + e_dif_aer
      meas_par =mfrsr_tot(i)
      write(6,*) '			T_FIT.eq.TRUE '
      else if (d_fit) then
      calc_old(i)= e_dif_aer
      meas_par =mfrsr_dif(i)
      write(6,*) '			D_FIT.eq.TRUE '
      else
      write(6,*) 'Specify fit parameter: '
      write(6,*) 'DD_FIT or DT_FIT or T_FIT or D_FIT should be true'
      endif


c     write(*,*) 'calc_old(i),meas_par, calc_par /meas_par   '
c     write(*,*) calc_old(i),meas_par, calc_old(i)/meas_par   

      n_iter(i)=0

c     write(*,*)                                                
c     write(*,*)  'calc completed with refr index k= ',iIOR1   
c     write(*,*) 'n_iter,n, 	k, 	calc_par, meas_par: ' 
c      write(*,*) n_iter(i),rIOR1,iIOR1,calc_old(i),meas_par   
c     write(*,*)
      write(28,*) 'dd_fit,dt_fit,t_fit,d_fit:',dd_fit,dt_fit,t_fit,d_fit               
      write(28,*) 'sza=',sza                               
      write(28,*) 'n_iter,     n,   k ,  calc_par,  meas_par,  SSA 
     &ASYM' 
      write(28,82) n_iter(i),rIOR1,iIOR1,calc_old(i), meas_par,
     &            om(i),asym(i)
 82   format(i6,6(2x,f7.5))

c---	Compare AERONET parameter with MFRSR measured parameter:

        err_par(i)=(calc_old(i) - meas_par   ) 

        if (abs(err_par(i)) .le. fit_tolerance) then 
        go to 301		! exit k iterations successfully
	endif

c----   SKIP TO RAYLEIGH CALCULATION IF NO FITTING
       if (max_k_iter .le. 0 ) then
       calc_new(i)=calc_old(i)
	go to 301
 	endif


c---		START FITTING!
cc---   Double imaginary k at this wavelength and repeat calculation
c      iIOR1_new=2.0*iIOR1
c      iIOR2_new=2.0*iIOR2

c---   Multiply AERONET imaginary k for the first iteration 
       iIOR1_new=firstmult_k*iIOR1
       iIOR2_new=firstmult_k*iIOR2

 300  continue		! continue k iterations, starting with n_iter(i)=1                      
      n_iter(i)=n_iter(i)+1
      if (n_iter(i) .ge. max_k_iter+1) go to 901	! quit k iterations unsuccesfully  after n_iter=max_k_iter

C-----------modify the flat.in    
      open(10,file=fname,status='old')
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo
      close(10)

      open(10,file=fname,status='old')
      write(10,'(a)') (str(k),k=1,2)
      write(10,'(4e11.4)') rIOR1,iIOR1_new,rIOR2,iIOR2_new  ! new values of refr index
      write(10,'(a)') (str(k),k=4,nstr)	
      close(10)
      
c----  RT calculations with new imaginary refr index
      call system('rm -f aerosol.phs')
      call system('rm -f miecalc.dat')
      call system('flatpsv.x')		! -> new miecalc.dat and aerosol.phs files for wave

C----------open output file and read in BOA radiance
      open(10,file=ftop,status='old',err=900)      
      do k=1,10
         read(10,'(a)') buff
      enddo
      read(10,'(4e13.4)') dum,e_dir_aer,e_dif_aer,dum       
      do k=12,13
         read(10,'(a)') buff
      enddo      
      do j=1,nphi 
      do k=1,nvza     
      read(10,'(f6.2,f6.1,f6.2,f7.2,2e11.4)') theta,theta,theta,theta,
     &rad_aer(k,j),t
      enddo
      enddo
      close(10) 
c---- Get single scattering albedo for k_fit

      open(10,file=faerosol,status='old')
      if(iaertype.eq.2) nn=8
      if(iaertype.eq.4) nn=12
      read(10,'(a)') (buf(k),k=1,nn)
      read(10,'(a,e12.4)') buff,qext
      read(10,'(a)') buf(1)
      read(10,'(a,f7.4,a,f7.4)') buff1,om(i),buff2,asym(i)
c     write(28,*) wave,qext,om(i),asym(i)
      close(10)

c--------------------------------------------------
c     WRITE(*,*) 'RAD_AER ',RAD_AER(1,1)
c     write(*,*) 'E_DIR_AER,E_DIF_AER,  DD  AT ',wave         
c     write(*,*)  e_dir_aer,e_dif_aer, e_dif_aer/e_dir_aer

c nk03  calc_new(i)=e_dif_aer/e_dir_aer
c       calc_new(i)=cos_sza*e_dif_aer/e_dir_aer
      if (dd_fit) then
      calc_new(i)=cos_sza*e_dif_aer/e_dir_aer     ! e_dir_aer - dorect_horizontal
c     write(6,*) '                      DD_FIT.eq.TRUE '
      else if (dt_fit) then
      calc_new(i)=e_dif_aer/(e_dir_aer + e_dif_aer)
c     write(6,*) '                      DT_FIT.eq.TRUE '
      else if (t_fit) then
      calc_new(i)=e_dir_aer + e_dif_aer
c     write(6,*) '                      T_FIT.eq.TRUE '
      else if (d_fit) then
      calc_new(i)= e_dif_aer
c     write(6,*) '                      D_FIT.eq.TRUE '
      else
      write(6,*) 'Specify fit parameter: '
      write(6,*) 'DD_FIT or DT_FIT or T_FIT or D_FIT should be true'
      endif


C--------------         Jacobian estimation:
c---    			! linear interpolation
        arr_dd(1)=calc_old(i)
        arr_dd(2)=calc_new(i)
        arr_k (1)=iIOR1 
        arr_k (2)=iIOR1_new 
        arr_w (1)=om(i) 
        arr_w (2)=om(i)
        dd_k=( (arr_dd(2)-arr_dd(1))/arr_dd(1)) /
     *       ( (arr_k(2) -arr_k(1)) /arr_k (1))	
c       dd_w=( (arr_dd(2)-arr_dd(1))/arr_dd(1)) /
c    *       ( (arr_w(2) -arr_w(1)) /arr_w (1))

        if (log_fit) then	!  logarithmic interpolation
        arr_dd(1)=alog(calc_old(i))
        arr_dd(2)=alog(calc_new(i))
        arr_k (1)=alog(iIOR1 )
        arr_k (2)=alog(iIOR1_new )
        dd_k=( arr_dd(2)-arr_dd(1)) /
     *       ( arr_k(2) -arr_k (1)) 
c       dd_w=( arr_dd(2)-arr_dd(1)) /
c    *       ( arr_w(2) -arr_w (1)) 
        endif


c     write(28 ,*)  'calc completed with refr index k= ',iIOR1_new
      write(28,82) n_iter(i),rIOR1,iIOR1_new,calc_new(i),meas_par,
     &om(i),asym(i)   !,dd_k,dd_w

c---	Compare DD ratio with MFRSR measured ratio:

c       err_par(i)=(calc_new(i) - meas_par   ) 
        err_par(i)=(calc_new(i) - meas_par   )/meas_par  ! nk 20100921  use relative fit tolerance

        if (abs(err_par(i)) .le. fit_tolerance) then 
        k_real(i)=iIOR1_new   ! change output value of k
        go to 301		! exit k iterations successfully
	endif

c---     check if iIOR1_new=0 AND calc_new(i) lt meas_par 	! => k fit is impossible
	if ((err_par(i) .lt. 0.0) .and. (iIOR1_new .le. 0.00) ) then
	write(28,*) '!! k fit is impossible !!'
	write(28,*) 'n_iter(i),iIOR1_new,calc_new(i),meas_par   '
	write(28,*)  n_iter(i),iIOR1_new,calc_new(i),meas_par    
	write(28,*) 'Exit k iterations unsuccessfully'
	n_iter(i)=33
        om(i)= -9.99	! SSA fill value =-9.99
        asym(i)= -9.99	! Asymmetry parameter fill value =-9.99
        go to 901	! quit fitting without calculating Rayleigh irradiance
	endif



c---    interpolate new k_fit value
        if (abs(calc_new(i) - calc_old(i)) .lt. 0.000001 ) then
        write(*,*) 'INTERPOLATION: abscissa values equal:'
        write(*,*) calc_old(i), calc_new(i)
        go to 301
        else if (log_fit)  then
        k_fit(i)=exp(interpol(2,arr_dd,arr_k,alog(meas_par),ierr))  ! logarithmic interpolation
        else
        k_fit(i)=interpol(2,arr_dd,arr_k,meas_par,ierr)   ! linear interpolation
        endif

c---  nk 2012/07/03:  constrain k_fit < max_k_real
      if (k_fit(i) .ge. max_k_real ) then
	write(28,*) '!! k is too large  !!'
	write(28,*) 'k_fit(i)= ',k_fit(i) 
	write(28,*) 'max_k_real(i)= ',max_k_real
	write(28,*) 'Exit k iterations unsuccessfully'
      k_real(i)=-9.99999   ! assign fill value          
        om(i)= -9.99
        asym(i)=-9.999999
      go to 901	! quit k iterations unsuccesfully because k_fit > max_k_real
      endif

c---	force k_fit>=0
	if (k_fit(i) .le. 0.0) then
	k_fit(i)=0.0
	endif

      k_real(i)=k_fit(i)   ! change output value of k

c----   Calculation with new k_fit value:
        iIOR1_new=k_fit(i)
        iIOR2_new=k_fit(i)

 
        go to 300	! go to one more k iteration


  301 continue		! exit k iterations susseccfully and proceed to compute SSA and Rayleigh dd ratio


c---- Get single scattering albedo for k_fit

      open(10,file=faerosol,status='old')
      if(iaertype.eq.2) nn=8
      if(iaertype.eq.4) nn=12
      read(10,'(a)') (buf(k),k=1,nn)
      read(10,'(a,e12.4)') buff,qext
      read(10,'(a)') buf(1)
      read(10,'(a,f7.4,a,f7.4)') buff1,om(i),buff2,asym(i)
c     write(28,*) wave,qext,om(i),asym(i)
      close(10)



C-----Gauss-Sedel computations of Pure Rayleigh radiances                          

      open(10,file=fname,status='unknown')
      
      do k=1,nstr
         read(10,'(a)') str(k)
      enddo
      close(10)
      
C---------modify an input file
      
      open(10,file=fname,status='unknown')      
      write(10,'(5e11.4,f10.7)') taumd,taur,tauc,totoz,totwat,wave
      write(10,'(a)') (str(k),k=2,nstr)
      close(10)
      
      call system('flatpsv.x')            

      if(sza.lt. 0.95) then
	 write(ftop,'(''flatpsv.s00'',f2.1,''.bot'')') sza
      else if(sza.lt.9.95) then
         write(ftop,'(''flatpsv.s0'',f3.1,''.bot'')') sza
      else
         write(ftop,'(''flatpsv.s'',f4.1,''.bot'')') sza
      endif
           
      open(10,file=ftop,status='old',err=900)      
      do k=1,10
         read(10,'(a)') buff
      enddo      
      read(10,'(4e13.4)') dum,e_dir_ray(i),e_dif_ray(i),dum       
      do k=12,13
         read(10,'(a)') buff
      enddo      
      do j=1,nphi 
      do k=1,nvza     
      read(10,'(f6.2,f6.1,f6.2,f7.2,2e11.4)') theta,theta,theta,theta,
     &rad(i,k,j),t
      enddo
      enddo
      close(10)
c     WRITE(*,*) 'RAD_NOAER',RAD(i,1,1)
c     write(*,*) 'e_dir_ray,e_dif_ray at ',wave         
c     write(*,*)  e_dir_ray(i),e_dif_ray(i)                   
      e_dir_ratio(i)=e_dir_aer/e_dir_ray(i)
      e_dif_ratio(i)=e_dif_aer/e_dif_ray(i)


      WRITE(*,*) 'RT CALCULATIONS COMPLETED SUCCESSFULLY'


      do j=1,nphi 
      do k=1,nvza     
      ratio(i,k,j)=rad_aer(k,j)/rad(i,k,j)
      enddo
      enddo

 100	continue	! skip radiance calculations

c     call system('rm -f aerosol.phs')

      enddo		! end wavelength loop : lambda(i), i=1,icount
      
C----------interpolation of aerosol optical thickness & single scattering
C----------albedo on fixed wavelengths

      
      do k=1,icount
         lambuf(k)=lambda(k)/1.0E4
      enddo
      do k=1,12
        tauaer(k)=0.0
        omega(k)=0.0
        if(lam(k).ge.lambuf(1).and.lam(k).le.lambuf(icount)) then
        tauaer(k)=interpol(icount,lambuf,taum,lam(k),ierr)
        omega(k)=interpol(icount,lambuf,om,lam(k),ierr)
c	write(28,*) LAM(K),TAUAER(K),OMEGA(K)
        endif    
      enddo

      return	!!!!!!  This is return from normal calculation of k_real

 900  continue
	write(6,*) 'can not open file: ',ftop
	write(28,*) 'can not open file: ',ftop                       
 901  continue
	write(*,*) '!! k fit is impossible !!'
	write(*,*) 'n_iter(i),iIOR1_new,calc_new(i),meas_par'
	write(*,*)  n_iter(i),iIOR1_new,calc_new(i),meas_par    
	write(*,*) 'Exit k iterations unsuccessfully'

	do i=1,icount
        om(i)= -9.99
	asym(i)= -9.99
        k_real(i)=-9.999999
	enddo


	write(28,*) '!!! k fit is impossible!!! '
	return
      end
